/*
 *--------------------------------------------------------------------
 *
 * graphDD
 *
 * Knihovna funkci pro praci s grafikou DirectDraw.
 *
 * verze   1.0.1 - 14.4.2000
 * autori  Vojvo & Gunny
 *
 *--------------------------------------------------------------------
 * zmeny
 *
 * 1.0.1  - snaha o prvni plnou verzi
 * 0.99.4 - snaha upravit init, aby sel vyvolat vickrat za behu programu
 * 0.99.3 - v headru makra pro sektorove clearovani
 * 0.11.2 - putcelBlt pro blitovani pomoci Blt
 * 0.11.1 - podle dostupne videopameti se urcuje prefferedCAPS
 * 0.10.1 - puttint50, puttint50cached, puttransparentcached
 * 0.9.4 - clear ma v parametrech barvu
 * 0.9.3 / putdark50 a putdark25
 * 0.9.2 - opravena funkce puttinted
 *       - pridana funkce puttransparent
 * 0.8.1 - pridan rFullScreen (puvodne v Game.cpp)
 *       - odstraneny nepouzivane lokalni promenne         
 * 0.0.3 - predelavka getcel a puttint
 * 0.0.2 - pridany konstanty COLOR_KEY5x5
 * 0.0.1 - prvni verze
 *--------------------------------------------------------------------
 */

#include <windows.h>
#include <stdio.h>
#include "types.h"
#include "surfaceManager.h"
#include "exceptions.h"
#include "graphDD.h"
#include "utils.h"

LPDIRECTDRAW lpDD = NULL;
LPDIRECTDRAW4 lpDD4 = NULL;
DDCAPS ddc1;

Surface frontbuffer;
Surface backbuffer;
Surface background;

bool usedmap1[10][12];
bool usedmap2[10][12];
bool (*usedmap)[10][12] = &usedmap1;
int clusmapx,clusmapy;

WORD colorKey;
WORD rMask;
WORD gMask;
WORD bMask;
BYTE gBits;

DDPIXELFORMAT ddpf;
HWND ddHWND = NULL;

bool ddRestored = false;
HRESULT ddResult = 0;
long myResult = 0;

RECT rFullScreen = { 0, 0, 640, 480 };

DWORD prefferedCAPS = DDSCAPS_OFFSCREENPLAIN;

//-------------------------------------------------------------------------
// init_DD()
//-------------------------------------------------------------------------
void initDD(HWND hWnd, DWORD width, DWORD height, DWORD bitdepth, 
            DWORD refreshrate)
{
   DDSURFACEDESC2 ddsd2;
   DDSCAPS2 ddsc2;
   DDCAPS ddc2;

   ddResult = DirectDrawCreate(NULL,&lpDD,NULL);
   if (ddResult != DD_OK)
      throw new DDException("initDD: DirectDrawCreate() failed!");
      
   ddResult = lpDD->QueryInterface(IID_IDirectDraw4,(LPVOID *)&lpDD4);
   if (ddResult != DD_OK)
      throw new DDException("initDD: QueryInterface() failed!",ddResult);
   
   ddResult = lpDD4->TestCooperativeLevel();
   if (ddResult != DDERR_EXCLUSIVEMODEALREADYSET)
   {
      ddResult = lpDD4->SetCooperativeLevel(hWnd,
         DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE);
      if (ddResult != DD_OK)
         throw new DDException("initDD: SetCooperativeLevel() failed!");
   }

   ddHWND = hWnd;

   ddResult = lpDD4->SetDisplayMode(width,height,bitdepth,refreshrate,0);
   if (ddResult != DD_OK)
   {
      // jestli nebyla default refresh rate, zkusim to jeste s ni
      if (refreshrate != 0)
      {
         ddResult = lpDD4->SetDisplayMode(width,height,bitdepth,85,0);
         if (ddResult != DD_OK)
            throw new DDException("initDD: SetDisplayMode() failed!");
      }
      else
         throw new DDException("initDD: SetDisplayMode() failed!");
   }
   
   ZeroMemory(&ddsd2, sizeof(ddsd2));
   ddsd2.dwSize = sizeof(ddsd2);
   ddsd2.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
   ddsd2.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
   ddsd2.dwBackBufferCount = 1;
   
   ddResult = lpDD4->CreateSurface(&ddsd2,&frontbuffer,NULL);
   
   if (ddResult != DD_OK)
      throw new DDException("initDD: CreateSurface() failed!");
   
   ddsc2.dwCaps = DDSCAPS_BACKBUFFER;
   ddResult = frontbuffer->GetAttachedSurface(&ddsc2,&backbuffer);
   if (ddResult != DD_OK)
      throw new DDException("initDD: GetAtachedSurface() failed!");
   
   ZeroMemory(&ddpf,sizeof(ddpf));
   ddpf.dwSize = sizeof(ddpf);
   ddResult = frontbuffer->GetPixelFormat(&ddpf);
   if (ddResult != DD_OK)
      throw new DDException("initDD: GetPixelFormat() failed!");
   
   rMask = (WORD) ddpf.dwRBitMask;
   gMask = (WORD) ddpf.dwGBitMask;
   bMask = (WORD) ddpf.dwBBitMask;
   if (gMask == 0x7E0)
   {
      gBits = 6;
      colorKey = COLOR_KEY565;
   }
   else
   {
      gBits = 5;
      colorKey = COLOR_KEY555;
   }

   lpDD4->GetCaps(&ddc1,&ddc2);
   if (ddc1.dwVidMemTotal > 3000000)
   {
      DEBUG_MESSAGE("preffered:VIDEO\n");
      prefferedCAPS = DDSCAPS_OFFSCREENPLAIN;
   }
   else
   {
      DEBUG_MESSAGE("preffered:SYSTEM\n");
      prefferedCAPS = DDSCAPS_OFFSCREENPLAIN+DDSCAPS_SYSTEMMEMORY;
   }

   surfaceManager = new SurfaceManager();
   surfaceManager->add(frontbuffer,"*frontbuffer",NULL,NULL);
}

/*==========================================================================*
   putcel()
 *==========================================================================*/
void putcel(int x, int y, Surface dest, RECT &r, Surface src, DWORD dwTrans)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(2);
#endif

   while(TRUE)
   {
      ddResult = dest->BltFast(x,y,src,&r,dwTrans);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         throw new IsLostException("Surface Lost");
      }
      if (ddResult == DDERR_INVALIDPARAMS)
      {
         sprintf(exc_string,"putcel: BltFast(%d,%d,(%d,%d,%d,%d),%p,%lu) invalid params!",
            x,y,r.left,r.top,r.right,r.bottom,src,dwTrans);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_SURFACEBUSY)
         throw new DDException("putcel: BltFast() surface busy!",ddResult);
      if (ddResult == DDERR_EXCEPTION)
         throw new DDException("putcel: BltFast() exception!",ddResult);
      if (ddResult == DDERR_GENERIC)
         throw new DDException("putcel: BltFast() generic!",ddResult);
      if (ddResult == DDERR_INVALIDOBJECT)
      {
         sprintf(exc_string,"putcel: BltFast(%d,%d,%p,(%d,%d,%d,%d),%p,%lu) invalid object!",
            x,y,dest,r.left,r.top,r.right,r.bottom,src,dwTrans);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_INVALIDRECT)
      {
         sprintf(exc_string,"putcel: BltFast(%d,%d,(%d,%d,%d,%d),%p,%lu) invalid rect!",
            x,y,r.left,r.top,r.right,r.bottom,src,dwTrans);
         sprintf(exc_string,"putcel: BltFast %d,%d<-%d,%d,%d,%d invalid rect!",
            x,y,r.left,r.top,r.right,r.bottom);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_SURFACELOST)
         throw new DDException("putcel: BltFast() surface lost!",ddResult);
      if (ddResult == DDERR_UNSUPPORTED)
         throw new DDException("putcel: BltFast() unsupported!",ddResult);
      if (ddResult == DDERR_NOBLTHW)
         throw new DDException("putcel: BltFast() no blitter HW!",ddResult);
      if (ddResult != DDERR_WASSTILLDRAWING)
         throw new DDException("putcel: BltFast() failed!",ddResult);
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(2);
#endif
}

//-------------------------------------------------------------------------
// putcel pres DirectDrawSurface->Blt s uvedenim flagu a efektu
//-------------------------------------------------------------------------
void putcelBlt(int x, int y, Surface dest, RECT &r, Surface src, 
               DWORD dwFlags, DDBLTFX *lpBltFX)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(14);
#endif
   RECT rdest;
   rdest.left = x;
   rdest.top = y;
   rdest.right = x+r.right-r.left;
   rdest.bottom = y+r.bottom-r.top;
   while(TRUE)
   {
      ddResult = dest->Blt(&rdest,src,&r,dwFlags,lpBltFX);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         throw new IsLostException("Surface Lost");
      }
      if (ddResult == DDERR_INVALIDPARAMS)
      {
         sprintf(exc_string,"putcelBlt: Blt(%d,%d,(%d,%d,%d,%d),%p,%lu) invalid params!",
            x,y,r.left,r.top,r.right,r.bottom,src,dwFlags);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_SURFACEBUSY)
         throw new DDException("putcel: BltFast() surface busy!",ddResult);
      if (ddResult == DDERR_EXCEPTION)
         throw new DDException("putcel: BltFast() exception!",ddResult);
      if (ddResult == DDERR_GENERIC)
         throw new DDException("putcel: BltFast() generic!",ddResult);
      if (ddResult == DDERR_INVALIDOBJECT)
      {
         sprintf(exc_string,"putcelBlt: Blt(%d,%d,%p,(%d,%d,%d,%d),%p,%lu) invalid object!",
            x,y,dest,r.left,r.top,r.right,r.bottom,src,dwFlags);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_INVALIDRECT)
      {
         sprintf(exc_string,"putcelBlt: Blt(%d,%d,(%d,%d,%d,%d),%p,%lu) invalid rect!",
            x,y,r.left,r.top,r.right,r.bottom,src,dwFlags);
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_SURFACELOST)
         throw new DDException("putcel: BltFast() surface lost!",ddResult);
      if (ddResult == DDERR_UNSUPPORTED)
         throw new DDException("putcel: BltFast() unsupported!",ddResult);
      if (ddResult == DDERR_NOBLTHW)
         throw new DDException("putcel: BltFast() no blitter HW!",ddResult);
      if (ddResult != DDERR_WASSTILLDRAWING)
         throw new DDException("putcel: BltFast() failed!",ddResult);
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(2);
#endif
}


/*==========================================================================*
   getcel()
 *==========================================================================*/
Surface getcel(Surface src, RECT &r)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(3);
#endif

   Surface lpDDS;
   DDSURFACEDESC2 ddsd2;

   ZeroMemory(&ddsd2, sizeof(ddsd2));
   ddsd2.dwSize = sizeof(ddsd2);
   ddsd2.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
   ddsd2.dwWidth = r.right - r.left;
   ddsd2.dwHeight = r.bottom - r.top;
   ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
   
   ddResult = lpDD4->CreateSurface(&ddsd2,&lpDDS,NULL);
   if (ddResult != DD_OK)
   {
      sprintf(exc_string,"getcel: Create surface(%lux%lu) failed!",
         ddsd2.dwWidth,ddsd2.dwHeight);
      throw new BubException(exc_string);

      if (ddResult == DDERR_INVALIDPARAMS)
         throw new DDException("getcel: CreateSurface() invalid params!",ddResult);
      if (ddResult == DDERR_INVALIDOBJECT)
         throw new DDException("getcel: CreateSurface() invalid object!",ddResult);
      throw new DDException("getcel: CreateSurface() failed!",ddResult);
   }

   while(TRUE)
   {
      ddResult = lpDDS->BltFast(0,0,src,&r,DDBLTFAST_NOCOLORKEY);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         lpDDS->Release();
         throw new IsLostException("Surface Lost");
      }
      if (ddResult == DDERR_INVALIDPARAMS)
         throw new DDException("getcel: BltFast() invalid params!",ddResult);
      if (ddResult == DDERR_SURFACEBUSY)
         throw new DDException("getcel: BltFast() surface busy!",ddResult);
      if (ddResult == DDERR_EXCEPTION)
         throw new DDException("getcel: BltFast() exception!",ddResult);
      if (ddResult == DDERR_GENERIC)
         throw new DDException("getcel: BltFast() generic!",ddResult);
      if (ddResult == DDERR_INVALIDOBJECT)
         throw new DDException("getcel: BltFast() invalid object!",ddResult);
      if (ddResult == DDERR_INVALIDRECT)
         throw new DDException("getcel: BltFast() invalid rect!",ddResult);
      if (ddResult == DDERR_SURFACELOST)
         throw new DDException("getcel: BltFast() surface lost!",ddResult);
      if (ddResult == DDERR_UNSUPPORTED)
         throw new DDException("getcel: BltFast() unsupported!",ddResult);
      if (ddResult == DDERR_NOBLTHW)
         throw new DDException("getcel: BltFast() no blitter HW!",ddResult);
      if (ddResult != DDERR_WASSTILLDRAWING)
         throw new DDException("getcel: BltFast() failed",ddResult);
    }

    return lpDDS;
#ifdef PROFILER_DEBUG
   profiler->stopTimer(3);
#endif

}

/*==========================================================================*
   puttint()
 *==========================================================================*/
void puttint(int x, int y, Surface dest, RECT &r, Surface src, TintTable tnt)
{
   RECT rdest;
   Surface destcel, srccel;
   DDSURFACEDESC2 ddsd2;
   WORD *lpSource;

#ifdef PROFILER_DEBUG
   profiler->startTimer(4);
#endif


   rdest.left = x;
   rdest.top = y;
   rdest.right = x + r.right - r.left;
   rdest.bottom = y + r.bottom - r.top;

   srccel = getcel(src, r);
   destcel = getcel(dest, rdest);

   lock(srccel,NULL,&ddsd2);

   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   long srcpitch = ddsd2.lPitch / 2;
   DWORD srcoffset,destoffset;
   lpSource = (WORD *)ddsd2.lpSurface;

   lock(destcel,NULL,&ddsd2);

   long destpitch = ddsd2.lPitch / 2;
   DWORD destwidth = ddsd2.dwWidth;
   DWORD destheight = ddsd2.dwHeight;
   WORD *destdata = (WORD *) ddsd2.lpSurface;
   DWORD i,j;

   for(i=0; i<destheight; i++)
   {
      srcoffset = i*srcpitch;
      destoffset = i*destpitch;
      for(j=0; j<destwidth; j++, destoffset++, srcoffset++)
      {
         if (lpSource[srcoffset] != colorKey)
            destdata[destoffset] = tnt[destdata[destoffset]];
      }
   }

   srccel->Unlock(NULL);
   srccel->Release();
   destcel->Unlock(NULL);
   
   rdest.left = rdest.top = 0;
   rdest.right = destwidth;
   rdest.bottom = destheight;
   try
   {
      putcel(x,y,dest,rdest,destcel,DDBLTFAST_NOCOLORKEY);
      destcel->Release();
   }
   catch (IsLostException *exc)
   {
      destcel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(4);
#endif

}

/*==========================================================================*
   puttintm() - puttint s pouzitim masky
 *==========================================================================*/
void puttintm(int x, int y, Surface dest, RECT &r, TintMask mask, TintTable tnt)
{
   RECT rdest;
   Surface destcel;
   DDSURFACEDESC2 ddsd2;

#ifdef PROFILER_DEBUG
   profiler->startTimer(5);
#endif

   rdest.left = x;
   rdest.top = y;
   rdest.right = x + r.right - r.left;
   rdest.bottom = y + r.bottom - r.top;

   DWORD srcPitch = *((DWORD *)mask);
   DWORD srcstart = r.top * srcPitch + r.left + 8;
   DWORD srcoffset;

   destcel = getcel(dest, rdest);
   lock(destcel,NULL,&ddsd2);
   
   long destpitch = ddsd2.lPitch / 2;
   DWORD destwidth = ddsd2.dwWidth;
   DWORD destheight = ddsd2.dwHeight;
   WORD *destdata = (WORD *) ddsd2.lpSurface;
   DWORD i,j;
   DWORD destoffset;

   for(i=0; i<destheight; i++)
   {
      srcoffset = srcstart+i*srcPitch;
      destoffset = i*destpitch;
      for(j=0; j<destwidth; j++, destoffset++, srcoffset++)
      {
         if (mask[srcoffset] != 0)
            destdata[destoffset] = tnt[destdata[destoffset]];
      }
   }

   destcel->Unlock(NULL);
   rdest.left = rdest.top = 0;
   rdest.right = destwidth;
   rdest.bottom = destheight;
   try
   {
      putcel(x,y,dest,rdest,destcel,DDBLTFAST_NOCOLORKEY);
      destcel->Release();
   }
   catch (IsLostException *exc)
   {
      destcel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(5);
#endif

}


/*==========================================================================*
   puttinted()
 *==========================================================================*/
void puttinted(int x,int y,Surface dest,RECT &r,Surface src,TintTable tnt)
{
   Surface srccel;
   DDSURFACEDESC2 ddsd2;
   RECT rsrc;
   
   srccel = getcel(src,r);

   lock(srccel,NULL,&ddsd2);
   
   long srcpitch = ddsd2.lPitch / 2;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   DWORD srcoffset;
   WORD pixel;
   
   for(DWORD i=0; i<srcheight; i++)
   {
      srcoffset = i*srcpitch;
      for(DWORD j=0; j<srcwidth; j++, srcoffset++)
      {
	 pixel = srcdata[srcoffset];
         if (pixel != colorKey)
            srcdata[srcoffset] = tnt[pixel];
      }
   }
   
   srccel->Unlock(NULL);
   
   rsrc.left = 0;
   rsrc.top = 0;
   rsrc.right = srcwidth;
   rsrc.bottom = srcheight;
   try
   {
      setColorKey(srccel,colorKey);
      putcel(x,y,dest,rsrc,srccel,DDBLTFAST_SRCCOLORKEY);
      srccel->Release();
   }
   catch (Exception *exc)
   {
      srccel->Release();
      throw exc;
   }
}   

//-------------------------------------------------------------------------
// ztintuje zdroj 50% dane barvy a prskne ho na cil s vyzitim SRCCOLORKEY
//-------------------------------------------------------------------------
void puttinted50(DWORD x,DWORD y,Surface dest,RECT &r,Surface src,
                 int color)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(10);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   WORD color50 = (color >> 1) & maska;
   RECT rsrc;
   Surface srccel;
   DDSURFACEDESC2 ddsd2;

   DWORD i,j;
   DWORD srcoffset;
   WORD pixel;

   srccel = getcel(src, r);
   lock(srccel,NULL,&ddsd2);
   
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;

   for(i=0; i<srcheight; i++)
   {
      srcoffset = i*srcPitch;
      for(j=0; j<srcwidth; j++, srcoffset++)
      {
         pixel = srcdata[srcoffset];
         if (pixel != colorKey)
            srcdata[srcoffset] = ((pixel >> 1) & maska) + color50;
      }
   }
   srccel->Unlock(NULL);

   rsrc.left = rsrc.top = 0;
   rsrc.right = srcwidth;
   rsrc.bottom = srcheight;
   try
   {
      setColorKey(srccel,colorKey);
      putcel(x,y,dest,rsrc,srccel,DDBLTFAST_SRCCOLORKEY);
      srccel->Release();
   }
   catch (Exception *exc)
   {
      srccel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(10);
#endif
}

//-------------------------------------------------------------------------
// ztintuje cely surface redalertem
//-------------------------------------------------------------------------
void tint_redalert(Surface src,RECT &r)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(18);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   maska = (maska & (gMask | bMask)); // ted bych tam mel mit G a B bez hornich bitu
   DDSURFACEDESC2 ddsd2;
   DWORD srcoffset;
   int i,j;
   WORD pixel;

   lock(src,NULL,&ddsd2);
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;

   for(i=r.top; i<r.bottom; i++)
   {
      srcoffset = i*srcPitch+r.left;
      for(j=r.left; j<r.right; j++, srcoffset++)
      {
         pixel = srcdata[srcoffset];
         srcdata[srcoffset] = ((pixel & rMask)|((pixel>>1)&maska));
      }
   }
   src->Unlock(NULL);
#ifdef PROFILER_DEBUG
   profiler->stopTimer(18);
#endif
}

//-------------------------------------------------------------------------
// ztintuje cely surface 50% dane barvy
//-------------------------------------------------------------------------
void tint50(Surface src,WORD color)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(19);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   WORD color50 = (color >> 1) & maska;
   DDSURFACEDESC2 ddsd2;
   DWORD srcoffset;
   DWORD i,j;
   WORD pixel;

   lock(src,NULL,&ddsd2);
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;

   for(i=0; i<srcheight; i++)
   {
      srcoffset = i*srcPitch;
      for(j=0; j<srcwidth; j++, srcoffset++)
      {
         pixel = srcdata[srcoffset];
         srcdata[srcoffset] = ((pixel >> 1) & maska) + color50;
      }
   }
   src->Unlock(NULL);
#ifdef PROFILER_DEBUG
   profiler->stopTimer(19);
#endif
}

//-------------------------------------------------------------------------
// ztintuje zdroj pod maskou 50% dane barvy
//-------------------------------------------------------------------------
void puttint50(DWORD srcx,DWORD srcy,Surface src,
          RECT &r, TintMask mask, WORD color)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(8);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   WORD color50 = (color >> 1) & maska;
   RECT rsrc;
   Surface srccel;
   DDSURFACEDESC2 ddsd2;

   rsrc.left = srcx;
   rsrc.top = srcy;
   rsrc.right = srcx + r.right - r.left;
   rsrc.bottom = srcy + r.bottom - r.top;

   DWORD maskPitch = *((DWORD *)mask);
   DWORD maskstart = r.top * maskPitch + r.left + 8;
   DWORD maskoffset;
   DWORD i,j;
   DWORD srcoffset;
   WORD pixel;

   srccel = getcel(src, rsrc);
   lock(srccel,NULL,&ddsd2);
   
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;

   for(i=0; i<srcheight; i++)
   {
      maskoffset = maskstart+i*maskPitch;
      srcoffset = i*srcPitch;
      for(j=0; j<srcwidth; j++, srcoffset++, maskoffset++)
      {
         if (mask[maskoffset] != 0)
         {
            pixel = srcdata[srcoffset];
            srcdata[srcoffset] = ((pixel >> 1) & maska) + color50;
         }
      }
   }
   srccel->Unlock(NULL);

   rsrc.left = rsrc.top = 0;
   rsrc.right = srcwidth;
   rsrc.bottom = srcheight;
   try
   {
      putcel(srcx,srcy,src,rsrc,srccel,DDBLTFAST_NOCOLORKEY);
      srccel->Release();
   }
   catch (IsLostException *exc)
   {
      srccel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(8);
#endif
}


//-------------------------------------------------------------------------
// ztintuje zdroj pod maskou 50% dane barvy a prskne to do ciloveho surface
//-------------------------------------------------------------------------
void puttint50cached(DWORD srcx,DWORD srcy,Surface src,
                   DWORD destx,DWORD desty,Surface dest,
                   RECT &r, TintMask mask, WORD color)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(7);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   WORD color50 = (color >> 1) & maska;
   RECT rsrc;
   Surface srccel;
   DDSURFACEDESC2 ddsd2;

   rsrc.left = srcx;
   rsrc.top = srcy;
   rsrc.right = srcx + r.right - r.left;
   rsrc.bottom = srcy + r.bottom - r.top;

   DWORD maskPitch = *((DWORD *)mask);
   DWORD maskstart = r.top * maskPitch + r.left + 8;
   DWORD maskoffset;
   DWORD i,j;
   DWORD srcoffset;
   WORD pixel;

   srccel = getcel(src, rsrc);
   lock(srccel,NULL,&ddsd2);
   
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;

   for(i=0; i<srcheight; i++)
   {
      maskoffset = maskstart+i*maskPitch;
      srcoffset = i*srcPitch;
      for(j=0; j<srcwidth; j++, srcoffset++, maskoffset++)
      {
         if (mask[maskoffset] != 0)
         {
            pixel = srcdata[srcoffset];
            srcdata[srcoffset] = ((pixel >> 1) & maska) + color50;
         }
         else
            srcdata[srcoffset] = colorKey;
      }
   }
   srccel->Unlock(NULL);

   rsrc.left = rsrc.top = 0;
   rsrc.right = srcwidth;
   rsrc.bottom = srcheight;
   try
   {
      putcel(destx,desty,dest,rsrc,srccel,DDBLTFAST_NOCOLORKEY);
      srccel->Release();
   }
   catch (IsLostException *exc)
   {
      srccel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(7);
#endif
}

//-------------------------------------------------------------------------
// puttransparentcached
//-------------------------------------------------------------------------
void puttransparentcached(int srcx, int srcy, Surface src,
                          int destx, int desty,Surface dest, 
                          RECT &r,Surface mask)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(9);
#endif
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   RECT rsrc;
   Surface srccel, maskcel;
   DDSURFACEDESC2 ddsd2;
   DWORD i,j;
   WORD pixel;

   rsrc.left = srcx;
   rsrc.top = srcy;
   rsrc.right = srcx + r.right - r.left;
   rsrc.bottom = srcy + r.bottom - r.top;

   maskcel = getcel(mask, r);
   srccel = getcel(src, rsrc);

   lock(maskcel,NULL,&ddsd2);
   long maskPitch = ddsd2.lPitch/2;
   WORD *maskdata = (WORD *)ddsd2.lpSurface;
   DWORD maskoffset;

   lock(srccel,NULL,&ddsd2);
   
   long srcPitch = ddsd2.lPitch / 2;
   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   WORD *srcdata = (WORD *) ddsd2.lpSurface;
   DWORD srcoffset;

   for(i=0; i<srcheight; i++)
   {
      maskoffset = i*maskPitch;
      srcoffset = i*srcPitch;
      for(j=0; j<srcwidth; j++, srcoffset++, maskoffset++)
      {
         if (maskdata[maskoffset] != colorKey)
         {
            pixel = srcdata[srcoffset];
            srcdata[srcoffset] = ((pixel>>1)&maska)+((maskdata[maskoffset]>>1)&maska);
         }
         else
            srcdata[srcoffset] = colorKey;
      }
   }
   maskcel->Unlock(NULL);
   srccel->Unlock(NULL);
   maskcel->Release();

   rsrc.left = rsrc.top = 0;
   rsrc.right = srcwidth;
   rsrc.bottom = srcheight;
   try
   {
      putcel(destx,desty,dest,rsrc,srccel,DDBLTFAST_NOCOLORKEY);
      srccel->Release();
   }
   catch (IsLostException *exc)
   {
      srccel->Release();
      throw exc;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(9);
#endif
}

/*==========================================================================*
   puttransparent()
 *==========================================================================*/
void puttransparent(int x, int y, Surface dest, RECT &r, Surface src)
{
   RECT rdest;
   Surface destcel, srccel;
   DDSURFACEDESC2 ddsd2;
   WORD *lpSource;

   rdest.left = x;
   rdest.top = y;
   rdest.right = x + r.right - r.left;
   rdest.bottom = y + r.bottom - r.top;

   srccel = getcel(src, r);
   destcel = getcel(dest, rdest);

   lock(srccel,NULL,&ddsd2);

   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   long srcpitch = ddsd2.lPitch / 2;
   DWORD srcoffset,destoffset;
   lpSource = (WORD *)ddsd2.lpSurface;

   lock(destcel,NULL,&ddsd2);

   long destpitch = ddsd2.lPitch / 2;
   DWORD destwidth = ddsd2.dwWidth;
   DWORD destheight = ddsd2.dwHeight;
   WORD *destdata = (WORD *) ddsd2.lpSurface;
   DWORD i,j;

   WORD maska = (gBits == 5) ? 0x3def : 0x7bef; // 0x3def: 0011 1101 1110 1111  pro 555;
	                                             //	0x7aef: 0111 1011 1110 1111  pro 565

   for(i=0; i<destheight; i++)
   {
      srcoffset = i*srcpitch;
      destoffset = i*destpitch;
      for(j=0; j<destwidth; j++, destoffset++, srcoffset++)
      {
         if (lpSource[srcoffset] != colorKey)
            destdata[destoffset] = ((destdata[destoffset] >> 1) & maska) + ((lpSource[srcoffset] >> 1) & maska);
      }
   }

   srccel->Unlock(NULL);
   srccel->Release();
   destcel->Unlock(NULL);
   
   rdest.left = rdest.top = 0;
   rdest.right = destwidth;
   rdest.bottom = destheight;
   try
   {
      putcel(x,y,dest,rdest,destcel,DDBLTFAST_NOCOLORKEY);
      destcel->Release();
   }
   catch (IsLostException *exc)
   {
      destcel->Release();
      throw exc;
   }
}

/*==========================================================================*
   putdark50()
 *==========================================================================*/
void putdark50(int x, int y, Surface dest, RECT &r, Surface src)
{
   RECT rdest;
   Surface destcel, srccel;
   DDSURFACEDESC2 ddsd2;
   WORD *lpSource;

   rdest.left = x;
   rdest.top = y;
   rdest.right = x + r.right - r.left;
   rdest.bottom = y + r.bottom - r.top;

   srccel = getcel(src, r);
   destcel = getcel(dest, rdest);

   lock(srccel,NULL,&ddsd2);

   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   long srcpitch = ddsd2.lPitch / 2;
   DWORD srcoffset,destoffset;
   lpSource = (WORD *)ddsd2.lpSurface;

   lock(destcel,NULL,&ddsd2);

   long destpitch = ddsd2.lPitch / 2;
   DWORD destwidth = ddsd2.dwWidth;
   DWORD destheight = ddsd2.dwHeight;
   WORD *destdata = (WORD *) ddsd2.lpSurface;
   DWORD i,j;

   WORD maska = (gBits == 5) ? 0x3def : 0x7bef; // 0x3def: 0011 1101 1110 1111  pro 555;
	                                             //	0x7aef: 0111 1011 1110 1111  pro 565

   for(i=0; i<destheight; i++)
   {
      srcoffset = i*srcpitch;
      destoffset = i*destpitch;
      for(j=0; j<destwidth; j++, destoffset++, srcoffset++)
      {
         if (lpSource[srcoffset] != colorKey)
            destdata[destoffset] = ((destdata[destoffset] >> 1) & maska);
      }
   }

   srccel->Unlock(NULL);
   srccel->Release();
   destcel->Unlock(NULL);
   
   rdest.left = rdest.top = 0;
   rdest.right = destwidth;
   rdest.bottom = destheight;
   try
   {
      putcel(x,y,dest,rdest,destcel,DDBLTFAST_NOCOLORKEY);
      destcel->Release();
   }
   catch (IsLostException *exc)
   {
      destcel->Release();
      throw exc;
   }
}

/*==========================================================================*
   putdark25()
 *==========================================================================*/
void putdark25(int x, int y, Surface dest, RECT &r, Surface src)
{
   RECT rdest;
   Surface destcel, srccel;
   DDSURFACEDESC2 ddsd2;
   WORD *lpSource;

   rdest.left = x;
   rdest.top = y;
   rdest.right = x + r.right - r.left;
   rdest.bottom = y + r.bottom - r.top;

   srccel = getcel(src, r);
   destcel = getcel(dest, rdest);

   lock(srccel,NULL,&ddsd2);

   DWORD srcwidth = ddsd2.dwWidth;
   DWORD srcheight = ddsd2.dwHeight;
   long srcpitch = ddsd2.lPitch / 2;
   DWORD srcoffset,destoffset;
   lpSource = (WORD *)ddsd2.lpSurface;

   lock(destcel,NULL,&ddsd2);

   long destpitch = ddsd2.lPitch / 2;
   DWORD destwidth = ddsd2.dwWidth;
   DWORD destheight = ddsd2.dwHeight;
   WORD *destdata = (WORD *) ddsd2.lpSurface;
   DWORD i,j;

   WORD maska = (gBits == 5) ? 0x3def : 0x7bef; // 0x3def: 0011 1101 1110 1111  pro 555;
	                                             //	0x7aef: 0111 1011 1110 1111  pro 565
   WORD maska2 = (maska >> 1) & maska ;

   for(i=0; i<destheight; i++)
   {
      srcoffset = i*srcpitch;
      destoffset = i*destpitch;
      for(j=0; j<destwidth; j++, destoffset++, srcoffset++)
      {
         if (lpSource[srcoffset] != colorKey)
            destdata[destoffset] = ((destdata[destoffset] >> 1) & maska) + ((destdata[destoffset] >> 2) & maska2);
      }
   }

   srccel->Unlock(NULL);
   srccel->Release();
   destcel->Unlock(NULL);
   
   rdest.left = rdest.top = 0;
   rdest.right = destwidth;
   rdest.bottom = destheight;
   try
   {
      putcel(x,y,dest,rdest,destcel,DDBLTFAST_NOCOLORKEY);
      destcel->Release();
   }
   catch (IsLostException *exc)
   {
      destcel->Release();
      throw exc;
   }
}

/*==========================================================================*
   clear()
 *==========================================================================*/
void clear(Surface surf, WORD fillColor=0,RECT *rect=NULL)
{
   DDBLTFX ddbltfx;

   // Use the blter to do a color fill to clear the back buffer
   ZeroMemory(&ddbltfx, sizeof(ddbltfx));
   ddbltfx.dwSize = sizeof(ddbltfx);
   ddbltfx.dwFillColor = (DWORD)fillColor;
   while(TRUE)
   {
      ddResult = surf->Blt(rect, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT, 
                           &ddbltfx);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         throw new IsLostException("Clear: Surface Lost");
      }
      if (ddResult == DDERR_GENERIC)
         throw new DDException("clear: Blt() generic!",ddResult);
      if (ddResult == DDERR_INVALIDCLIPLIST)
         throw new DDException("clear: Blt() invalid cliplist!",ddResult);
      if (ddResult == DDERR_INVALIDOBJECT)
         throw new DDException("clear: Blt() invalid object!",ddResult);
      if (ddResult == DDERR_INVALIDPARAMS)
         throw new DDException("clear: Blt() invalid params!",ddResult);
      if (ddResult == DDERR_INVALIDRECT)
      {
         sprintf(exc_string,"clear: Blt(%d,%d,%d,%d) invalid rect!",
            rect->left,rect->top,rect->right,rect->bottom);     
         throw new DDException(exc_string,ddResult);
      }
      if (ddResult == DDERR_NOALPHAHW)
         throw new DDException("clear: Blt() no alpha HW!",ddResult);
      if (ddResult == DDERR_NOBLTHW)
         throw new DDException("clear: Blt() no blitter HW!",ddResult);
      if (ddResult == DDERR_NOCLIPLIST)
         throw new DDException("clear: Blt() no cliplist!",ddResult);
      if (ddResult == DDERR_NODDROPSHW)
         throw new DDException("clear: Blt() no ROPS HW!",ddResult);
      if (ddResult == DDERR_NOMIRRORHW)
         throw new DDException("clear: Blt() no mirror HW!",ddResult);
      if (ddResult == DDERR_NORASTEROPHW)
         throw new DDException("clear: Blt() no raster op HW!",ddResult);
      if (ddResult == DDERR_NOROTATIONHW)
         throw new DDException("clear: Blt() no rotation HW!",ddResult);
      if (ddResult == DDERR_NOSTRETCHHW)
         throw new DDException("clear: Blt() no stretch HW!",ddResult);
      if (ddResult == DDERR_NOZBUFFERHW)
         throw new DDException("clear: Blt() no ZBuffer HW!",ddResult);
      if (ddResult == DDERR_SURFACEBUSY)
         throw new DDException("clear: Blt() surface busy!",ddResult);
      if (ddResult == DDERR_SURFACELOST)
         throw new DDException("clear: Blt() surface lost!",ddResult);
      if (ddResult == DDERR_UNSUPPORTED)
         throw new DDException("clear: Blt() unsupported!",ddResult);
      if (ddResult != DDERR_WASSTILLDRAWING && ddResult != DD_OK)
         throw new DDException("clear: Blt() failed!",ddResult);
   }
}

//-------------------------------------------------------------------------
// nastavi colorKey pro surface
//-------------------------------------------------------------------------
void setColorKey(Surface surf, WORD cKey)
{
 	DDCOLORKEY ddCKey;
	ddCKey.dwColorSpaceLowValue = (DWORD) cKey;
	ddCKey.dwColorSpaceHighValue = (DWORD) cKey;
	ddResult = surf->SetColorKey(DDCKEY_SRCBLT,&ddCKey);
	if (ddResult != DD_OK)
   {
		throw new DDException("SetColorKey() failed!",ddResult);
   }
}


//===========================================================================
// lock()
//===========================================================================
void lock(Surface surf, RECT *lpR, LPDDSURFACEDESC2 lpDDSD, DWORD dwFlags)
{
   ZeroMemory(lpDDSD,sizeof(*lpDDSD));
   lpDDSD->dwSize = sizeof(DDSURFACEDESC2);
   while(TRUE)
   {
      ddResult = surf->Lock(lpR,lpDDSD,dwFlags,NULL);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         throw new IsLostException("Lock: Surface Lost");
      }
      if (ddResult != DDERR_WASSTILLDRAWING && ddResult != DD_OK)
         throw new DDException("lock: Lock() failed");
   }
}
    
/*==========================================================================*
   flip()
 *==========================================================================*/
void flip(DWORD dwFlags)
{
#ifdef PROFILER_DEBUG
   profiler->startTimer(6);
#endif

   while(TRUE)
   {
      ddResult = frontbuffer->Flip(NULL,dwFlags);
      if (ddResult == DD_OK)
         break;
      if (ddResult == DDERR_SURFACELOST)
      {
         throw new IsLostException("Flip: Surface Lost");
      }
      if (ddResult != DDERR_WASSTILLDRAWING && ddResult != DD_OK)
         throw new DDException("flip: Flip() failed");
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(6);
#endif
}

//-----------------------------------------------------------------------------
// vytvori Bitmap a zkopiruje do nej zadanou surface
//-----------------------------------------------------------------------------
void screenshot(Surface pdds, const char *name)
{
   DDSURFACEDESC2    ddsd2;
   BYTE              *lpBMP;
   HBITMAP           hbm;
   DWORD             dwBMPOffset, dwSurfOffset;
   WORD              *lpSurface, wPixel;
   long              lPitch;
   DWORD             dwWidth, dwHeight;

   if (pdds == NULL)
      return;
   
   // Make sure this surface is restored.
   pdds->Restore();

   // Lock surface to obtain direct access to the surface data
   lock(pdds, NULL, &ddsd2);
   lPitch = ddsd2.lPitch / sizeof(WORD);
   lpSurface = static_cast<WORD *>(ddsd2.lpSurface);
   dwWidth = ddsd2.dwWidth;
   dwHeight = ddsd2.dwHeight;

   // Allocate memory for bitmap bits
   DWORD width = (dwWidth + 1)/2*2;
   lpBMP = new BYTE[width*dwHeight*3];
   if (lpBMP == NULL)
   {
      pdds->Unlock(NULL);
      return;
   }

   // SURFACE 2 BITMAP
	//,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	
   BYTE r,g,b;
   for (DWORD y=0; y < dwHeight; y++)
   {
      dwSurfOffset = (dwHeight-y-1)*lPitch;
      for(DWORD x=0; x < width; x++)
      {
         dwBMPOffset = y*width*3+x*3;
         
         if (x >= dwWidth)
         {
            lpBMP[dwBMPOffset] = 0;
            lpBMP[dwBMPOffset+1] = 0;
            lpBMP[dwBMPOffset+2] = 0;
            break;
         }
         
         wPixel = lpSurface[dwSurfOffset+x];
         
         b = (wPixel & bMask) << 3;
         g = (wPixel & gMask) >> (gBits - 3);
         r = (wPixel & rMask) >> (2+gBits);
         
         lpBMP[dwBMPOffset] = b;
         lpBMP[dwBMPOffset+1] = g;
         lpBMP[dwBMPOffset+2] = r;
      }
   }
   
   //''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
   
   // Don't forget to unlock the surface
   pdds->Unlock(NULL);
   
   hbm = CreateBitmap(width,dwHeight,1,24,(const void *)lpBMP);
   if (hbm == NULL)
   {
      delete [] lpBMP;
      return;
   }
   
   PBITMAPINFO pbmi;
   pbmi = (PBITMAPINFO) LocalAlloc(LPTR, sizeof(BITMAPINFOHEADER));
   
   pbmi->bmiHeader.biSize = sizeof(BITMAPINFOHEADER); 
   pbmi->bmiHeader.biWidth = width; 
   pbmi->bmiHeader.biHeight = dwHeight; 
   pbmi->bmiHeader.biPlanes = 1; 
   pbmi->bmiHeader.biBitCount = 24; 
   
   /* If the bitmap is not compressed, set the BI_RGB flag. */ 
   pbmi->bmiHeader.biCompression = BI_RGB; 
   
   /* 
   * Compute the number of bytes in the array of color 
   * indices and store the result in biSizeImage. 
   */ 
   
   pbmi->bmiHeader.biSizeImage = (pbmi->bmiHeader.biWidth + 7) /8 
      * pbmi->bmiHeader.biHeight 
      * 24; 
      /* 
      * Set biClrImportant to 0, indicating that all of the 
      * device colors are important. 
   */ 
   pbmi->bmiHeader.biClrImportant = 0; 
   
   HANDLE hf;                  /* file handle */ 
   BITMAPFILEHEADER hdr;       /* bitmap file-header */ 
   PBITMAPINFOHEADER pbih;     /* bitmap info-header */ 
   DWORD dwTotal;              /* total count of bytes */ 
   DWORD cb;                   /* incremental count of bytes */ 
   BYTE *hp;                   /* byte pointer */ 
   
   DWORD dwTmp; 
   
   
   pbih = (PBITMAPINFOHEADER) pbmi; 
   
   /* Create the .BMP file. */ 
   
   hf = CreateFile(name, 
      GENERIC_READ | GENERIC_WRITE, 
      (DWORD) 0, 
      (LPSECURITY_ATTRIBUTES) NULL, 
      CREATE_ALWAYS, 
      FILE_ATTRIBUTE_NORMAL, 
      (HANDLE) NULL); 
   
   if (hf == INVALID_HANDLE_VALUE) 
   {
      delete [] lpBMP;
      return;
   }
   
   hdr.bfType = 0x4d42;        /* 0x42 = "B" 0x4d = "M" */ 
   
   /* Compute the size of the entire file. */ 
   hdr.bfSize = (DWORD) (sizeof(BITMAPFILEHEADER) + 
      pbih->biSize + pbih->biSizeImage); 
   
   hdr.bfReserved1 = 0; 
   hdr.bfReserved2 = 0; 
   
   /* Compute the offset to the array of color indices. */ 
   
   hdr.bfOffBits = (DWORD) sizeof(BITMAPFILEHEADER) + 
      pbih->biSize;
   
   /* Copy the BITMAPFILEHEADER into the .BMP file. */ 
   
   if (!WriteFile(hf, (LPVOID) &hdr, sizeof(BITMAPFILEHEADER), 
      (LPDWORD) &dwTmp, (LPOVERLAPPED) NULL)) 
   {
      CloseHandle(hf);
      delete [] lpBMP;
      return;
   }
   
   /* Copy the BITMAPINFOHEADER and RGBQUAD array into the file. */ 
   
   if (!WriteFile(hf, (LPVOID) pbih, sizeof(BITMAPINFOHEADER), 
      (LPDWORD) &dwTmp, (LPOVERLAPPED) NULL)) 
   {
      CloseHandle(hf);
      delete [] lpBMP;
      return;
   }
   
   
   /* Copy the array of color indices into the .BMP file. */ 
   dwTotal = cb = pbih->biSizeImage; 
   hp = lpBMP; 
   while (cb > 32)  { 
      if (!WriteFile(hf, (LPSTR) hp, (int) 32, 
         (LPDWORD) &dwTmp, (LPOVERLAPPED) NULL)) 
      {
         CloseHandle(hf);
         delete [] lpBMP;
         return;
      }
      cb-= 32; 
      hp += 32; 
   } 
   if (!WriteFile(hf, (LPSTR) hp, (int) cb, 
      (LPDWORD) &dwTmp, (LPOVERLAPPED) NULL)) 
   {
      CloseHandle(hf);
      delete [] lpBMP;
      return;
   }
   
   
   /* Close the .BMP file. */ 
   
   if (!CloseHandle(hf)) 
   {
      delete [] lpBMP;
      return;
   }
   
   /* Free memory. */
   delete [] lpBMP;

}


/*==========================================================================*
   doneDD()
 *==========================================================================*/
void doneDD()
{
   if (surfaceManager != NULL)
   {
      delete surfaceManager;
      surfaceManager = NULL;
   }

   if (lpDD != NULL)
   {
      if (ddHWND != NULL)
         lpDD4->SetCooperativeLevel(ddHWND,DDSCL_NORMAL);
      lpDD->Release();
      lpDD = NULL;
      lpDD4 = NULL;
   }
}



