//*************************************************************************
// Utilitky pro profilovani vykonu programu
//*************************************************************************
// Written by : Vojvo
// Language   : VisuC6
// Version    : 0.0.1
// Last update: 09.11.1999
//*************************************************************************
// Changes:
// 0.0.1 - prvni verze

#include <stdio.h>
#include "profiler.h"

//-------------------------------------------------------------------------
// konstruktor s uvedenim maximalniho poctu timeru
//-------------------------------------------------------------------------
PerformanceProfiler::PerformanceProfiler(int maxTimers)
{
   LARGE_INTEGER ticksPerCount;

   timercount = maxTimers;
   starttime = new LARGE_INTEGER[timercount];
   timer = new double[timercount];
   if (QueryPerformanceFrequency(&ticksPerCount))
   {
      dTicks = ticksPerCount.LowPart + 0xFFFFFFFF * ticksPerCount.HighPart;
   }
   else
   {
      dTicks = -1.0;
   }
   reset();
}

//-------------------------------------------------------------------------
// resetuje jeden timer
//-------------------------------------------------------------------------
void PerformanceProfiler::resetTimer(int timerid)
{
   starttime[timerid].LowPart = 0;
   starttime[timerid].HighPart = 0;
   timer[timerid] = 0.0;
}


//-------------------------------------------------------------------------
// startuje timer
//-------------------------------------------------------------------------
void PerformanceProfiler::startTimer(int timerid)
{
   if (dTicks > 0.0)
      QueryPerformanceCounter(starttime+timerid);
}

//-------------------------------------------------------------------------
// zastavuje timer a pripocita jeho hodnotu
//-------------------------------------------------------------------------
void PerformanceProfiler::stopTimer(int timerid)
{
   LARGE_INTEGER stop_t,start_t;

   if (dTicks > 0.0)
   {
      QueryPerformanceCounter(&stop_t);
      start_t = starttime[timerid];
      double dStart = start_t.LowPart + 0xFFFFFFFF * start_t.HighPart;
      double dStop = stop_t.LowPart + 0xFFFFFFFF * stop_t.HighPart;
      double dDiff = dStop - dStart;
      timer[timerid] += (dDiff/dTicks);
   }
}

//-------------------------------------------------------------------------
// zapise namerene hodnoty do souboru
//-------------------------------------------------------------------------
void PerformanceProfiler::makeReport()
{
   FILE *fw;

   fw = fopen("profilerrep.txt","wt");
   if (fw == NULL)
      return;
   for (int i=0; i < timercount; i++)
   {
      fprintf(fw,"%-6d: %10.1lf\n",i,timer[i]);
   }
   fclose(fw);
}

//-------------------------------------------------------------------------
// resetuje vsehcny timery
//-------------------------------------------------------------------------
void PerformanceProfiler::reset()
{
   for (int i=0; i < timercount; i++)
   {
      resetTimer(i);
   }
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
PerformanceProfiler::~PerformanceProfiler()
{
   delete [] starttime;
   delete [] timer;
}




