#include "consts.h"
#include "graphDD.h"
#include "scrolling.h"

#define SI_CELPROP         1
#define SI_WINDOW          2
#define SI_POSITION        4
#define SI_DESTINATION     8
#define SI_SPEED           16
#define SI_ALL             15
#define SI_ALLBUTDEST      7

//-------------------------------------------------------------------------
// Scroller::Scroller
//-------------------------------------------------------------------------
// implicitni konstruktor
Scroller::Scroller()
{
   initialized = 0;
   mul_speed = MUL_MOVE;
   scrolling = false;
   cels = NULL;
}

//-------------------------------------------------------------------------
// Scroller::setCelProperties
//-------------------------------------------------------------------------
// nastavuje parametry scrollovanych jednotek
void Scroller::setCelProperties(DWORD celsx, DWORD celsy, DWORD celw, 
                                DWORD celh)
{
   celsX = celsx;
   celsY = celsy;
   celWidth = celw;
   celHeight = celh;
   cels = new Surface[celsX*celsY];
   for (DWORD i = 0; i < celsX; i++)
      for (DWORD j = 0; j < celsY; j++)
         cels[j*celsY+i] = NULL;
   initialized |= SI_CELPROP;
}

//-------------------------------------------------------------------------
// Scroller::setWindow
//-------------------------------------------------------------------------
// nastavuje pozici a velikost okna ve kterem se scrolluje
void Scroller::setWindow(DWORD wndx, DWORD wndy, DWORD wndw, DWORD wndh)
{
   wndWidth = wndw;
   wndHeight = wndh;
   wndX = wndx;
   wndY = wndy;
   initialized |= SI_WINDOW;
}

//-------------------------------------------------------------------------
// Scroller::setPosition
//-------------------------------------------------------------------------
// nastavuje pozici leveho horniho rohu okna
void Scroller::setPosition(DWORD x, DWORD y)
{
   mul_posX = x * MUL_MOVE;
   mul_posY = y * MUL_MOVE;
   if (initialized && SI_DESTINATION)
   {
      long difX = mul_destX - mul_posX;
      long difY = mul_destY - mul_posY;
      mul_speedX = difX * mul_speed / (difX + difY);
      mul_speedY = difY * mul_speed / (difX + difY);
      scrolling = true;
      initialized |= SI_SPEED;
   }
   initialized |= SI_POSITION;
}

//-------------------------------------------------------------------------
// Scroller::setDestination
//-------------------------------------------------------------------------
// nastavuje cilovou pozici scrollingu
void Scroller::setDestination(DWORD x, DWORD y)
{
   mul_destX = x * MUL_MOVE;
   mul_destY = y * MUL_MOVE;
   if (initialized && SI_POSITION)
   {
      long difX = mul_destX - mul_posX;
      long difY = mul_destY - mul_posY;
      long diff = abs(difX) + abs(difY);
      if (diff > 0)
      {
         mul_speedX = difX * mul_speed / diff;
         mul_speedY = difY * mul_speed / diff;
         scrolling = true;
         initialized |= SI_SPEED;
      }
   }
   initialized |= SI_DESTINATION;
}

//-------------------------------------------------------------------------
// Scroller::setSpeed
//-------------------------------------------------------------------------
// nastavuje relativni rychlost scrollingu
void Scroller::setSpeed(long speed)
{
   long old_speed = mul_speed;
   mul_speed = speed;
   if (initialized && SI_SPEED)
   {
      mul_speedX = mul_speedX * mul_speed / old_speed;
      mul_speedY = mul_speedY * mul_speed / old_speed;
   }
}

//-------------------------------------------------------------------------
// Scroller::setCel
//-------------------------------------------------------------------------
// nastavuje grafiku bunky
void Scroller::setCel(int x, int y, Surface surf)
{
   cels[x,y] = surf;
}

//-------------------------------------------------------------------------
// Scroller::scroll()
//-------------------------------------------------------------------------
// scrolluje k zadanemu cili zadanou rychlosti
void Scroller::scroll()
{
   if (!scrolling)
      return;

   if (abs(mul_destX-mul_posX) <= abs(mul_speedX) 
      && abs(mul_destY-mul_posY) <= abs(mul_speedY))
   {
      mul_posX = mul_destX;
      mul_posY = mul_destY;
      scrolling = false;
   }   
   else
   {
      mul_posX += mul_speedX;
      mul_posY += mul_speedY;
   }
}

//-------------------------------------------------------------------------
// Scroller::scroll(DWORD,DWORD)
//-------------------------------------------------------------------------
// scrolluje o zadany pocet pixli
void Scroller::scroll(DWORD byx, DWORD byy)
{
   mul_posX += byx * MUL_MOVE;
   mul_posY += byy * MUL_MOVE;
}

//-------------------------------------------------------------------------
// Scroller::paint
//-------------------------------------------------------------------------
// vykresluje scrollovanou grafiku do zadaneho okna cilove surface
void Scroller::paint(Surface to, DWORD dwFlags)
{
   long x, y, maxx, maxy, minpos;
   RECT r;
   int celx, cely;

   x = mul_posX / MUL_MOVE;
   maxx = x + wndWidth;
   y = mul_posY / MUL_MOVE;
   maxy = y + wndHeight;

   while (y < maxy)
   {
      cely = y / celHeight;
      minpos = min((cely+1)*celHeight,static_cast<DWORD>(maxy))-1;
      r.top = y - (cely * celHeight);
      r.bottom = minpos - (minpos / celHeight) * celHeight + 1;
      y = minpos;
      while (x < maxx)
      {
         celx = x / celWidth;
         minpos = min((celx+1)*celWidth,static_cast<DWORD>(maxx))-1;
         r.left = x - (celx * celWidth);
         r.right = minpos - (minpos / celWidth) * celWidth + 1;
         x = minpos;
         putcel(wndX,wndY,to,r,cels[cely*celsY+celx],dwFlags);
      }
   }
}

//-------------------------------------------------------------------------
// Scroller::~Scroller
//-------------------------------------------------------------------------
// destruktor
Scroller::~Scroller()
{
   if (cels != NULL)
   {
      delete [] cels;
      cels = NULL;
   }
}