/*
 *--------------------------------------------------------------------
 *
 * tint
 *
 * Knihovna funkci pro spravu tintu. Nahravani tintu ze souboru,
 * dealokace tintu.
 *
 * verze   0.9.7 - 14.05.1999
 * autori  Rory
 *
 *--------------------------------------------------------------------
 * zmeny
 *
 * 0.9.7 - nacitani pouziva fakeopen
 * 0.0.1 - prvni verze
 *--------------------------------------------------------------------
 */

#include <stdio.h>
#include "exceptions.h"
#include "fakedir.h"
#include "graphDD.h"
#include "tint.h"

TintTable tnts[MAX_TINTS];
int tntCount;

/*-------------------------------------------------------------------------*
    loadTint
 *-------------------------------------------------------------------------*/
void loadTint(char *name, unsigned long tntSize)
{
   FILE *fr;

   for (int i=0; i<MAX_TINTS; i++)
      tnts[i] = NULL;
   tntCount = 0;

   fr = fakeopen("data\\pictures.dir",name,"rb");
   if (!fr)
      throw new BubException("loadTint: TNT file not found!");
       
   TintTable pomtnt = new WORD[tntSize];
   while (fread(pomtnt,2,tntSize,fr) == tntSize && tntCount < MAX_TINTS)
   {
      tnts[tntCount] = pomtnt;
      tntCount++;
      pomtnt = new WORD[tntSize];
   }
   fclose(fr);
   delete pomtnt;
   if (tntCount == 0)
      throw new BubException("loadTint: Error reading TNT file!");
}       
        
/*-------------------------------------------------------------------------*
    doneTint
 *-------------------------------------------------------------------------*/
void doneTint()
{
   int i = 0;

   while (i < tntCount)
   {
      if (tnts[i] != NULL)
      {
         delete tnts[i];
         tnts[i] = NULL;
      }
      i++;
   }
   tntCount = 0;
}

//-------------------------------------------------------------------------
// vytvari masku k surface
//-------------------------------------------------------------------------
TintMask surface2mask(Surface surface)
{
   DDSURFACEDESC2 ddsd2;
   BYTE *mask;
   WORD *data;
   long lPitch;
   DWORD dwWidth, dwHeight;
   DWORD offset,maskoffset;

   lock(surface,NULL,&ddsd2);
   lPitch = ddsd2.lPitch / 2;
   dwWidth = ddsd2.dwWidth;
   dwHeight = ddsd2.dwHeight;
   data = (WORD *)ddsd2.lpSurface;

   mask = new BYTE[dwWidth*dwHeight+8];

   *((DWORD *)mask) = dwWidth;
   *((DWORD *)mask + 1) = dwHeight;
   for (DWORD i = 0; i<dwHeight; i++)
   {
      offset = i*lPitch;
      maskoffset = i*dwWidth;
      for (DWORD j = 0; j < dwWidth; j++, offset++, maskoffset++)
         mask[maskoffset+8] = (data[offset] == colorKey) ? 0:1;
   }

   surface->Unlock(NULL);

   return mask;
}



