//*************************************************************************
//                       Bunny  class implementation                       
//*************************************************************************
//  Written by : Gunny & Vojvo
//  Language   : VisuC6 + DX6.1
//  Version    : 0.10.1
//  Last update: 14.11.99
//*************************************************************************
// changes
// 0.10.1 - zvuky stereo


#include <stdio.h>
#include "utils.h"
#include "exceptions.h"
#include "map.h"
#include "wsound.h"
#include "game.h"
#include "sprites.h"

//#define BLASTDELAYREPORT //vypisuje hodnotu blastreport


#define SPEED_BUNMEATX  1966        // MOVE_100
#define SPEED_BUNMEATY  9830        // MOVE_500
#define SPEED_BUNPANTX  3932        // MOVE_200
#define SPEED_BUNPANTY  3584        // MOVE_350

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bunny::Bunny(int onx, int ony, Property *prop) : Monster(onx,ony, prop)
{
   blastdelay = 1000 + random(4500);
   // at se nejdriv nafoukne!
   action = SPMA_FIRE;
   olddecision = decision;
   decision = 0;
   old_stable_l = 0;
   old_stable_r = 0;
}

//-------------------------------------------------------------------------
// reakce na mapu a okoli
//-------------------------------------------------------------------------
void Bunny::MapReflex()
{
   int stable;

   #ifdef BLASTDELAYREPORT
   // prubezne vypisuje blastdelay
   char me[5] = "000";
   me[0] += blastdelay / 100;
   me[1] += (blastdelay / 10) % 10;
   me[2] += blastdelay % 10;
   message.Set(me,sfnt);
   #endif

   // na startu je treba si odlozit meze stability
   if (!old_stable_l)
   {
      old_stable_l = stable_l;
      old_stable_r = stable_r;
   }

   if (direction == DIR_R)
      stable = stable_r;
   else
      stable = stable_l;

   // otocka na konci plosinky
   if (!map_value(newx+stable,newy+ysize,map) && !(action & SPMA_JUMP))
   {
      newx = oldx;
      mul_x = newx << SHIFT_MOVE + MUL_HALF;
      movex = 0;
      decision ^= 3;
      if (decision<2)
      {
         direction = DIR_L;
         frames.setDirection(DIR_L);
      }
      else
      {
         direction = DIR_R;
         frames.setDirection(DIR_R);
      }
   }

   IsStable();

   // osetreni "prochazeni zdi" doleva a doprava
   if (CantGoLR())
   {
      newx = (newx/MAP_CELX) * MAP_CELX;
      // kdyz jde doleva je treba ho posunout o jeden zpatky
      if (direction = DIR_L)
         newx += MAP_CELX;
      mul_x = newx << SHIFT_MOVE + MUL_HALF;
      movex = 0;
   }

   // uz se nafouknul? tak to muze jit!
   if (action & SPMA_FIRE && frames.chainid == CHAIN_WALK)
   {
      //message.Set("Nafouknul se.",qfnt);
      action = 0;
      decision = olddecision;
      stable_l = old_stable_l;
      stable_r = old_stable_r;
   }

   // osetreni dopadu na plosinku
   if (action & SPMA_JUMP && movey > 0 && is_stable)
   {
      sound_device->Play(2,newx+xsize/2);
      newy = (int)(newy/MAP_CELY) * MAP_CELY;
      mul_y = newy << SHIFT_MOVE;
      movey = 0;
      movex = 0;
      frames.setChain(CHAIN_STAND);
      action = SPMA_FIRE;
      olddecision = decision;
      decision = 0;
      blastdelay += 900;
      // tohle by melo zabranit bugovani (prestaval chodit)
      // blastdelay tady MUSI byt >=58
      // nebo naopak - pokud je mensi nez 26, tak to kralik ustoji,
      //  ale vypada to divne, protoze hned po dopadu se odrazi
      // (konstanty jsou zavisle na framechainu a na frame_div!)
   }

   // nenadaly pad - nejspis z bubliny
   if (!(action & SPMA_JUMP) && !is_stable)
    // mozna jeste && !(action & SPMA_FIRE) ?
    // ...to uz by bylo zbytecne, ne?
   {
      decision = olddecision;
      action |= SPMA_JUMP;
      frames.setChain(CHAIN_FALL);
   }

   // snizovani pocitadla na rozprsk
   //at nepraska v padu nebo hned nahore
   if (!(action & SPMA_JUMP) && newy > 70)
      blastdelay--;
   // nafukovani a priprava
   if (blastdelay == 200) // zacina se nafukovat
   {
      movex = 0;
      olddecision = decision;
      decision = 0;
      action |= SPMA_FIRE;
      frames.setChain(CHAIN_FIRE);
      sound_device->Play(37,newx+xsize/2); // syci!
   }
   
   // prask?!
   if (blastdelay<=0)
   {
      blastdelay = 900;
      //snaska
      //action ^= SPMA_FIRE;
      decision = olddecision;
      //decision |= SPMA_JUMP;
      //
      if (UFOcnt < MAX_UFOS) 
      {
         UFOs[UFOcnt] = new Meat(newx+19,newy+19,
            properties.member[ID_BUNMEAT],gravitation);         
         UFOs[UFOcnt]->movex = -MOVE200;
         UFOs[UFOcnt]->movey = -MOVE300;
         UFOcnt++;
      }
      if (UFOcnt < MAX_UFOS) 
      {
         UFOs[UFOcnt] = new Meat(newx+19,newy+19,
            properties.member[ID_BUNMEAT],gravitation);
         UFOs[UFOcnt]->movex = MOVE200;
         UFOs[UFOcnt]->movey = -MOVE300;
         UFOcnt++;
      }
      if (UFOcnt < MAX_UFOS) 
      {
         UFOs[UFOcnt] = new Meat(newx+19,newy+19,
            properties.member[ID_BUNPANT],gravitation);
         UFOs[UFOcnt]->movex = -MOVE100;
         UFOs[UFOcnt]->movey = -MOVE400;
         UFOcnt++;
      }
      if (UFOcnt < MAX_UFOS) 
      {
         UFOs[UFOcnt] = new Meat(newx+19,newy+19,
            properties.member[ID_BUNPANT],gravitation);
         UFOs[UFOcnt]->movex = MOVE100;
         UFOs[UFOcnt]->movey = -MOVE400;
         UFOcnt++;
      }
      sound_device->Play(38,newx+xsize/2);
      if (s_drops) 
         drops->Blast(newx+10,newy+10,DROP_RED,gravitation);
      // korekce pozice hlavy
      newy -= 28;
      mul_y = newy << SHIFT_MOVE;
      // a odraz!
      action |= SPMA_JUMP;
      frames.setChain(CHAIN_JUMP);
      Slow(0,JUMP_STRENGTH);
      frames.jumpNext();
      stable_l = 18;
      stable_r = 21;
   } 
}