//*************************************************************************
//                       kinda like drops from bubble                      
//*************************************************************************
// Written by : Vojvo & Gunny
// Language   : VisuC6 + DX6.1
// Version    : 0.99.4
// Last update: 08.12.99 
//*************************************************************************
// 0.99.4 - droplist pouziva pro surface get() misto load()

#include <stdio.h>
#include <stdlib.h>
#include "graphDD.h"
#include "surfaceManager.h"
#include "gonio.h"
#include "utils.h"
#include "application.h"
#include "drops.h"


//#define DROPTEST

#define WARPCNT 144


//#########################################################################
//--------------------------------- kapky  --------------------------------
//#########################################################################
BubbleDrop::BubbleDrop(Surface gfx,int ix,int iy,int vx,int vy,
                       BYTE color, int g)
{
   mul_x = ix << SHIFT_MOVE;
   mul_y = iy << SHIFT_MOVE;
   if (color > MAX_DROP_COLOR)
      color = DROP_BLACK;
   frameRect.left = color * DROP_SIZE;
   frameRect.right = frameRect.left + DROP_SIZE;
   frameRect.top = 0;
   frameRect.bottom = DROP_SIZE;
   movex=vx;
   movey=vy;
   grave=g;
   warp=0;
   warpcnt=72;
   b_mminx = static_cast<DWORD>(PLAYFIELD_MIN_X) << SHIFT_MOVE;
   b_mmaxx = static_cast<DWORD>(PLAYFIELD_MAX_X - DROP_SIZE) << SHIFT_MOVE;
   b_mminy = static_cast<DWORD>(PLAYFIELD_MIN_Y) << SHIFT_MOVE;
   b_mmaxy = static_cast<DWORD>(PLAYFIELD_MAX_Y - DROP_SIZE) << SHIFT_MOVE;
   this->gfx = gfx;
}

//==========================================================================
void BubbleDrop::Paint(Surface dest)
{
   int xx = mul_x >> SHIFT_MOVE;
   int yy = mul_y >> SHIFT_MOVE;
   putcel(xx,yy,dest,frameRect,gfx);
   SET_USEDMAP(xx,yy);
   SET_USEDMAP(xx+DROP_SIZE,yy);
   SET_USEDMAP(xx,yy+DROP_SIZE);
   SET_USEDMAP(xx+DROP_SIZE,yy+DROP_SIZE);
}

//==========================================================================
void BubbleDrop::Clear(Surface dest, Surface src)
{
}

//=========================================================================
int BubbleDrop::Move()
{
   movey += grave;
   movex += warp;
   mul_x += movex;
   mul_y += movey;
   if (mul_x < b_mminx || mul_x > b_mmaxx || mul_y < b_mminy || mul_y > b_mmaxy)
   {
      return OUT_OF_BOUNDS;
   }
   ++warpcnt;
   if (warpcnt >= WARPCNT)
   {
      warpcnt = 0;
      warp = -warp;
   }
   return IN_BOUNDS;
}


//#########################################################################
//------------------------------ kapkecnik --------------------------------
//#########################################################################

DropList::DropList()
{ 
   count = 0; 
   gfx = surfaceManager->get("drops.r16");
}

//==========================================================================
void DropList::Blast(int ax, int ay, BYTE color, int grav)
//rozprsk bubliny na ax:ay (levy horni roh) a barvou color
{
   int tomake=(MAXDROP-count+4)/7;
   int angle;
   for (int i=0; i<tomake; i++)
   {
      angle= 220 + random(100);
      list[count]=new BubbleDrop(gfx,ax,ay,2*cos(angle),
				     3*sin(angle),color,grav);
      count++;
   }
}

//==========================================================================
void DropList::HorizontalBlast(int ax, int ay, BYTE color, int grav, int direction)
//rozprsk od zdi na ax:ay (levy horni roh) a barvou color
{
   int tomake=(MAXDROP-count+4)/7;
   int angle;
   for (int i=0; i<tomake; i++)
   {
      angle= 220 + random(100);
      list[count]=new BubbleDrop(gfx,ax,ay,direction*2*sin(angle),2*cos(angle),color,grav);
      count++;
   }
}

//==========================================================================
void DropList::OneDrop(int ax, int ay, int mx, int my, BYTE color, int grav,
                       int warp)
//vyrobi 1 bublinku na ax:ay (stred) s rychlosti mx:my, barvou color a grav.
{
   if (count<MAXDROP)
   {
      list[count]=new BubbleDrop(gfx,ax,ay,mx,my,color,grav);
      list[count]->warp = warp;
      count++;
   }
}

//==========================================================================
void DropList::Move()
//posachuje a pripadne vyhodi
{
   for (int i=0; i<count; i++)
   {
      if (list[i]->Move() == OUT_OF_BOUNDS)
      //vypadla z obrazovky! ZRUSIT!
      {
         count--;
         delete list[i];
         list[i]=list[count];
         list[count]=NULL;
         i--;
      }
   }
}

//==========================================================================
void DropList::RemoveAll()
{
   for (int i=0; i<count; i++)
      delete list[i];
   count=0;
}
