//*************************************************************************
//                   Fireable creatures related classes                    
//*************************************************************************
// Written by : Vojvo & Gunny
// Language   : VisuC6 + DX6.1
// Version    : 0.10.2
// Last update: 15.11.1999
//*************************************************************************
// Changes:
// 0.10.2 / upraveno turndelay (snad zabere)
// 0.10.1 - zvuky stereo
// 0.9.6 - funguje RedDragon
// 0.9.5 - prvni verze, funguje Fireable a YellowDragon

#include "dxconsts.h"
#include "consts.h"
#include "wsound.h"
#include "exceptions.h"
#include "utils.h"
#include "sprites.h"
#include "game.h"
#include "map.h"

#define FIRE_DURATION         1500
#define DELTA_TO_SEE          10

#define RD_LONGFALL           700
#define RD_MADEMIND           70
#define RD_BORED2JUMPSTATIC   1500
#define RD_BORED2JUMPDYNAMIC  1500
#define RD_TURNDELAY          380

//###########################################################################
//-------------------------------- Fireable ---------------------------------
//###########################################################################

Fireable::Fireable(int onx, int ony, Property *prop)
: Monster(onx,ony,prop)
{
   firedone = 0;
   fireframesfinished = TRUE;
   timefinished = 0;
}

//===========================================================================

int Fireable::Move()
{
   // pozrime sa, neni-li player v dohledu
   PlayerReflex();
   // pokud fireuje a skoncily framy strileni
   // obnovi pohyb
   if (!fireframesfinished && frames.chainid != CHAIN_FIRE)
   {
      fireframesfinished = TRUE;
      decision = olddecision;
      action &= ~SPMA_FIRE;
   }

   // Nastal cas povolit dalsi fireball ?
   if (timefinished)
      timefinished--;

  // defaultni obsluha
  return Monster::Move();
}

//===========================================================================

void Fireable::PlayerReflex()
{
   int xp,yp;
   
   // pokud muj fireball skoncil, ja jsem skoncil a je volne misto v poli
   // fireballu
   if (firedone == 0 && UFOcnt < MAX_UFOS && timefinished == 0)
   {
      // pokud player1 neni nesledovatelny
      if (!(player1->state & UNCHASEABLE))
      {
         xp = player1->newx;
         yp = player1->newy;
         // pokud je player1 ve smeru pohledu
         if ((long)movex * (long)(xp - newx) > 0)
         {
            yp -= newy;
            // a pokud je na moji urovni, tak FFFIIIRRREEE
            if (yp > -DELTA_TO_SEE && yp < DELTA_TO_SEE) {
               Fire();
               return;
            }
         }
      }
      // pokud player2 neni nesledovatelny
      if (!(player2->state & UNCHASEABLE))
      {
         xp = player2->newx;
         yp = player2->newy;
         // pokud je player2 ve smeru pohledu
         if ((long)movex * (long)(xp - newx) > 0L)
         {
            yp -= newy;
            // a pokud je na moji urovni, tak FFFIIIRRREEE
            if (yp > -DELTA_TO_SEE && yp < DELTA_TO_SEE)
               Fire();
         }
      }
   }
}

//===========================================================================

void Fireable::Fire()
{
   if (UFOcnt >= MAX_UFOS)
   {
      return;
   }
   UFOs[UFOcnt++] = new Fireball(newx+((movex>0)?35:5),newy,
				properties.member[ID_FIREBALL],this,(movex>0) ? MOVE300:-MOVE300);
  firedone++;
  fireframesfinished = FALSE;
  timefinished = FIRE_DURATION;
  movex = 0;
  olddecision = decision;
  decision = 0;
  frames.setChain(CHAIN_FIRE);
  action |= SPMA_FIRE;
  sound_device->Play(9,newx+xsize/2);
  // co kdyz je fireball hned na zacatku uz za zdi
  if ((direction) ?
     map_value(newx+59,newy+11,map):
	   map_value(newx-9,newy+11,map))
  {
    sound_device->Play(10,newx+xsize/2);
    UFOs[UFOcnt-1]->Melt();
  }
}

//===========================================================================

void Fireable::Notify()
{
  firedone--;
}

//===========================================================================

Fireable::~Fireable()
{
  // je treba zrusit reference v mych fireballech
  int i = 0;
  while (i < UFOcnt)
  {
    if (UFOs[i]->parent == this)
    {
      UFOs[i]->parent = NULL;
    }
    i++;
  }
}


//###########################################################################
//------------------------------- YellowDragon ------------------------------
//###########################################################################
YellowDragon::YellowDragon(int onx, int ony, Property *prop)
: Fireable(onx,ony,prop)
{}

//===========================================================================

void YellowDragon::MapReflex()
{
  int stable;

  stable = (direction) ? stable_r : stable_l;
  if (!map_value(newx+stable,newy+ysize,map) && !(action & SPMA_JUMP))
  {
    newx = oldx;
    mul_x = newx << SHIFT_MOVE + MUL_HALF;
    movex = 0;
    decision ^= 3;
  }
  Fireable::MapReflex();
}

//###########################################################################
//-------------------------------   RedDragon  ------------------------------
//###########################################################################
RedDragon::RedDragon(int onx, int ony, Property *prop, int turnpos)
: Fireable(onx,ony,prop)
{
   turndelay = 0;
   bored2jump = 20;
   turnpossibility = turnpos-1; //misto 1-10 bude 0-9
   mademind = 0;
   longfall = 0;
}

//===========================================================================
void RedDragon::KeyReflex()
{
   int stable;
   int futurex = (mul_x + movex) >> SHIFT_MOVE;
   
   if (action & SPMA_JUMP)
   { // osetreni dlouhych padu
      if (!longfall)
         longfall = RD_LONGFALL;
      if (longfall==1)
         decision ^= 3;
      longfall--;
   }
   if (turndelay)
   { // pred chvili svisle vyskocil
      turndelay--;
      if (!turndelay)
      { // dopad po svislem vyskoku
         decision = decided;
         // pokud vyrazi planovanym smerem, nespadne nahodou?
         stable = (decision-1) ? stable_r : stable_l;
         if (!map_value(futurex+stable,newy+ysize,map))
         {
            decision ^= 3;
         }
      }
   }
   else
   { //uz nejakou dobu neposkocil
      bored2jump--;
      if (!bored2jump)
      {
         bored2jump = RD_BORED2JUMPSTATIC+random(RD_BORED2JUMPDYNAMIC);
         longfall = 2*RD_LONGFALL;
         decision ^= 4;
      }
   }
   //
   if (mademind) 
      mademind--;
   stable = (direction == DIR_R) ? stable_r : stable_l;
   if (!map_value(futurex+stable,newy+ysize,map) && !(action & SPMA_JUMP) && !mademind)
   {
      mademind = RD_MADEMIND; //dalsich x testu nezmeni rozhodnuti
      mul_x -= movex;
      newx = mul_x >> SHIFT_MOVE;
      //takze co udela?
      int r = random(10);
      if (r>5)
      { //skok
         longfall = 2*RD_LONGFALL;
         decision ^= 4;
         //a aby zase chvili nemel chut skakat
         bored2jump = RD_BORED2JUMPSTATIC + random(RD_BORED2JUMPDYNAMIC);
      }
      r = random(100);
      if (r<turnpossibility)
      { //obratka
         movex = 0;
         decision ^= 3;
         if (decision&4)
         { //tohle resi svisly vyskok
            turndelay = RD_TURNDELAY;
            decided = decision & 3;
            decision ^= decided;
            action &= ~(SPMA_LEFT+SPMA_RIGHT);
         }
      }
      else if (decision<4)
      {
         decided++;
      }
   }
   Fireable::KeyReflex();
}

