//*************************************************************************
//                              Game class
//*************************************************************************
//  Written by : Vojvo (& Gunny)
//  Version    : 1.1.2
//  Last update: 12.11.2004
//*************************************************************************
/*
   changes:
   1.1.2  / opraven rozpad redalertu po alt-tabu v interscreenu
			 / pro jistotu upravena prce s m_background
			 / opraveno restore - _redalert se pekresloval, jen kdy u bel
			 / opravena realertova casomira po restoru
   1.1.1  / opraven preklep u startu Murrayho
	       / pridana metoda Place pro usazeni hrace do levelu
			 / zruseny nefunkcni plynuly nastup redalertu
			 / opraveno pocitadlo casu pri novem red alertu
			 / zmenen interval sireny pri redalertu na 2 sekundy (tedy 1.98 =)
			 / upravena inicializace sireny, aby houkala hned i bonusovych lvl
// 1.1.0  / predelany redalert na 2 surfaces a 1 pointer (funkcni!? =)
// 1.0.1  - pokus o finalni verzi
// 0.99.5 - to_fuq presunut mezi instancni promenne, aby pri dominu fungoval 
//          tak, jak ma
//        - snaha o upravu zobrazeni telportu a bublin pri redalertu
//        - nase stare hodiny biji ctyri vteriny (nebo 5)
//        - hraci mizi ve final teleportech
//        - ukazatele casove omezenych kouzel
// 0.99.4 - loadovani globalni grafiky vycleneno do special funkce gfx_load
//        - pri murraym se pocita cas bez zavislosti na poctu hracu ve hre
// 0.99.3 - reverse stereo
//        - sektorove clearovani
//        - zmena teleportu
// 0.99.1 - demoverze
//        - FPS se vypisuji jen na switch test
// 0.11.3 - cheaty z menu
// 0.11.2 - test mirroru
//        - pri putcel(backbuffer <- background) BltFast nahrazen Blt
// 0.11.1 - snad odstranen murray after monstercnt==0 bug
//        - magictime se nasobi 3 sekundami
//        - bublifuky se stridaji v bubleni
// 0.10.3 - bubliny pouzivaji kopii backgroundu v pameti
// 0.10.2 - stav klavesnice se cte primo v update
// 0.10.1 - opraven 0,0<-20,36,30,48 bug
//        - uprava pauzy
// 0.9.8 / darken a upravene popisky levelu na hornim panelu
// 0.9.7 / upraveny red alert
// 0.9.6 - red dragon
//       - score labels
// 0.9.5 - yellow dragon
// 0.9.3 - vypichnuti KeyReflexu a testu konce levelu mimo logickou smycku
//       - murray
//       - statistiky na Tab
// 0.9.2 - odstraneni par bugu
//       - timer a cas v levelu
// 0.9.1 - red alert
//       - vypis autora a jmena levelu
//       - pouziva se gravitace z levelu
// 0.8.5 - pauza
//       - pridany zvuky
// 0.8.4 - symbolove bubliny a bublifuky
// 0.8.3 - energie playera2 se vykresluje zprava
//       - na F1 - F8 se pridavaji a vyskresluji symboly  
// 0.8.2 - vykreslovani energie
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "graphDD.h"
#include "surfaceManager.h"
#include "exceptions.h"
#include "keyboard.h"
#include "r16.h"
#include "tint.h"
#include "font.h"
#include "map.h"
#include "fakedir.h"
#include "sheet.h"
#include "application.h"
#include "utils.h"
#include "wsound.h"
#include "pass.h"
#include "topper.h"
#include "score.h"
#include "game.h"

#define BUB_UNIT     14   // (MAX_SBUBS div 3) + 1

struct Time {
   BYTE min;
   BYTE sec;
};

/*,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
globalni promenne, ktere jsou dostupne i spritum
''''''''''''''''''''''''''''''''''''''''''''''''''''*/

// --- mapy ---
Map            map;
WindMap        windmap;

// --- grafika ---
Surface        m_background;
// kopie pozadi v pamti potae (pokud u tam nen normln pozad)
// pouv se pro rychlej pstup pi generovn bublin

Surface        tiles;
Surface        sym_bubgfx;
Surface        timer10_gfx;
Surface        statbar_gfx;
RGlass         **glasses;
int            glasscount;

// --- sprites ---
DECLARE_ARRAY(Creature*,monsters,monsters_ctrl,MAX_MONSTERS+2)
DECLARE_ARRAY(Bonus*,bonuses,bonuses_ctrl,MAX_BONUSES);
DECLARE_ARRAY(Bubble*,qbubbles,qbubbles_ctrl,MAX_QBUBS);
DECLARE_ARRAY(Bubble*,sbubbles,sbubbles_ctrl,MAX_SBUBS);
DECLARE_ARRAY(SymbolBubble*,symbubs,symbubs_ctrl,MAX_SYMBUBS);
DECLARE_ARRAY(Teleport*,teleports,teleports_ctrl,MAX_TPORTS);
DECLARE_ARRAY(Teleport*,ftports,ftports_ctrl,MAX_FTPORTS);

DropList       *drops;
int            arr_i = 0;
Player         *player1, *player2;
short          monstercnt;
Properties     properties;
int            gravitation;
int            vanish;

// --- options ---
BYTE           startlives;
bool           s_drops;
bool           s_catchy;
bool           s_darken;
DWORD          passcount = 0, 
globalpasscount = 0;

// --- technicke detaily ---
char           *backname;
Message        message;
DWORD          time, oldtime, deltat, oldtime2, starttime;
DWORD          fps, fpsavg;
Message        fpsmsg;
char           filnam[20];
int            picnum = 0;
int            step_global;
unsigned int   oldt;
WORD           timesecond;
int            oddsecond;
int            sk_steps1,sk_steps2,*sk_steps,sk_stepsg;
bool           spadla_klec;

#ifdef MURRAYDEBUG
Message        murraygmsg;
Message        murraybmsg;
#endif

// --- cas ---
DWORD          timelimit;		// limit na level v milisekundach!!!
DWORD          timecurrent;	// ubehly cas v pixelech (pribyva 0->150)
DWORD          timeunit;		// delka casu na jeden pixel (timelimit/150 nebo 10000/150)
int            time_chg;		// priznak, ze se ma prekreslit
int            timer;			// ubihajici cas do konce levelu (postupne klesa)

// --- herni priznaky ---
bool           exitgame;
BYTE           system_frozen;
BYTE           freeze;
DWORD          lastscore1, lastscore2;
BYTE           lives_chg;
char           levelname[20] = "nolevel",*nextlvls;
char           usedname;
int            mirror;
Timer          frozen;
BYTE           diedyoung=0;   //ma se ukazovat a zapisovat do hiscore?
char           s_name[4][32];
BYTE           s_key[4][4];
char           s_password[32];
BYTE           s_vol;
BYTE           s_sfx;
BYTE           s_cd;
bool           s_solidbubbles;
bool           s_usecd;
bool           s_cdorder;
bool           s_turtle;
bool           s_test;
bool           s_testsymbol;
bool           s_testholy;
bool           s_holy;
bool           s_report;
bool           s_cdcheck;
bool           s_changepass;
bool           s_digi;
bool           s_reverse_stereo;
char           s_leveldesc[MAX_TEXTS][30];
bool           ruby;
bool           enteredpass;

#ifdef FILEDEBUG
FILE *fw;
#endif

#define RA_STEP_COUNT      4
#define RA_STEP_SIZE       100
bool is_red_alert;

#define COUNTDOWN_TIME     2700
#define COUNTDOWN_INTERVAL 500
static bool countdown;
static int nextcountdown;

#define TA_FTGS            40
#define TA_DUR             200
#define TA_X               300
#define TA_Y               220
#define TA_XSIZE           40
#define TA_YSIZE           40
Surface timealert_gfx;
int timealert_dur;
RECT timealert_fRect;

//
// Globalni promenne - "lokalni"
//
static Bublifuk *bublifux[MAX_BUBLIFUX];
static int fuqcount;
static int fuq;
static RECT rPlayfield = { PLAYFIELD_MIN_X,PLAYFIELD_MIN_Y, 
PLAYFIELD_MAX_X, PLAYFIELD_MAX_Y };
static DWORD score1_x = 12;
static DWORD score2_x = 548;
static DWORD name2_x;
static DWORD energy1_x = 110;
static DWORD energy2_x = 530;
static DWORD energy_y = 466;
static DWORD lives_x = 304;
static DWORD lives_y = 452;
static DWORD score_y = 463;
static DWORD num_x = 152;
static DWORD desc_x = 200;
static RECT score1_rect = { 12,463,92,479 };
static RECT score2_rect = { 548,463,628,479 };
static RECT lives_rect = { 304,452,336,468 };
static RECT energy0cel_rect = {20,0,24,4};
static RECT energy1cel_rect = {0,0,4,4};
static RECT energy2cel_rect = {4,0,8,4};
static RECT timer_rect = { 450,15,600,25 }; // cela plocha timeru
static Surface ftback[MAX_FTPORTS];
static RECT rFTport = { 0,0,40,40 } ;


//-------------------------------------------------------------------------
// ztmavovani pri pauze
//-------------------------------------------------------------------------
void winquake_dark(Surface dest)
{
   DDSURFACEDESC2 ddsd2;
   WORD *lpSurface;
   DWORD offset;
   long lPitch;
   int shouldbej;
   
   WORD maska = (gBits == 5) ? 0x3def:0x7bef;
     
   lock(dest,NULL,&ddsd2);
   lPitch = ddsd2.lPitch / 2;
   lpSurface = (WORD*) ddsd2.lpSurface;
   for (int i=40; i<440; i++)
   {
      offset = i*lPitch;
      shouldbej = (i & 2) ? 1:0;
      for (int j=0; j < 640; j++, offset++)
      {        
         if ((i & 1) || (j & 1) == shouldbej)
            lpSurface[offset] = (lpSurface[offset] >> 1) & maska;
      }
   }
   
   dest->Unlock(NULL);
}

//-------------------------------------------------------------------------
// zrudnut pozad - red alert
//-------------------------------------------------------------------------
void red_alert(Surface dest)
{
   DDSURFACEDESC2 ddsd2;
   WORD *lpSurface;
   DWORD offset;
   long lPitch;  
   WORD pix;
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   maska = (maska & (gMask | bMask)); // ted bych tam mel mit G a B bez hornich bitu
   
   lock(dest,NULL,&ddsd2);
   lPitch = ddsd2.lPitch / 2;
   lpSurface = (WORD*) ddsd2.lpSurface;
   for (int i=40; i<440; i++)
   {
      offset = i*lPitch;
      for (int j=0; j < 640; j++)
      {        
         pix = lpSurface[offset+j];
         lpSurface[offset+j] = ( (pix & rMask) | ((pix >> 1) & maska) );
      }
   }   
   dest->Unlock(NULL);
}

//-------------------------------------------------------------------------
// ztmaveni pozadi na polovinu, pokud nekomu pripada moc rusive
//-------------------------------------------------------------------------
void darken(Surface dest)
{
   DDSURFACEDESC2 ddsd2;
   WORD *lpSurface;
   DWORD offset;
   long lPitch;  
   WORD maska = (gBits == 5) ? 0x3def : 0x7bef;
   
   lock(dest,NULL,&ddsd2);
   lPitch = ddsd2.lPitch / 2;
   lpSurface = (WORD*) ddsd2.lpSurface;
   for (int i=40; i<440; i++)
   {
      offset = i*lPitch;
      for (int j=0; j < 640; j++)
      {        
         lpSurface[offset+j] = ((lpSurface[offset+j] >> 1) & maska) ;
      }
   }
   
   dest->Unlock(NULL);
}

//-------------------------------------------------------------------------
// tint pro statistiky
//-------------------------------------------------------------------------
void statsTint(Surface dest)
{
   DDSURFACEDESC2 ddsd2;
   WORD *lpSurface;
   DWORD offset;
   long lPitch;
   WORD g50 = (gBits == 5) ? 0x01E0:0x03E0;
   WORD b50 = 0x000F;
   WORD maska = (gBits == 5) ? 0x3def:0x7bef;
   int i,j;
   
   lock(dest,NULL,&ddsd2);
   lPitch = ddsd2.lPitch / 2;
   lpSurface = (WORD*) ddsd2.lpSurface;
   for (i=40; i<440; i++)
   {
      offset = i*lPitch;
      for (j=0; j < 320; j++, offset++)
      { 
         lpSurface[offset] =  g50 + ((lpSurface[offset] >> 1) & maska);
      }
      for (j=320; j < 640; j++, offset++)
      { 
         lpSurface[offset] =  b50 + ((lpSurface[offset] >> 1) & maska);
      }
   }
   
   dest->Unlock(NULL);
}

//-------------------------------------------------------------------------
// natazeni grafiky
//-------------------------------------------------------------------------
void gfx_load()
{
   //------------------------------------------ nahrani vlastnosti a grafiky
   loadFonts();
   properties.load("data\\sprites.def");
   surfaceManager->load("drops.r16","data\\pictures.dir",true);
   tiles = surfaceManager->load("tiles.r16","data\\pictures.dir",true);
   sym_bubgfx = surfaceManager->load("bubsymbols.r16","data\\pictures.dir",true);
   timer10_gfx = surfaceManager->load("timer.r16","data\\pictures.dir",true);
   timealert_gfx = surfaceManager->load("hurry.r16","data\\pictures.dir",true);
   statbar_gfx = surfaceManager->load("statbar.r16","data\\pictures.dir",true);
}
   
//=========================================================================
// inicializace systemu
//=========================================================================
Game::Game(void)
{
   int i;

   background = NULL;
	background_normal = NULL;
	background_redalert = NULL;
   glasses = NULL;
   map = NULL;
   windmap = NULL;
   UFOcnt = 0;
   nextlvls = new char[128];

   srand(GetTickCount());
   
   //zjisteni prvniho volneho shotu
   i = 1;
   FILE *ftest;
   while(1)
   {
      sprintf(filnam,"bshot%03d.bmp",i);
      ftest = fopen(filnam,"rb");
      if (ftest == NULL)
      {
         break;
      }
      fclose(ftest);
      i++;
   }
   picnum = i;
   
   exitsys = false;
   exitgame = false;
   system_frozen = 0;
   freeze = 0;
   frozen.options(7500,0,30,true,false);
   backname = new char[30];
   if (backname == NULL)
      throw new BubException("Game::Game: cannot allocate background!");
   
   for(i=0; i<MAX_TEXTS; i++)
      s_leveldesc[i][0] = '\x0';
   
   s_usecd = false;
}

//-------------------------------------------------------------------------
// nahrava data pro hru
//-------------------------------------------------------------------------
void Game::load()
{
   //-------------------------------------------------------- nahrani bublin
   if (!RGlass::LoadGlasses("data\\bubbles.bbl", glasses, glasscount))
   {
      throw new BubException("Game(): failed to load bubbles!");
   }
   //----------------------------------------------------------------- kapky
}


//==========================================================================
//    Nacpe na naloadovane pozadi (_normal) plosinky a teleporty
//==========================================================================
void Game::PrepareBackground()
{
	int i = 0;
	Teleport *pomt;

	// pokud se m ztmavovat, tak ztmavm pozad
	if (s_darken)
   {
     darken(background_normal);
   }

	// nasazim tam plosinky...
   draw_map2(map,tiles,background_normal);

	//...a teleporty
   I while(i<teleports_ctrl.count)
   {
      sprintf(pom_string,"*tport%d",i);
      ARRAY_AT(teleports,i)->prepare(background_normal,pom_string);
      sprintf(pom_string,"*tportd%d",i);
      ARRAY_AT(teleports,i++)->prepareDest(background_normal,pom_string);
   }
   I while(i<ftports_ctrl.count)
   {
      sprintf(pom_string,"*ftport%d",i);
      pomt = ARRAY_AT(ftports,i);
      pomt->prepare(background_normal,pom_string);
      pomt->Paint(background_normal);
      i++;
   }
}

//==========================================================================
//    Zahajeni hry, nastaveni prostredi
//==========================================================================
void Game::StartGame()
{
   startlives = DEF_LIVES;
   enteredpass = true;
   if (strlen(s_password)>11)
      s_password[11]=0;
   if (*s_password=='/')
   {
      usedname=2;
      strcpy(levelname,s_password+1);
      enteredpass = false;
   }
   else
   {
      usedname=0;
      strupr(s_password);
      decode(s_password,levelname,&startlives);
   }
   if (!(*levelname))
   {
      strcpy(levelname,"start");
   }
   
   exitgame = false;
   player1 = new Player(properties.member[ID_PLAYER1],&qfont_g);
   player1->SetKeys(s_key[0][0],s_key[0][1],s_key[0][2],s_key[0][3]);
   player1->sheet.lives = startlives;
   player1->sheet.name = s_name[0];  
   player1->sheet.scorefont = &scorefont_g;
   player1->gravity = DFT_GRAVITY;
   player1->dropcolor = DROP_GREEN;
   player1->symbuf.x = 200;
   
   player2 = new Player(properties.member[ID_PLAYER2],&qfont_b);
   player2->SetKeys(s_key[1][0],s_key[1][1],s_key[1][2],s_key[1][3]);
   player2->sheet.name = s_name[1];  
   player2->sheet.scorefont = &scorefont_b;
   player2->gravity = DFT_GRAVITY;
   player2->dropcolor = DROP_BLUE;
   player2->symbuf.x = 360;
   
   if (s_testholy)
   {
      player1->state |= IMMORTAL;
      player2->state |= IMMORTAL;
   }   

   drops = new DropList();  

#ifdef MURRAYDEBUG
   strcpy(murrayg_msg,"MG:");
   strcpy(murrayb_msg,"MB:");
#endif
}

//==========================================================================
//    Nahrani a inicializace levelu
//==========================================================================
int Game::LoadLevel(void)
{
   FILE *fr;
   char s[40];
   char mapname[24];
   char wndname[24];
   char path[10]="levels\\";
   int id,x,y,tox,toy;
   int j,count;
   Monster *pom;
   Teleport *pomt;
   Bonus * pomb;
   
   DEBUG_MESSAGE("LoadLevel:");
   
   for (j=0;j<MAX_BUBLIFUX;j++)
      bublifux[j] = NULL;
   
   DEBUG_MESSAGE("b");
   
   // nejsou nahodou prohozene klavesy?
   if (player1->dik_left != s_key[0][0])
   {
      player1->SetKeys(s_key[0][0],s_key[0][1],s_key[0][2],s_key[0][3]);
      player2->SetKeys(s_key[1][0],s_key[1][1],s_key[1][2],s_key[1][3]);
   }
   
   DEBUG_MESSAGE("k");
   
   //---------------------------------------------- otevreni souboru s levelem
   //bud je dobre heslo, nebo jde do start
   do
   {
      strcpy(s,path);
      strcat(s,levelname);
      strcat(s,".lvl");
      fr = fopen(s,"rt");
      if (fr == NULL)
      {
         strcpy(s,levelname);
         strcat(s,".lvl");
         strcpy(exc_string,s);
         fr = fakeopen("data\\levels.dir",s,"rt");
      }
      if (fr == NULL)
      {
         if (!usedname)
         {
            player1->sheet.lives=DEF_LIVES;
            strcpy(levelname,"start"); //jeste muzem zkusit 'start'
            usedname++;
         }
         else
         {
            levelname[0] = '\x0';
            return 0;
         }
      }
   } 
      
   while (fr==NULL && usedname<2);
   usedname = 2; //jistota pro dalsi cyklus
   // tak co? klaplo otevreni souboru?
   if (fr == NULL)
   {
      diedyoung = 0;
      throw new BubException("loadlevel");
   }
   //- ctu z LVL -------------------------------------------------------------
   fscanf(fr,"%s",s);
   if (strcmp(s,"LVL"))
   {
      fclose(fr);
      throw new BubException("loadlevel");
   }

   DEBUG_MESSAGE("1");
   
   //---------------------------------------------------- nacteni jmena pozadi
   fscanf(fr,"%s",s);
   strcpy(backname,s);
   strcat(backname,".r16");
   
   DEBUG_MESSAGE("B");

   
   //------------------------------------------------------------ nacteni mapy
   fscanf(fr,"%s",s);
   s[8] = 0;
   strcpy(mapname,path);
   strcat(mapname,s);
   strcat(mapname,".emb");
   map = load_map(mapname);
   if (map==NULL)
   {
      fclose(fr);
      sprintf(exc_string,"loadlevel: load(%s) failed!",mapname);
      throw new BubException(exc_string);
   }

   DEBUG_MESSAGE("M");
   
   //----------------------------------------------------- nacteni vetrne mapy
   fscanf(fr,"%s",s);
   s[8] = 0;
   strcpy(wndname,path);
   strcat(wndname,s);
   strcat(wndname,".wnd");
   windmap = load_wndmap(wndname);
   if (windmap==NULL)
   {
      fclose(fr);
      sprintf(exc_string,"loadlevel: load(%s) failed!",wndname);
      throw new BubException(exc_string);
   }
   fscanf(fr,"%s",s);

   DEBUG_MESSAGE("W");
   
   //--------------------------------------------------------- nacteni playeru
   fscanf(fr,"%s",s);
   fscanf(fr,"%d %d %d",&id,&x,&y);
   
   ARRAY_ADD(monsters,monsters_ctrl,player1);
   
	// V. 2004 - predelano na metodu
   player1->Place(x,y);
	//
   
   fscanf(fr,"%s",s);
   fscanf(fr,"%d %d %d",&id,&x,&y);
   
   ARRAY_ADD(monsters,monsters_ctrl,player2);

	// V. 2004 - predelano na metodu
   player2->Place(x,y);
	//
   
   DEBUG_MESSAGE("P");

   //--------------------------------------------------------- nacteni monster
   fscanf(fr,"%s",s);
   fscanf(fr,"%d",&count);
   int turnpos = 0;
   monstercnt = count;
   if (count>0)
   {
      for(j=0;j<count;j++)
      {
         fscanf(fr,"%s",s);
         fscanf(fr,"%d %d %d",&id,&x,&y);
         x *= MAP_CELX;
         y = y * MAP_CELY + PLAYFIELD_TOP;       
         pom = NULL;
         if (id>=40 && id < 48)
         {
            monstercnt--;
            pomb = new Bonus(x,y,properties.member[id]);
            pomb->type = BONUS;
            pomb->action = 0;
            pomb->movex = 0;
            pomb->movey = 0;
            
            if (ARRAY_FULL(bonuses_ctrl))
            {
               delete pomb;
               pomb = NULL;
            }
            else
            {
               if (pomb != NULL)
                  ARRAY_ADD(bonuses,bonuses_ctrl,pomb);
            }
         }
         else
         {
            switch (id) 
            {
            case 4:
               pom = new Zomb(x,y,properties.member[id]);
               break;
            case 5:
               pom = new Nebulus(x,y,properties.member[id]);
               break;
            case 6:
               pom = new Butterfly2(x,y,properties.member[id]);
               break;
            case 7:
               pom = new Butterfly(x,y,properties.member[id]);
               break;
            case 8:
               pom = new Kachna(x,y,properties.member[id]);
               break;
            case 9:
               pom = new Butterfly3(x,y,properties.member[id]);
               break;
            case 10:
               if (!turnpos) 
                  turnpos = countturnpos(map);
               pom = new RedDragon(x,y,properties.member[id],turnpos);
               break;
            case 11:
               pom = new YellowDragon(x,y,properties.member[id]);
               break;
            case 12:
               pom = new Bunny(x,y,properties.member[12]);
               break;
            case 13:
               pom = new Zomb(x,y,properties.member[13]);
               break;            
            }
            if (ARRAY_FULL(monsters_ctrl))
            {
               delete pom;
               pom = NULL;
            }
            else
            {
               if (pom != NULL)
                  ARRAY_ADD(monsters,monsters_ctrl,pom);
            }
         }
      }
   }

   DEBUG_MESSAGE("M");
     
   //------------------------------------------------------- nacteni bublifuku
   
   fscanf(fr,"%s",s);
   fscanf(fr,"%d",&count);
   J while(j<count)
   {
      fscanf(fr,"%s",s);
      fscanf(fr,"%d %d ",&x,&y);
      x *= MAP_CELX;
      y = y * MAP_CELY + PLAYFIELD_TOP;
      if (j<MAX_BUBLIFUX)
      {
         bublifux[j] = new Bublifuk(x,y,properties.member[ID_BFUK]);
      }
      j++;
   }
   fuqcount = count;
   if (fuqcount > MAX_BUBLIFUX)
      fuqcount = MAX_BUBLIFUX;

   DEBUG_MESSAGE("B");
   
   //------------------------------------------------------- nacteni teleportu
   fscanf(fr,"%s",s);
   fscanf(fr,"%d",&count);
   if (count>0)
   {
      for(j=0;j<count;j++)
      {
         fscanf(fr,"%s",s);
         fscanf(fr,"%d %d %d %d",&x,&y,&tox,&toy);
         x *= MAP_CELX;
         y = y*MAP_CELY + PLAYFIELD_TOP;
         pomt = new Teleport(x,y,properties.member[ID_TPORT]);
         pomt->destx = tox * MAP_CELX;
         pomt->desty = toy * MAP_CELY + PLAYFIELD_TOP;
         if (ARRAY_FULL(teleports_ctrl))
         {
            delete pomt;
            pomt = NULL;
         }
         else
         {
            if (pomt != NULL)
               ARRAY_ADD(teleports,teleports_ctrl,pomt);
         }
      }
   }
   DEBUG_MESSAGE("T");
      
   //------------------------------------------- nacteni zaverecnych teleportu
   fscanf(fr,"%s",s);
   fscanf(fr,"%s",s);
   strcpy(nextlvls,s);
   fscanf(fr,"%d",&count);
   if (count>0)
   {
      for(j=0;j<count;j++)
      {
         fscanf(fr,"%s",s);
         fscanf(fr,"%d %d",&x,&y);
         fscanf(fr,"%s",s);
         strcpy(nextlvls+(j+1)*9,s);
         x *= MAP_CELX;
         y = y*MAP_CELY + PLAYFIELD_TOP;          
         pomt = new Teleport(x,y,properties.member[ID_FTPORT]);
         pomt->type = FTPORT;
         pomt->destx = j+1;
         if (ARRAY_FULL(ftports_ctrl))
         {
            delete pomt;
            pomt = NULL;
            app->sayConsole("2 many ftports... deleted");
         }
         else
         {
            if (pomt != NULL)
               ARRAY_ADD(ftports,ftports_ctrl,pomt);
         }
      }
   }
   DEBUG_MESSAGE("F");
      
   //-------------------------------------------------------------- level info
   fscanf(fr,"%s",s);    // vata
   char tclr;
   fscanf(fr,"%d %d",&tclr,&count); // barva a pocet
   if (count>0)
   {
      for(j=0;j<MAX_TEXTS;j++)
         s_leveldesc[j][0] = '\x0';
      for(j=0;j<count;j++)
      {
         fscanf(fr,"%s",s);//vata
         fscanf(fr,"%s",s);//x
         fscanf(fr,"%s",s);//y
         fgets(s,10,fr); //sezere enter
         fgets(s,30,fr); //fscanf(fr,"%s",s);//hura, text!
         if (*s) s[strlen(s)-1]=0; //odseknu crlf na konci
         if (j >= MAX_TEXTS)
            throw new BubException("Level with 2much texts");
         strcpy(s_leveldesc[j],s);
      }
   }
   DEBUG_MESSAGE("I");

   //--------------------------------------------------------------- gravitace
   fscanf(fr,"%s",s);    // vata
   j=fscanf(fr,"%d",&gravitation);
   if (j!=0 && j!=EOF)
   {
      for (j=0; j<monsters_ctrl.count; j++)
         ARRAY_AT(monsters,j)->gravity = gravitation * 2;
      player1->gravityLow = gravitation * 2;
      player1->gravityHigh = gravitation * 4;
      player2->gravityLow = gravitation * 2;
      player2->gravityHigh = gravitation * 4;
   }
   else 
   {
      gravitation=DFT_GRAVITY;
      player1->gravityLow = gravitation * 2;
      player1->gravityHigh = gravitation * 4;
      player2->gravityLow = gravitation * 2;
      player2->gravityHigh = gravitation * 4;
   }
   //--------------------------------------------------------------- timelimit
   fscanf(fr,"%s",s);    // vata timelimit
   j=fscanf(fr,"%d",&timelimit);
   timelimit *= 1000;
   if (j==0) 
      timelimit = DFT_TIMELIMIT;
   timeunit = timelimit / 150;   //sirka timeru
   timer = timelimit;				// vychozi cas na dokonceni levelu
   //------------------------------------------------------------------- kanec
   fscanf(fr,"%s",s);    // vata magic
   j=fscanf(fr,"%d",&magictime);
   if (j==0) magictime = 0;
   magictime = magictime * 1500;
   //------------------------------------------------------------------- kanec
   fclose(fr);
   DEBUG_MESSAGE(".Done\n");
   return 1;
}

//-------------------------------------------------------------------------
// start levelu
//-------------------------------------------------------------------------
void Game::startLevel()
{
   //Teleport *pomt;
   //int i;

   DEBUG_MESSAGE("startLevel...");
   if (!LoadLevel())
      return;

	// zlikviduju star pozad v pamti
	// (je zvltn, e i _redalert je teba mt tady - jinak to bugovalo)
   surfaceManager->remove(background, true);
   surfaceManager->remove(background_normal, true);
	surfaceManager->remove(background_redalert, true);

	// pripravim si pozad
   background_normal = surfaceManager->load(backname,"data\\pictures.dir",true);
	// V. 2004
	PrepareBackground();
	//

	// background je nejdriv normalni
	background = background_normal;

	// pripraveni bonusu a casomiry
   frozen.set(0);
   ruby = (monstercnt > 0);
   fuq = 0;
   to_fuq = 0;
   vanish = 0;
   countdown = false;
   nextcountdown = COUNTDOWN_TIME;
   timealert_dur = 0;
   timealert_fRect.left = 0;
   timealert_fRect.right = TA_XSIZE;
   timealert_fRect.top = 0;
   timealert_fRect.bottom = TA_YSIZE;
   DEBUG_MESSAGE(".Done.\n");
}

      
//-------------------------------------------------------------------------
// posledni pripravy pred startem...
//-------------------------------------------------------------------------
void Game::start()
{
   DDSCAPS2 ddsCaps2;

   DEBUG_MESSAGE("Game::start()");

	// teprve tady je vhodne misto pripravit si i kopii pozadi pro redalert
	// ve startLevel by to jet mohl zniit alt-tab v interscreenu
	// V. 2004
   //surfaceManager->remove(background_redalert, true);
	// tohle tu z nejasnch pin nesm bt..?! tak je to ve startLevel
	background_redalert = surfaceManager->create(640,480,"*bckg_r");
   putcel(0,0,background_redalert,rFullScreen,background_normal);
	red_alert(background_redalert);
	//

	// je teba si pipravit pracovn pozad pro generovn bublin
	// mlo by bt v RAMce, take pokud tam nen, tak je teba kopii
	// (pro jistotu upraveno na _normal) V. 2004
	ddResult = background_normal->GetCaps(&ddsCaps2);
   if (ddResult != DD_OK)
      throw new DDException("Game::startLevel: GetCaps failed!",ddResult);

   if (ddsCaps2.dwCaps & DDSCAPS_SYSTEMMEMORY)
   {
      DEBUG_MESSAGE("Background:SYSTEM\n");
      m_background = background_normal;
   }
   else
   {
      DEBUG_MESSAGE("Background:VIDEO\n");
      m_background = surfaceManager->create(640,480,"*bckg_m",
      DDSCAPS_SYSTEMMEMORY | DDSCAPS_OFFSCREENPLAIN);
      putcelBlt(0,0,m_background,rFullScreen,background_normal);
   }
	//

   mirror = 0;
   magiccount = 0;
   //  message.duration = 0;
     
   // dolni panel - jmena hracu a zivoty
   name2_x = score2_rect.right - strlen(player2->sheet.name) * qfont_b.xsize;
   putString(score1_x,446,background,player1->sheet.name,qfont_g);
   putString(name2_x,446,background,player2->sheet.name,qfont_b);
   DEBUG_MESSAGE(" 1");
   
   // horni panel - cislo levelu, nazev a autor
   char * desc = s_leveldesc[0]+1;
   while (*desc!='.' && (desc-s_leveldesc[0])<4) desc++;
   if (*desc=='.')
   { // je tam nejake cislo, tak ho odrizneme a vypiseme zvlast
     *desc = 0;
     desc++; if (*desc == ' ') desc++;
     putString(num_x+(3-strlen(s_leveldesc[0]))*12,4,background,s_leveldesc[0],qfont_r);
   }
   else
   {  // cislo nikde - jenom nazev 
      desc = s_leveldesc[0];
   }
   putString(desc_x,4,background,desc,qfont_w);
   if (desc != s_leveldesc[0])
      s_leveldesc[0][strlen(s_leveldesc[0])]= '.';
   putString(desc_x,21,background,s_leveldesc[1],qfont_y);
   DEBUG_MESSAGE("2");
   
   putcel(0,0,backbuffer,rFullScreen,background);
   flip(0);
   putcel(0,0,backbuffer,rFullScreen,background);
   DEBUG_MESSAGE("3");
   
   player1->Resurrection();
   player2->Resurrection();
   DEBUG_MESSAGE("4");
      
   player1->sheet.score_changed = 2;
   player1->sheet.energy_changed = 2;
   player1->symbuf.changed = 2;
   player2->sheet.score_changed = 2;
   player2->sheet.energy_changed = 2;
   player2->symbuf.changed = 2;
   lives_chg = 2;
   
   time = 0;
   time_chg = 0;
   
   sound_device->Play(14);
   time = GetTickCount();
   exitgame = 0;
   spadla_klec = false;
   is_red_alert = false;
   last_alert = -9999; // kvuli sireny v bonusovych levelech
   panpos = 240;
   oldtime = time;
   oldtime2 = time;
   timecurrent = 0;
   starttime = time;
   step_global = 0;
   CLEAR_USEDMAP;
   FLIP_USEDMAP;
   CLEAR_USEDMAP;
   fpsmsg.Options(10,40,&sfont_w,"*fpsmsg",IND_LEFT);
   DEBUG_MESSAGE("5");
#ifdef MURRAYDEBUG
   murraygmsg.Options(10,56,&sfont_w,"*mgmsg",IND_LEFT);
   murraybmsg.Options(630,56,&sfont_w,"*mbmsg",IND_RIGHT);
#endif
   DEBUG_MESSAGE("6");
   passcount = 0;
   globalpasscount = 0;
   DEBUG_MESSAGE(".Done.\n");
}

//==========================================================================
// jeden prubeh hlavni smycky
//==========================================================================
void Game::update(void)
{
   int i,j,k;
   Bub *pom_bub;

#ifdef PROFILER_DEBUG
   profiler->startTimer(0);
#endif
#ifdef PROFILER_DEBUG
   profiler->startTimer(11);
#endif

	// tady bylo postupne spusteni redalertu po obrazovce
	// stejn nefungovalo - zrueno!
	// V. 2004

   if (!system_frozen)
   {
      if (vanish > 0)
         vanish--;
      //----------------------------------------------------------------- PAINT
      // teleporty
      I while(i<teleports_ctrl.count)
         ARRAY_AT(teleports,i++)->Paint(backbuffer);
      // bonusy
      I while(i<bonuses_ctrl.count)
         ARRAY_AT(bonuses,i++)->Paint(backbuffer);
      // potfurky
      I while(i<monsters_ctrl.count)
         ARRAY_AT(monsters,i++)->Paint(backbuffer);
      // kreslime rychle bubliny
      I while(i<qbubbles_ctrl.count)
         ARRAY_AT(qbubbles,i++)->Paint(backbuffer);
      // kreslime pomale bubliny
      I while(i<sbubbles_ctrl.count)
      {
         pom_bub = ARRAY_AT(sbubbles,i);
         J while(j < monsters_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(monsters,j++));
         J while(j < bonuses_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(bonuses,j++));
         J while(j < teleports_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(teleports,j++));
         pom_bub->Paint(backbuffer);
         pom_bub->monsterbehind = false;
         i++;
      }
      // kreslime symbolove bubliny
      I while(i<symbubs_ctrl.count)
      {
         pom_bub = ARRAY_AT(symbubs,i);
         J while(j < monsters_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(monsters,j++));
         J while(j < bonuses_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(bonuses,j++));
         J while(j < teleports_ctrl.count)
            pom_bub->checkBehind(ARRAY_AT(teleports,j++));
         pom_bub->Paint(backbuffer);
         pom_bub->monsterbehind = false;
         i++;
      }
      // kreslime bublifuky
      I while(i < fuqcount)
         bublifux[i++]->Paint(backbuffer);
      // kreslime UFOny
      I while(i<UFOcnt)
         UFOs[i++]->Paint(backbuffer);
      // kreslime bonbony =)
      I while(i<drops->count)
         drops->list[i++]->Paint(backbuffer);
      // a scorelabels
      paint_score_labels(backbuffer);

      // kresleni hodin - hurry up!
      if (timealert_dur > 0)
      {
         if (timealert_dur < 4*TA_FTGS && timealert_fRect.left == 0)
         {
            timealert_fRect.left += TA_XSIZE;
            timealert_fRect.right += TA_XSIZE;
         }
         else if (timealert_dur < 3*TA_FTGS && timealert_fRect.left <= TA_XSIZE)
         {
            timealert_fRect.left += TA_XSIZE;
            timealert_fRect.right += TA_XSIZE;
         }
         else if (timealert_dur < 2*TA_FTGS && timealert_fRect.left <= 2*TA_XSIZE)
         {
            timealert_fRect.left += TA_XSIZE;
            timealert_fRect.right += TA_XSIZE;
         }
         if (timealert_dur > TA_FTGS)
         {
            putcel(TA_X,TA_Y,backbuffer,timealert_fRect,timealert_gfx,
               DDBLTFAST_SRCCOLORKEY);
         }
         else
         {  
            puttransparent(TA_X,TA_Y,backbuffer,timealert_fRect,timealert_gfx);
         }
      }


#ifdef PROFILER_DEBUG
   profiler->stopTimer(11);
#endif      
      message.Paint();
      fpsmsg.Paint();
#ifdef MURRAYDEBUG
      murraygmsg.Set(murrayg_msg,&sfont_w);
      murraybmsg.Set(murrayb_msg,&sfont_w);
      murraygmsg.Paint();
      murraybmsg.Paint();
#endif      
      if (lives_chg > 0)
      {
         putcel(lives_x,lives_y,backbuffer,lives_rect,background,DDBLTFAST_NOCOLORKEY);
         numtos(player1->sheet.lives,2,pom_string);
         putStringCentered(320,lives_y,backbuffer,
            (pom_string[0]!='0')?pom_string:(pom_string+1),qfont_r);
         lives_chg--;
      }
      if (time_chg > 0)
      {
			// doslo ke zmene casomiry - je treba prekreslit
			timer_rect.right = timecurrent + 450;
			clear(backbuffer,0,&timer_rect);
			time_chg--;
      }
      if (player1->sheet.score_changed > 0)
      {
         putcel(score1_x,score_y,backbuffer,score1_rect,background,DDBLTFAST_NOCOLORKEY);
         numtos(player1->sheet.score,8,pom_string);
         putString(score1_x,score_y,backbuffer,pom_string,scorefont_g);
         player1->sheet.score_changed--;
      }
      Surface dgfx = drops->gfx;
      if (player1->sheet.energy_changed > 0)
      {
         I while (i < player1->sheet.energy)
         {
            putcel(energy1_x+i*(DROP_SIZE+1),energy_y,backbuffer,energy1cel_rect,dgfx);
            i++;
         }
         while (i < player1->sheet.maxenergy)
         {
            putcel(energy1_x+i*(DROP_SIZE+1),energy_y,backbuffer,energy0cel_rect,dgfx);
            i++;
         }
         player1->sheet.energy_changed--;
      }
      if (player1->symbuf.changed > 0 || player1->symbuf.flash)
      {
         player1->symbuf.Paint(backbuffer);
         if (player1->symbuf.changed > 0) player1->symbuf.changed--;
      }
      
      if (player2->sheet.score_changed > 0)
      {
         putcel(score2_x,score_y,backbuffer,score2_rect,background,DDBLTFAST_NOCOLORKEY);
         numtos(player2->sheet.score,8,pom_string);
         putString(score2_x,score_y,backbuffer,pom_string,scorefont_b);
         player2->sheet.score_changed--;
      }
      if (player2->sheet.energy_changed > 0)
      {
         I while (i < player2->sheet.energy)
         {
            putcel(energy2_x-i*(DROP_SIZE+1),energy_y,backbuffer,energy2cel_rect,dgfx);
            i++;
         }
         while (i < player2->sheet.maxenergy)
         {
            putcel(energy2_x-i*(DROP_SIZE+1),energy_y,backbuffer,energy0cel_rect,dgfx);
            i++;
         }
         player2->sheet.energy_changed--;
      }
      if (player2->symbuf.changed > 0 || player2->symbuf.flash)
      {
         player2->symbuf.Paint(backbuffer);
         if (player2->symbuf.changed > 0) player2->symbuf.changed--;
      }
   }


   //------------------------------------------------------- klavesnice
   kbd.getState();

   //------------------------------------------------------ spadla klec
   if (spadla_klec)
   {     
      BYTE key;
      winquake_dark(backbuffer);
      putStringCentered(320,230,backbuffer,"A spadla klec!",qfont_w);
      if ((key = kbd.LastScan()) != 0 
         && key != DIK_LMENU && key != DIK_RMENU && key != DIK_TAB)
      {
         kbd.clearKey(key);
         app->stateChange();
         kbd.filter_lastscan = true;
         kbd.lastASCIIReset();
         kbd.lastScanReset();
         return;
      }
      flip(0);
      putcel(0,PLAYFIELD_TOP,backbuffer,rPlayfield,background);
      return;
   }

   //-------------------------------------------------- statistiky
   if (!system_frozen && freeze & SF_STATS)
   {
      statsTint(backbuffer);
      player1->Stats(20);
      player2->Stats(340);
   }
   //-------------------------------------------------- pozastaveni hry
   if (!system_frozen && freeze & SF_PAUSED)
   {
      winquake_dark(backbuffer);
   }
   //-------------------------------------------------- help
   if (!system_frozen && freeze & SF_HELP)
   {
      winquake_dark(backbuffer);
      putString(20,22,backbuffer,"Npovda",qfont_r);
      putString(20,52,backbuffer,"Klvesy",qfont_y);
      putString(20,82,backbuffer,"F1   tato obrazovka",qfont_w);
      putString(20,102,backbuffer,"F9   screenshot",qfont_w);
      putString(20,142,backbuffer,"Tab  statistiky hr",qfont_w);
   }

   system_frozen ^= freeze;
   freeze = 0;
   //testovaci figliky / debugovani /cheaty
   if (s_testsymbol)
   {
      Player *player;
      //skladani kouzel ze symbolu
      for (j=0; j<8; j++)     //projedou se klavesy F1 az F8
      {
         if (KEYDOWN(kbd.diKeys,DIK_F1+j))
         {
            kbd.clearKey(DIK_F1+j);
            if (j<4) 
               player=player1;
            else 
               player=player2;
            if (!(player->state & NOCREDIT))
            { //musim testnout, jestli je drak ve hre
               k = j & 3;
               player->symbuf.Add((BYTE)k);
               player->symbuf.Realize(player);
            }
         }
      }
      if (KEYDOWN(kbd.diKeys,DIK_F10))
      {
         I while (i < sbubbles_ctrl.count)
         {
            ARRAY_AT(sbubbles,i)->Blow(NULL);
            i++;
         }
      }
   }
   else
   {
      //pokud necituju, je na F1 help
      if (!(system_frozen & ~(SF_HELP)) && KEYDOWN(kbd.diKeys,DIK_F1))
      {
         sound_device->Play(14);
         freeze = SF_HELP;
         if (!(system_frozen & SF_HELP))
         {
            winquake_dark(backbuffer);
            putString(20,22,backbuffer,"Npovda",qfont_r);
            putString(20,52,backbuffer,"Klvesy",qfont_y);
            putString(20,82,backbuffer,"F1   tato obrazovka",qfont_w);
            putString(20,102,backbuffer,"F9   screenshot",qfont_w);
            putString(20,142,backbuffer,"Tab  statistiky hr",qfont_w);
         }
         else
         {
            FILL_USEDMAP_PLAYFIELD(usedmap1);
            FILL_USEDMAP_PLAYFIELD(usedmap2);
         }
         kbd.clearKey(DIK_F1);
         time = GetTickCount();
         oldtime = time;
         oldtime2 = time;
         starttime = time - starttime;        // 1: st = y - st 0: st = z-(y-st) = st+z-y
      }
   }
   if (system_frozen > 0 && KEYDOWN(kbd.diKeys,DIK_ESCAPE))
   {
      sound_device->Play(14);
      FILL_USEDMAP_PLAYFIELD(usedmap1);
      FILL_USEDMAP_PLAYFIELD(usedmap2);
      system_frozen = 0;
      kbd.clearKey(DIK_ESCAPE);
      time = GetTickCount();
      oldtime = time;
      oldtime2 = time;
      starttime = time - starttime;        // 1: st = y - st 0: st = z-(y-st) = st+z-y
   }
   if (!(system_frozen & ~(SF_PAUSED)) && KEYDOWN(kbd.diKeys,DIK_P))
   {
      sound_device->Play(14);
      freeze = SF_PAUSED;
      if (!(system_frozen & SF_PAUSED))
      {
         winquake_dark(backbuffer);
         putStringCentered(320,230,backbuffer,"Pauza!",qfont_w);
      }
      else
      {
         FILL_USEDMAP_PLAYFIELD(usedmap1);
         FILL_USEDMAP_PLAYFIELD(usedmap2);
      }
      kbd.clearKey(DIK_P);
      time = GetTickCount();
      oldtime = time;
      oldtime2 = time;
      starttime = time - starttime;        // 1: st = y - st 0: st = z-(y-st) = st+z-y
   }
   if (!(system_frozen & ~(SF_STATS)) && KEYDOWN(kbd.diKeys,DIK_TAB) 
      && !KEYDOWN(kbd.diKeys,DIK_LALT) && !KEYDOWN(kbd.diKeys,DIK_RALT))
   {
      sound_device->Play(14);
      freeze = SF_STATS;
      if (!(system_frozen & SF_STATS))
      {
         statsTint(backbuffer);
         player1->Stats(20);
         player2->Stats(340);
      }
      else
      {
         FILL_USEDMAP_PLAYFIELD(usedmap1);
         FILL_USEDMAP_PLAYFIELD(usedmap2);
      }
      kbd.clearKey(DIK_TAB);
      time = GetTickCount();
      oldtime = time;
      oldtime2 = time;
      starttime = time - starttime;        // 1: st = y - st 0: st = z-(y-st) = st+z-y
   }

   //------------------------------------------------------------------ FLIP
   if (mirror > 0)
   {
      DDBLTFX bltFX;
      ZeroMemory(&bltFX,sizeof(bltFX));
      bltFX.dwSize = sizeof(bltFX);
      bltFX.dwDDFX = DDBLTFX_MIRRORUPDOWN;
      putcelBlt(0,0,frontbuffer,rFullScreen,backbuffer,DDBLT_DDFX,&bltFX);
   }
   else
   {
      flip(0);
   }
   if (ddc1.dwVidMemTotal > 1500000)
   {
      FLIP_USEDMAP;
   }
   
   if (system_frozen)
   {
      return;
   }

   //------------------------------------------------------------ screenshot
   if (KEYDOWN(kbd.diKeys,DIK_F9))
   {
      kbd.clearKey(DIK_F9);
      sound_device->Play(14);
      filnam[5]=(picnum / 100)+'0';
      filnam[6]=(picnum % 100)/10+'0';
      filnam[7]=(picnum % 10)+'0';
      screenshot(backbuffer,filnam);
      picnum++;
      message.Set("Screenshot sest\251elen...",&sfont_w);
   }

   //----------------------------------------------------------------- CLEAR
   if (ddc1.dwVidMemTotal > 1500000)
   {
      RECT umr;
      for (i=0;i<10;i++)
      {
         for(j=0;j<12;j++)
         {
            if ((*usedmap)[i][j])
            {
               umr.left = i << 6;
               umr.right = umr.left + 64;
               umr.top = j * 40;
               umr.bottom = umr.top + 40;
               putcel(umr.left,umr.top,backbuffer,umr,background,DDBLTFAST_NOCOLORKEY);
            }
         }
      }
      CLEAR_USEDMAP;
   }
   else
   {
      putcel(0,PLAYFIELD_TOP,backbuffer,rPlayfield,background,DDBLTFAST_NOCOLORKEY);
   }
   
   //------------------------------------------------------------------ MISC
   if (KEYDOWN(kbd.diKeys,DIK_ESCAPE))
   {
      sound_device->Play(14);
      exitgame=true;
      diedyoung=0;
      app->stateChange(REASON_NOTOPPERS);
      return;
   }
       
   if (freeze)
      return;

#ifdef PROFILER_DEBUG
   profiler->stopTimer(0);
#endif

#ifdef PROFILER_DEBUG
   profiler->startTimer(1);
#endif

   //------------------------------------------------------------------ TIME
   passcount++;
   globalpasscount++;
   time = GetTickCount();
   if (s_test)
   {
      deltat = time-oldtime;
      fps = 0;
      if (deltat > 1000)
      {
         fps = (passcount * 10000) / deltat;
         fpsavg = globalpasscount * 10000 / (time-starttime);
         sprintf(pom_string,"FPS:%d.%d A:%d.%d",fps/10,fps%10,fpsavg/10,fpsavg%10);
         fpsmsg.Set(pom_string,&sfont_w);
         oldtime = time;
         passcount = 0;
      }
   }
   deltat = time - oldtime2;
   oldtime2 = time;
   int game_steps;
   if (deltat > 1000)
      deltat = 20;
   game_steps = deltat / GAMESTEP_T;

#define ALERT_INTERVAL 990
   if (is_red_alert && (step_global - last_alert) > ALERT_INTERVAL)
   {
		// sirena
      sound_device->Play(21,panpos);
      last_alert = step_global;
      panpos = 640-panpos;
   }
   
   //------------------------------------------------------------------ AKCE
   Creature *pom_cr;
   Teleport *pom_t;
   Bonus *pom_bo;
   UFO *pom_ufo;
   Bubble *pom_b;
   SymbolBubble *pom_sb;
   int r1;
   int plnum = 1; // pracovni promenna - pocet aktivnich hracu
   
   if (timer > 0)
   {
		// ubihani casu podle stavu hry a hracu
      if (monstercnt == -1)
		{
			// v red alertu je rychlost konstatni
         timer -= game_steps;
		}
      else
      {
			// podle poctu hracu (a nesmel jeste byt Murray)
         if (!(player1->state & NOTTAKEN))
         {
            timer -= game_steps;
            if (!(player2->state & NOTTAKEN) && ruby)
            {
               plnum = 2;
               timer-= game_steps;
            }
         }
         else
            if (!(player2->state & NOTTAKEN))
               timer-= game_steps;
      }

      // pip, pip, pip, peeeep
      if (!countdown && plnum == 2 && timer < 2*COUNTDOWN_TIME)
         nextcountdown = 2*COUNTDOWN_TIME;

		if (!is_red_alert && (timer < nextcountdown))
      {
         nextcountdown = nextcountdown - COUNTDOWN_INTERVAL * plnum;
         if (nextcountdown <= 0)
         {
            sound_device->Play(27);
         }
         else
         {
            timealert_dur = TA_DUR;
            sound_device->Play(26);
         }
         timealert_fRect.left = 0;
         timealert_fRect.right = TA_XSIZE;
         timealert_fRect.top = 0;
         timealert_fRect.bottom = TA_YSIZE;
         countdown = true;
      }
      if (timer <= 0) 
      {
         timer = 0;
         nextcountdown = COUNTDOWN_TIME;
         countdown = false;
      }

		// nastaveni rozmeru pro vykresleni casove linky
		// != nebo > ? Asi spis nerovnost - pro pripad, ze se cas skokove zmn
		// sakra, neni to upln blb?
		// mm dojem, e to pvodn mlo eit ppad, kdy je as del ne ukazatel a ono se to nevejde
      //if (((150 - (timer/timeunit)) > timecurrent) && (timecurrent < 150))
	   // druha cast podminky opravdu vyrazne setri strojovy cas, ale co uz... =)
      if ((150 - (timer/timeunit)) != timecurrent && timecurrent < 150)
      {
         timecurrent = 150 - (timer/timeunit);
			// V 2004 - melo by resit vymazane pocitadlo redalertu
			/*
         if (timecurrent > 150)
            timecurrent = 150;
				*/
         if (timecurrent >= 150)
			{
            timecurrent = 0;
				time_chg = 0;
			}
			else
			//
         time_chg = 2;
      }
    }  
   
   if (!(player1->state & (INACTIVE)))
      player1->KeyReflex();
   if (!(player2->state & (INACTIVE)))
      player2->KeyReflex();

   // hurry up!
   if (timealert_dur > 0)
   {
      timealert_dur -= game_steps;
      if (timealert_dur < 0)
         timealert_dur = 0;
   }

   // samobublive bublifuky
   if (magictime)
   {
      magiccount += game_steps;
      if (magiccount>magictime)
      {
         magiccount = 0;
         to_fuq++;
      }
	}

   //############################################################
   // Logicka smycka - start
   //############################################################
   for (int step_local=0; step_local < game_steps; step_local++)
   {
      step_global++;
      message.Tick();
      move_score_labels();
      //-------------------------------------------------- animace + pohyb
      //------------------------------------------- bublifuky
      I while(i < fuqcount)
      {
         bublifux[i]->Action();
         if ((step_global % bublifux[i]->fTgs) == 0)
         {
            bublifux[i]->jumpNext();
         }
         i++;
      }
      
      //---------------------------------------------- hrac 1
      if ((step_global % player1->fTgs) == 0 && !(player1->state & INBUBBLE)) 
         player1->jumpNext();
      
      if (!(player1->state & (INACTIVE)))
      {
         player1->stateReflex();
         player1->Move();
         player1->MapReflex();
      }
      else
      {
         if (player1->frames.chainid == CHAIN_DYING &&
            player1->frames.looped)
         {
            if (!player1->sheet.lives)
            {                  
               if (player2->state & DEAD)
               {
                  exitgame = 1;
                  levelname[0] = '%';
                  levelname[1] = 0;
                  break;
               }
               else
               {
                  player1->Resurrection();
                  player1->state |= DEAD;
                  ARRAY_REMOVE(monsters,monsters_ctrl,player1);
               }
            }
            else
               player1->Resurrection();
         }
      }
      //---------------------------------------------- hrac 2
      if ((step_global % player2->fTgs) == 0 && !(player2->state & INBUBBLE)) 
         player2->jumpNext();
      
      if (!(player2->state & (INACTIVE)))
      {
         player2->stateReflex();
         player2->Move();
         player2->MapReflex();
      }
      else
      {
         if (player2->frames.chainid == CHAIN_DYING &&
            player2->frames.looped)
         {
            if (!player1->sheet.lives)
            {
               if (player1->state & DEAD)
               {
                  exitgame = 1;
                  levelname[0] = '%';
                  levelname[1] = 0;
                  break;
               }
               else
               {
                  player2->Resurrection();
                  player2->state |= DEAD;
                  ARRAY_REMOVE(monsters,monsters_ctrl,player2);
               }
            }
            else
               player2->Resurrection();
         }
      }
      
      //-------------------------------------------- potfurky
      I while(i<monsters_ctrl.count)
      {
         if (frozen.timecurrent == 0)
         {
            pom_cr = ARRAY_AT(monsters,i);
            if (pom_cr->type == MONSTER)
            {
               if ((step_global % pom_cr->fTgs) == 0)
               {
                  pom_cr->jumpNext();
               }
               pom_cr->KeyReflex();
               pom_cr->Move();
               pom_cr->MapReflex();
            }
         }
         i++;
      }
      //-----------------------------------------------bonusy
      I while(i<bonuses_ctrl.count)
      {
         pom_bo = ARRAY_AT(bonuses,i);
         if (pom_bo->counter == BONUS_WARNING)
         {
            sound_device->Play(20);
         }
         if (pom_bo->counter == BONUS_LIFE)
         {
            ARRAY_REMOVE_AT_AND_DELETE(bonuses,bonuses_ctrl,i);
            continue;
         }
         
         if ((step_global % pom_bo->fTgs) == 0) 
            pom_bo->jumpNext();
         pom_bo->Move();
         pom_bo->MapReflex();
         i++;
      }
      
      //----------------------------------------------- ufoni
#ifdef MURRAYDEBUG
      strcpy(murrayg_msg,"MG:");
      strcpy(murrayb_msg,"MB:");
#endif
      I while(i<UFOcnt)
      {
         pom_ufo = UFOs[i];
         if ((step_global % pom_ufo->fTgs) == 0) 
            pom_ufo->jumpNext();
         r1 = pom_ufo->Move();
         if (r1 == OUT_OF_BOUNDS) 
         {
            delete pom_ufo;
            --UFOcnt;
            UFOs[i] = UFOs[UFOcnt];
            --i;
         }
         ++i;
      }
      
      //--------------------------------------------- bubliny
      // rychle
      I while(i < qbubbles_ctrl.count)
      {
         pom_b = ARRAY_AT(qbubbles,i);
         if ((step_global % pom_b->fTgs) == 0)
            pom_b->jumpNext();
         pom_b->Move();
         if (pom_b->IsSlow())
         {
            ARRAY_REMOVE_AT(qbubbles,qbubbles_ctrl,i);
            if (ARRAY_FULL(sbubbles_ctrl))
            {
               delete pom_b;
               app->sayConsole("Too much slow bubbles!");
            }
            else
            {
               ARRAY_ADD(sbubbles,sbubbles_ctrl,pom_b);
               //samopraskaci bubliny
               if (pom_b->selfblowing)
               {
                  pom_b->counter = BUB_DOMINATED;
                  pom_b->sender = pom_b->whose;
               }
            }
            i--;
         }
         i++;
      }
      // pomale
      I while(i < sbubbles_ctrl.count)
      {
         pom_b = ARRAY_AT(sbubbles,i);
         if ((step_global % pom_b->fTgs) == 0) 
            pom_b->jumpNext();
         r1 = pom_b->Move();
         if (r1 & BUB_BLOWSTART)
         {
            to_fuq += pom_b->Blow(NULL);
         }
         if (r1 & BUB_BLOWEND)
         {
            ARRAY_REMOVE_AT_AND_DELETE(sbubbles,sbubbles_ctrl,i);
            i--;
         }
         i++;
      }
      // symbolove
      I while(i < symbubs_ctrl.count)
      {
         pom_sb = ARRAY_AT(symbubs,i);
         if ((step_global % pom_sb->fTgs) == 0)
            pom_sb->jumpNext();
         r1 = pom_sb->Move();
         if (r1 & BUB_BLOWSTART)
         {
            pom_sb->Blow(NULL);
         }
         if (r1 & BUB_BLOWEND)
         {
            ARRAY_REMOVE_AT_AND_DELETE(symbubs,symbubs_ctrl,i);
            i--;
         }
         i++;
      }
      
      //--------------------------------------------teleporty
      I while(i<teleports_ctrl.count)
      {
         pom_t = ARRAY_AT(teleports,i++);
         if ((step_global % pom_t->fTgs) == 0)
         {
            pom_t->jumpNext();
            if (is_red_alert)
               pom_t->tintPrepared();
         }
      }         
      
      //-----------------------------------------------dropsy
      if (drops->count) 
         drops->Move();         
      
      //----------------------------------------------------------- kolize
      // playeri vs. mostra
      I while(i<monsters_ctrl.count)
      {
         if (!(player1->state & RESISTANT))
            player1->CollideWith(ARRAY_AT(monsters,i));
         if (!(player2->state & RESISTANT))
            player2->CollideWith(ARRAY_AT(monsters,i));
         i++;
      }
      // playeri vs. ufa
      I while(i<UFOcnt)
      {
         if (!(player1->state & UFO_RESISTANT))
         {
            UFOs[i]->CollideWith(player1);
         }
         if (!(player2->state & UFO_RESISTANT))
         {
            UFOs[i]->CollideWith(player2);
         }
         i++;
      }
      
      // teleporty vs.others
      I while(i<teleports_ctrl.count)
      {
         J while(j<monsters_ctrl.count)
         {
            if(ARRAY_AT(teleports,i)->CollideWith(ARRAY_AT(monsters,j)))
               ARRAY_AT(teleports,i)->Transport(ARRAY_AT(monsters,j));
            j++;
         }
         i++;
      }
      
      // bubliny vs. others
      // nejprve symbolove, jestli je player neprasknul
      I while(i < symbubs_ctrl.count)
      {
         pom_sb = ARRAY_AT(symbubs,i);
         if (player1->action & SPMA_JUMP && !(player1->state & NOSCORING))
         {
            if(pom_sb->CollideWith(player1) > 0)
            {
               pom_sb->Blow(player1);
               i++;
               continue;
            }
         }
         if (player2->action & SPMA_JUMP && !(player2->state & NOSCORING))
         {
            if(pom_sb->CollideWith(player2) > 0)
            {
               pom_sb->Blow(player2);
            }
         }
         i++;
      }
      // pak ty rychlejsi, ze by jako neco chytly
      I while(i<qbubbles_ctrl.count)
      {
         J while(j<monsters_ctrl.count)
         {
            pom_b = ARRAY_AT(qbubbles,i);
            pom_cr = ARRAY_AT(monsters,j);
            if(pom_b->CollideWith(pom_cr) > 0)
            {
               if (pom_cr->type == PLAYER &&
                  (((Player *)pom_cr)->state & DEAD))
               {
                  j++;
                  continue;
               }
               sound_device->Play(5);
               pom_b->Catch(pom_cr);
               ARRAY_REMOVE_AT(monsters,monsters_ctrl,j);
               break;
            }
            j++;
         }
         i++;
      }
      // a jiz testime praskani pomalejsich z nas
      I while(i < sbubbles_ctrl.count)
      {
         pom_b = ARRAY_AT(sbubbles,i);
         if (player1->action & SPMA_JUMP && !(player1->state & NOSCORING))
         {
            if(pom_b->CollideWith(player1) > 0)
            {
               to_fuq += pom_b->Blow(player1);
               i++;
               continue;
            }
         }
         if (player2->action & SPMA_JUMP && !(player2->state & NOSCORING))
         {
            if(pom_b->CollideWith(player2) > 0)
            {
               to_fuq += pom_b->Blow(player2);
            }
         }
         i++;
      }
      // a bubliny mezi sebou
      i = (step_global & 3) * BUB_UNIT;
      k = i + BUB_UNIT;
      if (i > MAX_SBUBS)               // tak to jsou symbolive bubliny
      {
         I while(i < symbubs_ctrl.count)
         {
            j = i+1;
            while(j < symbubs_ctrl.count) 
            {
               ARRAY_AT(symbubs,i)->CollideWith(ARRAY_AT(symbubs,j));
               j++;
            }
            i++;
         } 
      }
      else
      {
         while(i < sbubbles_ctrl.count && i<k) 
         {
            j = i+1;
            while(j < sbubbles_ctrl.count) 
            {
               ARRAY_AT(sbubbles,i)->CollideWith(ARRAY_AT(sbubbles,j));
               j++;
            }
            J while(j < symbubs_ctrl.count) 
            {
               ARRAY_AT(sbubbles,i)->CollideWith(ARRAY_AT(symbubs,j));
               j++;
            }
            i++;
         }
      }
      // playeri vs. bonusy
      I while(i<bonuses_ctrl.count)
      {
         if (!(player1->state & NOSCORING)
            && player1->CollideWith(ARRAY_AT(bonuses,i))
            || !(player2->state & NOSCORING)
            && player2->CollideWith(ARRAY_AT(bonuses,i)))
         {
            sound_device->Play(1);
         }
         i++;
      }
      // a bublifuky
      while(to_fuq)
      {
         if (fuqcount > 0)
         {
            fuq++;
            if (fuq == fuqcount) fuq = 0;
            bublifux[fuq]->to_fuq++;
         }
         to_fuq--;
      }
      player1->symbuf.UpdateFlash();
      player2->symbuf.UpdateFlash();
   }
       
   //############################################################
   // Logicka smycka - konec
   //############################################################

   // vhodne misto pro snizeni platnosti bonusu
   player1->CheckState(game_steps);
   player2->CheckState(game_steps);
   frozen.decrease(game_steps);
   mirror -= game_steps;
   if (mirror < 0)
      mirror = 0;


   //-------------------------------------------------- finalni akce
   // vybeh murrayho
   if (monstercnt > 0 && timer == 0)
   {
      message.Set("Vybh Murray!",&sfont_w);
      sound_device->Play(28); //uvitaci chechot!
      ruby = false;
      timer = 15000;
      RECT rTimer = {0,15,600,25};
      if (timer > (int)timelimit)
      {
         timelimit = 15000;
         timeunit = 100;
      }
      rTimer.left = 600 - 15000/timeunit;
      timecurrent = 0;
      time_chg = 0;
      putcel(rTimer.left,15,backbuffer,rTimer,background,DDBLTFAST_NOCOLORKEY);
      putcel(rTimer.left,15,frontbuffer,rTimer,background,DDBLTFAST_NOCOLORKEY);
      if (UFOcnt<MAX_UFOS && !(player1->state & NOCREDIT))
      {
         UFOs[UFOcnt++] = new Murray(310,230,
            properties.member[ID_MURRAYG],player1);
      }
      if (UFOcnt<MAX_UFOS && !(player2->state & NOCREDIT))
      {
         UFOs[UFOcnt++] = new Murray(310,230,
            properties.member[ID_MURRAYB],player2);
      }
   }
	// dosly potvory
   if (!monstercnt)
   {
      // zapnuti endportu
      I while(ftports_ctrl.count>0)
      {
         Teleport *pom_ft = ARRAY_AT(ftports,0);
         if (ARRAY_FULL(teleports_ctrl))
         {
            delete pom_ft;
         }
         else
         {
            ARRAY_ADD(teleports,teleports_ctrl,pom_ft);
         }
         ARRAY_REMOVE_AT(ftports,ftports_ctrl,0);
         i++;
      }
      // likvidace vsech UFOs
      for (I i<UFOcnt; i++)
      {
         UFOs[i]->Melt();
      }

      // zacina red alert
		is_red_alert = true;
      // novy redalert presachovanim pointeru
		// V. 2004
		// red_alert(background);
		background = background_redalert;
		//

      FILL_USEDMAP_PLAYFIELD(usedmap1);
      FILL_USEDMAP_PLAYFIELD(usedmap2);

		// maze se normalni cas, prechazime na redalertovy
      message.Set("10 sekund do oputn zny!",&sfont_w);
      timer = 5000;
      timeunit = timer / 150;
      timecurrent = 0;
      time_chg = 0;
		//
      RECT tR = {0,0,150,10};
      putcel(450,15,backbuffer,tR,timer10_gfx,DDBLTFAST_NOCOLORKEY);
      putcel(450,15,frontbuffer,tR,timer10_gfx,DDBLTFAST_NOCOLORKEY);

		// potvory nejsou a uz nebudou
      monstercnt = -1;

		// zaslouzi si rubn?
      if (ruby)
      {
         pom_bo = new Bonus(295,40,properties.member[ID_RUBY]);
         pom_bo->movex = 0;
         pom_bo->movey = 0;
         if (ARRAY_FULL(bonuses_ctrl))
         {
            delete pom_bo;
            app->sayConsole("Ruby: Bonuses full...deleted");
         }
         else
         {
            ARRAY_ADD(bonuses,bonuses_ctrl,pom_bo);
         }
      }
   }
   if (monstercnt == -1 && timer == 0 && vanish == 0)
   {
      strcpy(levelname,nextlvls);
      sound_device->Play(11);
      exitgame = true;
   }
#ifdef PROFILER_DEBUG
   profiler->stopTimer(1);
#endif

   if (exitgame && vanish == 0)
   {
      if (!(player1->state & NOCREDIT))
      {
         ++player1->sheet.lives;
      }
      if (!(player2->state & NOCREDIT))
      {
         ++player1->sheet.lives;
      }
      if (levelname[0] == '%')
      {
         spadla_klec = true; 
         sound_device->Play(22);
         toppers->clearNew();
         if (!(s_testholy || s_testsymbol))
         {
            toppers->Ins(player1->sheet.name,player1->sheet.score,&qfont_g);
            toppers->Ins(player2->sheet.name,player2->sheet.score,&qfont_b);
         }
         putcel(0,0,backbuffer,rFullScreen,frontbuffer,DDBLTFAST_NOCOLORKEY);
         kbd.filter_lastscan = false;
         kbd.lastASCIIReset();
         kbd.lastScanReset();
      }
      else if (levelname[0] == '@')
      {
         toppers->clearNew();
         if (!(s_testholy || s_testsymbol))
         {
            toppers->Ins(player1->sheet.name,player1->sheet.score,&qfont_g);
            toppers->Ins(player2->sheet.name,player2->sheet.score,&qfont_b);
         }
         app->stateChange(REASON_THEEND);
      }
      else
      {
         app->stateChange(REASON_NEXTLEVEL);
      }
      return;
   }
}
      
//-------------------------------------------------------------------------
// uvedeni hry do provozuschopneho stavu pri ztraceni surfaces
//-------------------------------------------------------------------------
void Game::restore()
{
   int i;
   Bub *pom_bub;
   Bonus *pom_bo;

   remove_all_score_labels();

	// plosinky musi prijit driv, nez redalert, aby je to zahrnulo
	PrepareBackground();

	// v kadm ppad je teba pegenerovat pozad pro redalert!
	surfaceManager->remove(background_redalert,true);
	background_redalert = surfaceManager->create(640,480,"*bckg_r");
	putcel(0,0,background_redalert,rFullScreen,background_normal);
	red_alert(background_redalert);

	// realertov asomra - V. 2004
	RECT tR = {0,0,150,10};
   putcel(450,15,background_redalert,tR,timer10_gfx,DDBLTFAST_NOCOLORKEY);

	// take co vlastn probh?
	if (is_red_alert)
	{
		background = background_redalert;
	}
	else
	{
		background = background_normal;
	}
	//

   // dolni panel - jmena hracu a zivoty
   name2_x = score2_rect.right - strlen(player2->sheet.name) * qfont_b.xsize;
   putString(score1_x,446,background,player1->sheet.name,qfont_g);
   putString(name2_x,446,background,player2->sheet.name,qfont_b);
   // horni panel - cislo levelu, nazev a autor
   char *desc = s_leveldesc[0]+1;
   while (*desc!='.' && (desc-s_leveldesc[0])<4) desc++;
   if (*desc=='.')
   { // je tam nejake cislo, tak ho odrizneme a vypiseme zvlast
     *desc = 0;
     desc++; if (*desc == ' ') desc++;
     putString(num_x+(3-strlen(s_leveldesc[0]))*12,4,background,s_leveldesc[0],qfont_r);
   }
   else
   {  // cislo nikde - jenom nazev 
      desc = s_leveldesc[0];
   }
   putString(desc_x,4,background,desc,qfont_w);
   if (desc != s_leveldesc[0])
      s_leveldesc[0][strlen(s_leveldesc[0])]= '.';
   putString(desc_x,21,background,s_leveldesc[1],qfont_y);

	// pozadi by melo byt ok
	// jdeme na aktualni frame
   putcelBlt(0,0,backbuffer,rFullScreen,background);

   if (spadla_klec)
   {
      winquake_dark(backbuffer);
      putStringCentered(320,230,backbuffer,"A spadla klec!",qfont_w);
   }

   if (system_frozen > 0)
   {     
      // teleporty
      I while(i<teleports_ctrl.count)
         ARRAY_AT(teleports,i++)->Paint(backbuffer);
      // bonusy
      I while(i<bonuses_ctrl.count)
      {
         pom_bo = ARRAY_AT(bonuses,i++);
         pom_bo->cached = 0;
         pom_bo->Paint(backbuffer);
         // potfurky
      }
      I while(i<monsters_ctrl.count)
         ARRAY_AT(monsters,i++)->Paint(backbuffer);
      // kreslime rychle bubliny
      I while(i<qbubbles_ctrl.count)
         ARRAY_AT(qbubbles,i++)->Paint(backbuffer);
      // kreslime pomale bubliny
      I while(i<sbubbles_ctrl.count)
      {
         pom_bub = ARRAY_AT(sbubbles,i);
         pom_bub->computed = false;
         pom_bub->monsterbehind = true;
         pom_bub->Paint(backbuffer);
         pom_bub->monsterbehind = false;
         i++;
      }
      // kreslime symbolove bubliny
      I while(i<symbubs_ctrl.count)
      {
         pom_bub = ARRAY_AT(symbubs,i);
         pom_bub->computed = false;
         pom_bub->monsterbehind = true;
         pom_bub->Paint(backbuffer);
         pom_bub->monsterbehind = false;
         i++;
      }
      // kreslime bublifuky
      I while(i < fuqcount)
         bublifux[i++]->Paint(backbuffer);
      // kreslime UFOny
      I while(i<UFOcnt)
         UFOs[i++]->Paint(backbuffer);
      // kreslime bonbony =)
      I while(i<drops->count)
         drops->list[i++]->Paint(backbuffer);
      if (system_frozen & SF_STATS)
      {
         statsTint(backbuffer);
         player1->Stats(20);
         player2->Stats(340);
      }
      if (system_frozen & SF_PAUSED)
      {
         winquake_dark(backbuffer);
         putStringCentered(320,230,backbuffer,"Pauza!",qfont_w);
      }
      if (system_frozen & SF_HELP)
      {
         winquake_dark(backbuffer);
         putString(20,22,backbuffer,"Npovda",qfont_r);
         putString(20,52,backbuffer,"Klvesy",qfont_y);
         putString(20,82,backbuffer,"F1   tato obrazovka",qfont_w);
         putString(20,102,backbuffer,"F9   screenshot",qfont_w);
         putString(20,142,backbuffer,"Tab  statistiky hr",qfont_w);
      }
   }
	// flip
   putcelBlt(0,0,frontbuffer,rFullScreen,backbuffer);

   fpsmsg.Hide();
   message.Hide();
#ifdef MURRAYDEBUG
   murraygmsg.Hide();
   murraybmsg.Hide();
#endif
   lives_chg = 2;
   player1->sheet.score_changed = 2;
   player1->sheet.energy_changed = 2;
   player1->symbuf.changed = 2;
   player2->sheet.score_changed = 2;
   player2->sheet.energy_changed = 2;
   player2->symbuf.changed = 2;
   time = GetTickCount();
   oldtime = time;
   oldtime2 = time;
}

//-------------------------------------------------------------------------
// cinnost pri prechodu aplikace do jineho stavu -> menu, konec, ...
//-------------------------------------------------------------------------
void Game::stop()
{
   remove_all_score_labels();
   DoneLevel();
   fpsmsg.Hide();
   message.Hide();

	// ze by tohle byl potenciln zdroj problm?
	// radji _normal - V. 2004
   if (m_background != background_normal)
      surfaceManager->remove(m_background);

	// tohle by asi bylo uzitecne udelat...?
	// nebo se to i tak udela samo?
	// V. 2004
	surfaceManager->remove(background_normal);
	surfaceManager->remove(background_redalert);
	//
#ifdef MURRAYDEBUG
   murraygmsg.Hide();
   murraybmsg.Hide();
#endif
}

//-------------------------------------------------------------------------
// odstraneni permanentich dat pri ukonceni aplikace
//-------------------------------------------------------------------------
void Game::unload()
{
   int i;
   
   if (nextlvls) 
   {
      delete [] nextlvls;
      nextlvls = NULL;
   }
   if (glasses != NULL)
   {
      for (i = 0; i < glasscount; i++)
      {
         if (glasses[i] != NULL)
            delete glasses[i];
      }
      delete [] glasses;
      glasses = NULL;
   }   
}

//==========================================================================
//    Deinicializace levelu
//==========================================================================
void Game::DoneLevel()
{
   int i;
   
   // musi byt pred array_1.RemoveAll, protoze vola notify sveho monstra
   I while (i < UFOcnt) 
   {
      delete(UFOs[i]);
      UFOs[i] = NULL;
      ++i;
   }
   UFOcnt = 0;
   ARRAY_REMOVE(monsters,monsters_ctrl,player1);
   ARRAY_REMOVE(monsters,monsters_ctrl,player2);
   
   ARRAY_REMOVE_ALL(monsters,monsters_ctrl);
   ARRAY_REMOVE_ALL(bonuses,bonuses_ctrl);
   ARRAY_REMOVE_ALL(qbubbles,qbubbles_ctrl);
   ARRAY_REMOVE_ALL(sbubbles,sbubbles_ctrl);
   ARRAY_REMOVE_ALL(symbubs,symbubs_ctrl);
   ARRAY_REMOVE_ALL(teleports,teleports_ctrl);
   ARRAY_REMOVE_ALL(ftports,ftports_ctrl);
   drops->RemoveAll();
   I while (i<fuqcount)
   {
      delete(bublifux[i]);
      bublifux[i++] = NULL;
   }
   
   if (map != NULL) delete [] map;
   if (windmap != NULL) delete windmap;
   map = NULL;
   windmap = NULL;
}

//==========================================================================
//    Deinicializace hry
//==========================================================================
void Game::DoneGame()
{
   SAFE_DELETE(player1);
   SAFE_DELETE(player2);
   SAFE_DELETE(drops);
}

//==========================================================================
//    Destruktor - deinicializace systemu
//==========================================================================
Game::~Game()
{
   if (backname)
   {
      delete [] backname;
      backname = NULL;
   }
   
#ifdef FILEDEBUG
   fclose(fw);
#endif
}
