/*****************************************************************************
                                Handling maps                               
******************************************************************************
  Written by : Vojvo  & G.
  Language   : VisuC 6 + DX6.1
  Version    : 0.7.2
  Last update: 12.11.1999
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include "consts.h"
#include "map.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "exceptions.h"
#include "tint.h"
#include "fakedir.h"

#define TILE_TRANSPARENT   32768


//==========================================================================
Map load_map (char *name)
{
   FILE *fr;
   Map dest = NULL;

   // alokace
   dest = new WORD[MAP_SIZE];

   if ((fr = fakeopen("data\\levels.dir",name,"rb")) == NULL)
   {
      if (dest != NULL)
         delete [] dest;
      dest = NULL;
   }
      
   // nacteni
   if (dest != NULL)
   {
      ZeroMemory(dest,MAP_SIZE);
      fread((void *)dest,MAP_SIZE,2,fr);
      fclose(fr);
   }
   
   return dest;
}

//-------------------------------------------------------------------------
// nahrava vetrnou mapu
//-------------------------------------------------------------------------
WindMap load_wndmap (char *name)
{
   FILE *fr;
   WindMap dest = NULL;

   // alokace
   dest = new char[WNDMAP_SIZE*2];

   if ((fr = fakeopen("data\\levels.dir",name,"rb")) == NULL)
   {
      if (dest != NULL)
         delete [] dest;
      dest = NULL;
   }
      
   // nacteni
   if (dest != NULL)
   {
      ZeroMemory(dest,WNDMAP_SIZE*2);
      fread((void *)dest,WNDMAP_SIZE,2,fr);
      fclose(fr);
   }
   
   return dest;
}

//===========================================================================
WORD map_value (int x, int y, Map map)
{
   int offs;

   offs = (int)(x/MAP_CELX) + MAP_XSIZE*(int)(y/MAP_CELY);
   if (offs > MAP_SIZE)
      return (char) MAP_ERROR;
   return *(map+offs);
}

//-------------------------------------------------------------------------
// vraci vitr na zadane pozici
//-------------------------------------------------------------------------
char wndmap_value (int x, int y, WindMap map)
{
   int offs;

   offs = WNDMAP_YSIZE*(int)(x/MAP_CELX) + (int)(y/MAP_CELY);
   if (offs > WNDMAP_SIZE)
      return  (char) MAP_ERROR;
   return *(map+offs);
}

//==========================================================================
void draw_map2(Map map, Surface cels, Surface dest)
{
   RECT r;
   BYTE cels_r;
   int x, y;
   WORD value;
   bool transparent = false;
   
   cels_r = surfaceManager->getHeader(cels)->byCelRow;
	for (int jloc = MAP_TOP; jloc<MAP_BOTTOM; jloc++)
		for (int iloc = 0; iloc<MAP_XSIZE;iloc++)
		{
         x = iloc * MAP_CELX;
         y = jloc * MAP_CELY;
// 		if (map_value(iloc*MAP_CELX, jloc*MAP_CELY,map))
       	value = map[jloc*MAP_XSIZE+iloc];
         transparent = false;
         if (value & TILE_TRANSPARENT)
         {
            value -= TILE_TRANSPARENT;
            transparent = true;
         }
         if (value)
         {
            r.left = (value % cels_r) * MAP_CELX;
            r.right = r.left + MAP_CELX;
            r.top = (value /cels_r) * MAP_CELY;
            r.bottom = r.top + MAP_CELY;
            //ted prijde tint pod plosinkou
            //krome posledniho radku
            if (jloc<(MAP_BOTTOM-1))
            {
               //bez poslediho sloupce vpravo
               if (iloc<(MAP_XSIZE-1))
               {
                  if (transparent)
                     putdark25(x+10,y+10,dest,r,cels);
                  else
                     putdark50(x+10,y+10,dest,r,cels);
                  //prvni levy sloupec jako druhy
                  if (iloc==0)
                  {
                     r.right -= 10;
                     if (transparent)
                        putdark25(x,y+10,dest,r,cels);
                     else
                        putdark50(x,y+10,dest,r,cels);
                     r.right += 10;
                  }
               }
               //krajni pravy sloupec
               else
               {
                  r.right -= 10;
                  if (transparent)
                     putdark25(x+10,y+10,dest,r,cels);
                  else
                     putdark50(x+10,y+10,dest,r,cels);
                  r.right += 10;
               }
            }          
            //a ted pres to plosinka
            if (transparent)
               puttransparent(x,y,dest,r,cels);
            else
               putcel(x,y,dest,r,cels);
         }
      }
}

//===========================================================================
int under(int x, int y, Map map, int bound) {
   int retv=0, s;

   int offs = (int)(x/MAP_CELX) + MAP_XSIZE * (int)(y/MAP_CELY);
   int limit = (offs % MAP_XSIZE) + (MAP_YSIZE-1) * MAP_XSIZE;

   do 
   {
      s = *(map+offs);
      if (s > 0) {
         retv++;
      }
      offs += MAP_XSIZE;
   } while (offs < limit && s > 0 && retv < bound);
   return retv;
}

//===========================================================================
//#define MAP_REPORT
int countturnpos (Map map)
{
   long tiles = 0;
   long plats = 0;
   long over =  0;
   WORD lasttile = 0;
   WORD thistile = 0;
   long res = 0;
   for (int jloc = 3; jloc<(MAP_YSIZE-2); jloc++)
   {
      lasttile = 0;
      for (int iloc = 0; iloc<MAP_XSIZE; iloc++)
      {
         thistile = map[iloc+jloc*MAP_XSIZE];
         if (thistile && !map[iloc+(jloc-1)*MAP_XSIZE])
         {
	         tiles++;
	         if (!lasttile) plats++;
	         if ((jloc<(MAP_YSIZE-2) && map[iloc+(jloc+2)*MAP_XSIZE]) ||
	            (jloc<(MAP_YSIZE-3) && map[iloc+(jloc+3)*MAP_XSIZE]) )
	            over++;
         }
         lasttile = thistile;
      }
   }
   res = 100*over/tiles + 100*plats/over;
#ifdef MAP_REPORT
   FILE * f;
   if ((f = fopen("maprep.txt","a+t")) != NULL)
   {
      fprintf(f,"\ntiles: %i\n",tiles);
      fprintf(f,"plats: %i\n",plats);
      fprintf(f,"over : %i\n",over);
      fprintf(f,"res  : %i\n",res);
      fclose(f);
   }
#endif

   return (int)res;
}
