//*****************************************************************************
//                   Stupid utility for graphic text output
//*****************************************************************************
//  Written by : Vojvo over Gunny                                              
//  Language   : VisuC + DX6.1                                                 
//  Version    : 0.99.3                                                          
//  Last update: 30.11.1999                                                    
//*****************************************************************************
// Changes
// 0.99.3 - uprava na sektorove clearovani
// 0.9.1 - odstraneno nadbytecne snizovani duration v Paintu

#include <stdlib.h>
#include <string.h>
#include "dxconsts.h"
#include <ddraw.h>
#include "consts.h"
#include "surfaceManager.h"
#include "types.h"
#include "graphDD.h"
#include "font.h"
#include "message.h"

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Message::Message()
{
   x = MSG_X;
   y = MSG_Y;
   msg = NULL;
   duration = 0;
   fnt = &sfont_w;
   surfname = "*msg";
   indent = IND_CENTER;
}

//-------------------------------------------------------------------------
// nastaveni vlastnosti zobrazovani zprav
//-------------------------------------------------------------------------
void Message::Options(int nx, int ny, Font *f, const char *name, BYTE ind)
{
   x = nx;
   y = ny;
   fnt = f;
   if (name != NULL)
      surfname = (char *)name;
   indent = ind;
}

//-------------------------------------------------------------------------
// iniciuje zobrazeni nove zpravy
//-------------------------------------------------------------------------
void Message::Set(char *what, Font *f)
{
   if (f != NULL)
      fnt=f;
   if (msg != NULL)
   {
      surfaceManager->remove(msg,true);
      msg = NULL;
   }
   msg = getPutString(what,*fnt);
   surfaceManager->add(msg,surfname,NULL,NULL,false);
   r.left = 0;
   r.right = strlen(what) * fnt->xsize;
   r.top = 0;
   r.bottom = fnt->ysize;
   switch (indent)
   {
   case IND_LEFT:
      realx = x;
      break;
   case IND_RIGHT:
      realx = x - r.right;
      break;
   case IND_CENTER:
   default:
      realx = x - (r.right+1)/2;
   }

   if (realx < 0)
      duration = 0;
   else
   {
      duration=MSG_DURATION;
      destr.left = realx;
      destr.right = realx + r.right;
      destr.top = y;
      destr.bottom = y + r.bottom;
      used.left = destr.left >> 6; 
      used.right = destr.right >> 6;
      used.top = destr.top / 40;
      used.bottom = destr.bottom / 40;
   }
}

//-------------------------------------------------------------------------
// Snizeni counteru
//-------------------------------------------------------------------------
void Message::Tick()
{
   if (duration > 0)
   {
      duration --;
   }
   else
   {
      if (msg != NULL)
      {
         surfaceManager->remove(msg,true);
         msg = NULL;
      }
   }
}


//-------------------------------------------------------------------------
// Vykresluje zpravu
//-------------------------------------------------------------------------
void Message::Paint()
{
   if (duration > 1)
   {
      putcel(realx,y,backbuffer,r,msg,DDBLTFAST_SRCCOLORKEY);
      for (int j=used.top; j <= used.bottom; j++)
         for (int i=used.left; i <= used.right; i++)
            (*usedmap)[i][j] = true;
   }
}

//-------------------------------------------------------------------------
// Skryje zpravu
//-------------------------------------------------------------------------
void Message :: Hide()
{
   duration = 0;
}

//-------------------------------------------------------------------------
// Smaze zpravu na cilovem surface
//-------------------------------------------------------------------------
void Message :: Clear()
{
   if (duration > 0)
   {
      putcel(realx,y,backbuffer,destr,background,DDBLTFAST_NOCOLORKEY);
   }
}
