//*************************************************************************
//                              Score labels                              
//*************************************************************************
//  Written by : Rory                                                          
//  Language   : VisuC6 + DX6.1                                                 
//  Version    : 1.0.1                                                          
//  Last update: 14.04.2000                                                      
//*************************************************************************
// Changes:
// 1.0.1  - snaha o prvni plnou verzi
// 0.99.3 - uprava na sektorove clearovani

#include <string.h>
#include "consts.h"
#include "types.h"
#include "exceptions.h"
#include "utils.h"
#include "graphDD.h"
#include "font.h"
#include "game.h"
#include "application.h"
#include "score.h"

#define SCCELS_MAX  30
#define SCCELS_DUR  500
#define SCCELS_MINY 60


int score_cnt = 0;
Surface score_cels[SCCELS_MAX];
int score_x[SCCELS_MAX];
int score_y[SCCELS_MAX];
int score_sizex[SCCELS_MAX];
int score_sizey;
int score_dur[SCCELS_MAX];


//===========================================================================
void add_score_label(int x, int y, int sc_value, Font *font)
{
   int charcnt;
   Surface surf;
   char s[5];  

   if (sc_value == 0)
      return;

   if (score_cnt >= SCCELS_MAX)
   {
      app->sayConsole("add_score_label: Maximum reached");
      return;
   }

   // vypocet poctu cislic
   __asm 
   {
      xor edx,edx
      mov eax, sc_value
      mov ebx,10
      mov ecx,0
   }
Loop1:
   __asm 
   {
      div ebx
      inc ecx
      xor edx,edx
      or eax,eax
      jnz Loop1
   
      mov charcnt, ecx
   }

   if (charcnt > 5)
      charcnt = 5;

   // vytvoreni a ulozeni surface a jeho vlastnosti
   score_sizey = font->ysize;
   score_sizex[score_cnt] = charcnt*font->xsize;
   score_x[score_cnt] = x;
 
   if (y < SCCELS_MINY)
      score_y[score_cnt] = SCCELS_MINY;
   else
      score_y[score_cnt] = y;

   numtos(sc_value,charcnt,s);
   surf = getPutString(s,*font);

   score_cels[score_cnt] = surf;
   score_dur[score_cnt] = SCCELS_DUR;
   score_cnt++;
}

//===========================================================================
void remove_all_score_labels()
{
   if (score_cnt <= 0)
      return;
   
   do 
   {
      score_cnt--;
      SAFE_RELEASE(score_cels[score_cnt]);
   }
   while (score_cnt > 0);
}

//===========================================================================
void move_score_labels()
{
   int i = 0;
   while (i < score_cnt)
   {
      --score_dur[i];
      if (!(score_dur[i] & 31))
      {
         if (score_y[i] > PLAYFIELD_MIN_Y)
            score_y[i]--;
      }        
      if (score_dur[i] <= 0)
      {
         score_cels[i]->Release();
         --score_cnt;
         score_cels[i] = score_cels[score_cnt];
         score_cels[score_cnt] = NULL;
         score_x[i] = score_x[score_cnt];
         score_y[i] = score_y[score_cnt];
         score_sizex[i] = score_sizex[score_cnt];
         score_dur[i] = score_dur[score_cnt];
         i--;
      }
      i++;
   }
}


//===========================================================================
void paint_score_labels(Surface dest)
{
   RECT r;
   int i = 0,x,y;
   while (i < score_cnt)
   {
      x = score_x[i];
      y = score_y[i];
      r.left = 0; 
      r.right = score_sizex[i];
      r.top = 0;
      r.bottom = score_sizey;
      putcel(x,y,dest,r,score_cels[i]);
      SET_USEDMAP(x,y);
      SET_USEDMAP(x+r.right,y);
      SET_USEDMAP(x,y+r.bottom);
      SET_USEDMAP(x+r.right,y+r.bottom);
      i++;
   }
}

//===========================================================================
void clear_score_labels(Surface dest, Surface bckg)
{
}
