// BUBWIN verze

#ifndef SPRITE_H_BUBWINY
#define SPRITE_H_BUBWINY

#include <stddef.h>
#include <stdlib.h>
#include "types.h"
#include "consts.h"
#include "keyboard.h"
#include "tint.h"
#include "sheet.h"
#include "property.h"
#include "utils.h"

//#define MURRAYDEBUG
#ifdef MURRAYDEBUG
extern char murrayb_msg[256];
extern char murrayg_msg[256];
#endif

typedef enum {COMMON,PLAYER,MONSTER,BUBBLE,TPORT,FTPORT,BONUS} SPRITETYPE;

class Player;
class UFO;

extern UFO *UFOs[MAX_UFOS];
extern int UFOcnt;

//###########################################################################
//-------------------------------- Symbol -----------------------------------
//###########################################################################
typedef BYTE Symbol;

//###########################################################################
//-------------------------------- SymBuf -----------------------------------
//###########################################################################
class SymBuf {
public:
   Surface bubgfx;
   RECT rect[4];
   DWORD type;
   DWORD x,y;
   BYTE changed;
   WORD flash;
   
   SymBuf() {
      type = 0;
      x = 0; 
      y = 450; 
      changed = 0;
      flash = 0;
      rect[0].left = 20;
      rect[0].right = 40;
      rect[0].top = 260;
      rect[0].bottom = 280;
      rect[1].left = 40;
      rect[1].right = 60;
      rect[1].top = 260;
      rect[1].bottom = 280;
      rect[2].left = 60;
      rect[2].right = 80;
      rect[2].top = 260;
      rect[2].bottom = 280;
      rect[3].left = 80;
      rect[3].right = 100;
      rect[3].top = 260;
      rect[3].bottom = 280;
   }
   
   void Add(Symbol s);
   void Realize(Player *p);
   void Paint(Surface dest);
   void UpdateFlash();
   
   virtual ~SymBuf()
   {
   }
};
      

//##########################################################################
//--------------------------------- Sprite ---------------------------------
//##########################################################################

class Sprite {
   
   //== VARIABLES =============================================================
public:
   BYTE id;                            //id sprite
   SPRITETYPE type;                    //typ sprite
   DWORD fTgs;                         //game steps per frame
   Surface gfx;                        //frames
   Surface bubgfx;                     //grafika v bubline
   TintMask gfxmask;                   //maska
   DWORD celw,celh;                    //rozmery celu v surface
   DWORD surfw,surfh;                  //rozmery surface
   BYTE direction;                     //smer pohybu
   BYTE celr;                          //pocet celu v radku surface
   RECT frameRect;                     //vyrez aktualniho framu v surface
             
   int frameLeft, frameTop;            //puvodni leva a horni souradnice
                                       //vyrezu v surface pro orezavani
   // souradnice udavaji levy horni roh spritu
   int mul_x,mul_y;                    //nova pozice * mul_move;
   int movex,movey;                    //rychlost
   int bottomx,topx,bottomy,topy;      //hranice rychlosti
   int oldx,oldy,newx,newy;            //minula a nynejsi pozice v pixlich
   int hitx,hity;                      //meze pro zasah playera monstrem
   int b_miny,b_maxy,eb_maxy,b_maxx;   //meze pro testovani hranice zobrazeni
   BYTE xsize,ysize;                   //velikost sprite
   BYTE xoff,yoff;                     //ofset spritu v celu
   FrameController frames;             //ovladac animace framu
   unsigned int collision;             //urcuje, kde narazil na okraj
   int gravity;                        //gravitace
   int stable_l,stable_r;              //urcuje x-pixel v zakladne,
   //ktery urcuje je-li stabilni
   int sort_priority;                  //priorita trideni na prekryvani
   int is_stable;                      //je stabilni?
   bool clipped;                       //je to orezany frame?
   
   //== METHODS ===============================================================
   //vykresli sprite na <newx,newy>
   virtual void Paint(Surface dest);
   //vypocte <newx:newy>,<x,y> z rychlosti a <x,y>
   virtual int Move();
   //premazesmaze sprite na pozici <oldx,oldy> a updatuje <oldx,oldy>
   virtual void Clear(Surface dest, Surface src);
   //zpomali v x nebo y resp. "zrychli smerem nahoru|doleva"
   int Slow(int by_x, int by_y);
   // "zrychli smerem dolu|doprava"
   int Haste(int by_x, int by_y);
   // zpomali, a to globalne = zpomali pohyb
   void SlowIt(int by_x, int by_y);
   //urcuje, ktery bude dalsi frame
   virtual void jumpNext();
   // vraci, jestli je stabilni
   virtual int IsStable();
   // vraci true, kdyz nemuze doprava nebo doleva podle smeru pohybu
   virtual int CantGoLR();
   // vraci true, jestlize mezi this a what doslo ke kolizi
   virtual int CollideWith(Sprite *what) { return 0; };
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Sprite(int onx, int ony, Property *prop);
   Sprite() 
   { 
      gfxmask = NULL; 
   }
   virtual ~Sprite() 
   { 
   }
};

//##########################################################################
//------------------------------- Creature ---------------------------------
//##########################################################################

class Creature : public Sprite {
   
   //== VARIABLES =============================================================
public:
   char action;            //povolene akce,prave provadena akce
   int updateable;         //update pozice kreatury v bubline (A/N)
   BYTE old_chain;          //puvidni chain pred firem a skokem
   
   //== METHODS ===============================================================
   // reakce na klavesy
   virtual void KeyReflex() = 0;
   // reakce na mapu
   virtual void MapReflex();
   // obnoveni vlastnosti po vypadnuti z bubliny
   virtual void Reincarnate();
   // update v bubline
   virtual void BubbleAction(int bubx, int buby) = 0;
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Creature (int onx, int ony, Property *prop) : Sprite (onx,ony,prop)
   {
      type = COMMON;
      action = 0;
      updateable = 0;
   }
};



//##########################################################################
//-------------------------------- Player ----------------------------------
//##########################################################################
class Player : public Creature {
   
   //== VARIABLES =============================================================
public:
	void Place(int x, int y);
   BYTE dik_left, dik_right, dik_jump, dik_fire, sc_last; //reakcni klavesy
   BYTE litetint;     // barva tintu pro bubliny (a blikani)
   BYTE hardtint;     // barva tintu pro hrace
   BYTE dropcolor;    // barva kapek pri skytani a z bublin
   PSheet sheet;
   int fk_pressed;         //docasne promenne pro fire
   int state;
   SymBuf symbuf;
   Timer time_resurrected; //doba platnosti nezranitelnosti
   Timer time_invisible;   //doba platnosti neviditelnosti
   Timer time_selfblowing; //doba platnosti samopraskacich bublin
   int acceleration;       // zrychleni
   int jump_strength;      // sila odrazu
   int bubble_speed;       // rychlost vyplivnute bubliny
   int repelent;           // jestli odpuzuje Murrayho
   int gravityLow, gravityHigh;  //gravitace pro skok

	int lastseenx;				// posledni viditelna pozice
	int lastseeny;				// pokud je viditelny a mimo bublinu, 
									// tak se automaticky aktualizuje v checkstate
   
   //== METHODS ===============================================================
   // reakce na klavesy
   virtual void KeyReflex ();
   // reakce na stav playera
   virtual void stateReflex();
   // nastaveni ovladani
   void SetKeys (BYTE dikl,BYTE dikr,BYTE dikj,BYTE dikf);
   // kolize s ostatnimi objekty
   virtual int CollideWith (Sprite *what);
   // smrt playera
   void Die();
   // obnoveni playera po smrti
   void Resurrection ();
   // updateovani v bubline
   virtual void BubbleAction(int bubx, int buby);
   // vykresleni playera
   virtual void Paint (Surface dest);
   // snizovani trvani bonusu
   void CheckState(int stepcnt);
   // vypis vlastnosti
   void Stats(int startx);
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Player (Property *prop, Font *font);
};

//##########################################################################
//-------------------------------- Monster ---------------------------------
//##########################################################################

class Monster : public Creature {
   
   //== VARIABLES =============================================================
public:
   //kterym smerem chce potvurka jit - pouziva konstanty SCL_direction
   int decision;
   //vysledek minuleho pohybu
   //ctu to z 'move'
   //asi by bylo lepsi 'move' predelat
   int move_result;
   int kill_points;
   int catch_points;
   int wasstable;
   int bonus_cr,bonus_vl;
   
   //== METHODS ===============================================================
   // reakce na klavesy
   // tady bude mozna nejaka logika chovani =)
   virtual void KeyReflex();
   // test stability
   virtual int IsStable();
   // obzivnuti po vypadnuti z bodliny
   virtual void Reincarnate();
   // akce v bubline
   virtual void BubbleAction(int bubx, int buby);
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Monster(int onx, int ony, Property *prop);
};



//#### Z O M B #############################################################

class Zomb : public Monster {
   //== VARIABLES =============================================================
public:
   int wannajump;
   //== METHODS ===============================================================
   virtual void KeyReflex();
   virtual void MapReflex();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Zomb(int onx, int ony, Property *prop) : Monster(onx,ony,prop)
   {
      frames.setChain(CHAIN_WALK);
      wannajump=0;
   }
};


//#### N E B U L U S #######################################################


class Nebulus : public Monster {
   //== VARIABLES =============================================================
public:
   //== METHODS ===============================================================
   virtual void KeyReflex();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Nebulus(int onx, int ony, Property *prop) : Monster(onx,ony,prop) { }
};



//#### B U T T E R F L Y ###################################################

class Butterfly : public Monster {
   //== VARIABLES =============================================================
public:
   //== METHODS ===============================================================
   virtual void KeyReflex();
   virtual void MapReflex();
   virtual int Move();
   virtual void Reincarnate();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Butterfly(int onx, int ony, Property *prop);
};


//#### B U T T E R F L Y 2 #################################################

class Butterfly2 : public Butterfly {
   //== VARIABLES =============================================================
public:
   int mul_prevx,mul_prevy;   //stara pozice * mul_move
   int oldmapx,oldmapy;      //minula pozice v mape
   //== METHODS ===============================================================
   virtual void MapReflex();
   virtual void Reincarnate();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Butterfly2(int onx, int ony, Property *prop) : Butterfly(onx,ony,prop)
   {
      //tohle bohuzel neresi stav, kdy se motyl uz pri startu
      // objevi ve - to se pak 'portne' na [0,0]
      mul_prevx = mul_x;
      mul_prevy = mul_y;
      oldmapx=(newx/MAP_CELX)+1;
      oldmapy=(newy/MAP_CELY+1) * MAP_XSIZE;
   };
};


//#### B U T T E R F L Y 3 #horizontalni####################################

class Butterfly3 : public Butterfly {
   //== VARIABLES =============================================================
public:
   //== METHODS ===============================================================
   virtual void MapReflex();
   virtual void Reincarnate();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Butterfly3(int onx, int ony, Property *prop);
};


//#### K A C H N A #########################################################

class Kachna : public Monster {
   //== VARIABLES =============================================================
public:
   int olddecision;// pri snaseni je zrusen pohyb, tak si ho pamatuju
   int eggdelay;   // interval do dalsiho vejce
   //== METHODS ===============================================================
   virtual void MapReflex();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Kachna(int onx, int ony, Property *prop);
};

//#### K R U L I K #########################################################

class Bunny : public Monster {
//== VARIABLES =============================================================
public:
   int olddecision;  // pri vybuchu je zrusen pohyb, tak si ho pamatuju
   int blastdelay;   // interval do dalsiho vybuchu
   int old_stable_l, old_stable_r;  // pri premene na meat se meni stable

   //== METHODS ===============================================================
   virtual void MapReflex();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Bunny(int onx, int ony, Property *prop);
};

//#### F I R E A B L E ######################################################

class Fireable : public Monster {
public:   
   //== VARIABLES ==============================================================
   int firedone;           // indikuje, ze posledni jim fireovany fball je done
   int fireframesfinished; // signalizuje, ze framy fireovani skoncily
   DWORD timefinished;     // cas, ve kterem bude povolen dalsi firebal (pak 0)
   int olddecision;        // pri fire je zrusen pohyb, tak si ho musim pamatovat
   
   //== METHODS ================================================================
   virtual int  Move();
   virtual void PlayerReflex();
   virtual void Fire();
   virtual void Notify();
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Fireable(int onx, int ony, Property *prop);
   virtual ~Fireable();
};

//#### Y E L L O W   D R A G O N ############################################
class YellowDragon : public Fireable {

  //== VARIABLES ==============================================================
  public:
  //== METHODS ================================================================
  virtual void MapReflex();
  //== CONSTRUCTORS & DESTRUCTORS ============================================
  YellowDragon(int onx, int ony, Property *prop);
};

//#### R E D  D R A G O N ###################################################
class RedDragon : public Fireable {
public:
   //== VARIABLES ==============================================================
   int turndelay;      // zpozdeni rozbehu po otocce a vyskoku
   int decided;        // k cemu se minule rozhodl (nastavi se pozdeji)
   int mademind;       // uz se rozhodl, nikdo ho nepremluvi =)
   int bored2jump;     // aby si taky sem tam skocil
   int turnpossibility;// x:10 ze se na konci plosinky otoci
   int longfall;       // doba padu
   //== METHODS ================================================================
   virtual void KeyReflex();
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   RedDragon(int onx, int ony, Property *prop, int turnpos);
};

//###########################################################################
//----------------------------------- Bub -----------------------------------
//###########################################################################
class Bub : public Sprite {
   
   //== VARIABLES =============================================================
public:
   DWORD counter;
   WORD *glass;
   BYTE tint;
   BYTE dropcolor;
   int state;
   Player * sender; // kdo dostane body pri prasknuti
   Surface bubbuf;
   bool computed;
   bool monsterbehind;
   
   //== METHODS ===============================================================
   virtual int Move(void);
   virtual int Blow (Player *thissender) = 0;
   virtual void jumpNext() = 0;
   virtual int CollideWith (Sprite *what);
   void checkBehind(Sprite *what);
   int IsSlow ();
   int Domino ();
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Bub (int onx, int ony, Property *prop);
   virtual ~Bub()
   {
      if (bubbuf != NULL)
      {
         bubbuf->Release();
         bubbuf = NULL;
      }
   }
};

//###########################################################################
//--------------------------------- Bubble ----------------------------------
//###########################################################################

class Bubble : public Bub {
   
   //== VARIABLES =============================================================
public:
   Creature *monster;
   DWORD nextscrape;
   Player *whose;
   int selfblowing;
   int mon_chain;
   int mon_dir;
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual void Paint(Surface dest);
   void Catch(Creature *monster);
   virtual int  Blow(Player *thissender);
   virtual void jumpNext();
   virtual int  CollideWith(Sprite *what);
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Bubble (int onx, int ony, Property *prop);
   
   virtual ~Bubble() {
      if (monster && monster->type != PLAYER)
         delete monster;
   }
};

//###########################################################################
//------------------------------ SymbolBubble -------------------------------
//###########################################################################

class SymbolBubble : public Bub {
   
   //== VARIABLES =============================================================
public:
   Symbol symbol;
   Surface mask;
   
   //== METHODS ===============================================================
   virtual void Paint(Surface dest);
   virtual int Blow(Player *thissender);
   virtual void jumpNext();
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   SymbolBubble(int onx, int ony, Property *prop, Symbol s) 
      : Bub (onx,ony,prop)
   {
      symbol = s;
      mask = NULL;
      jumpNext();   
   }
};

//###########################################################################
//------------------------------- Teleport ----------------------------------
//###########################################################################
class Teleport : public Sprite {
   
   //== VARIABLES =============================================================
public:
   RECT destRect;
   int destx,desty;
   BYTE offset_dest;
   WORD tintcolor, tintcolordest;
   Surface cachedgfx;
   Surface cachedgfxdest;
   int redden;
   
   //== METHODS ===============================================================
   void prepare(Surface dest, char *name);
   void prepareDest(Surface dest, char *name);
   void tintPrepared();
   virtual void jumpNext();
   virtual void Paint(Surface dest);
   virtual void Clear(Surface dest, Surface src);
   virtual int  CollideWith(Sprite *what);
   void Transport(Creature *passenger);
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Teleport (int onx, int ony, Property *prop);
   virtual ~Teleport();
};

//###########################################################################
//------------------------------ Bublifuk -----------------------------------
//###########################################################################
class Bublifuk {
   
   //== VARIABLES =============================================================
public:
   int state;
   DWORD fTgs;                         // gamestates per frame
   Surface gfx;                        // grafika - maska
   int x,y;                            // souradnice leveho horniho rohu
   int xsize, ysize;
   DWORD lastT;
   RECT r;
   BYTE to_fuq;                        // kolik ma jeste fouknout bublin
   BYTE tint;                          // tint
   SymbolBubble *bubble;               // vytvarena bublina
   
   //== METHODS ===============================================================
   void Paint(Surface dest);
   void Clear(Surface dest, Surface src);
   void Action();
   void jumpNext();
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Bublifuk(int onx, int ony, Property *prop);
};

//###########################################################################
//-------------------------------- Bonus ------------------------------------
//###########################################################################
class Bonus : public Sprite {
   
   //== VARIABLES ==============================================================
public:
   WORD value;
   WORD counter;
   int  action;
   BYTE tint;
   bool cacheable;
   DWORD cached;
   WORD tintcolor;
   DWORD totalxsize, totalysize;
   Surface cachedgfx;
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual void Paint(Surface dest);
   virtual void MapReflex();
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Bonus (int onx, int ony, Property *prop);
   ~Bonus();
};

//###########################################################################
//---------------------------------- UFO ------------------------------------
//###########################################################################

class UFO : public Sprite {
   //== VARIABLES =============================================================
public:
   Fireable *parent;
   int melting;
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   UFO(int onx, int ony, Property *prop, Fireable *par)
      : Sprite(onx, ony, prop)
   {
      melting = FALSE;
      parent = par;
   }
   
   ~UFO() 
   { 
      if (parent) parent->Notify();
   }
   
   
   //== METHODS ===============================================================
   virtual int Move();
   virtual void Clear(Surface dest, Surface src);
   virtual int CollideWith(Player *p);
   virtual void Paint(Surface dest);
   virtual void Melt();
};

//###########################################################################
//------------------------------- Fireball ----------------------------------
//###########################################################################
class Fireball : public UFO {
public:   
   //== VARIABLES =============================================================

   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Fireball(int onx, int ony, Property *prop, Fireable *par, int spd);
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual int  CanGoLR();
   virtual void Melt();
};

//###########################################################################
//-------------------------------  Vejce   ----------------------------------
//###########################################################################
class Egg : public UFO {
   
   //== VARIABLES =============================================================
public:
   int gravity;        // gravitace
   int this_tile;      // v jake kostce je ted spodek vejce
   int last_tile;      // kde byl spodek pri minulem testu
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Egg(int onx, int ony, Property *prop, int g) : UFO(onx,ony,prop,NULL)
   {
      gravity = g;
      last_tile = 1;
   };
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual int  CanGoLR();
   virtual void Melt();
};

//###########################################################################
//-------------------------------  Masox   ----------------------------------
//###########################################################################
class Meat : public UFO 
{  
   //== VARIABLES =============================================================
public:
   int gravity;        // gravitace
   int this_tile;      // v jake kostce je ted spodek masa
   int last_tile;      // kde byl spodek pri minulem testu
   
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Meat(int onx, int ony, Property *prop,int g)
      : UFO(onx,ony,prop,NULL)
   {
      gravity = g;
      last_tile = 1;
   }
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual int  CanGoLR();
   virtual void Melt();
};

//###########################################################################
//-------------------------------  Murray  ----------------------------------
//###########################################################################
class Murray : public UFO {
   
   //== VARIABLES =============================================================
public:
   int accx,accy;
   int targx,targy;
   int speed,maxspeed;
   Player * target;
   WORD accelcounter;
  
   //== CONSTRUCTORS & DESTRUCTORS ============================================
   Murray(int onx, int ony, Property *prop, Player * plr);
   
   //== METHODS ===============================================================
   virtual int  Move();
   virtual void Melt();
};
#endif