/*
//****************************************************************************
//                        Teleport class implementation
//****************************************************************************
//  Written by : Vojvo (& G.)
//  Language   : VisuC6 + DX6.1
//  Version    : 1.1.2
//  Last update: 12.11.2004
// ***************************************************************************
// changes:
   1.1.2  / zablokovno provdn prepareDest u FTPORTU
// 0.99.5 - teleporty se pri red alertu zobrazuji "redalertove"
//        - hraci mizi ve final teleportech
// 0.99.3 - uprava zobrazovani teleportu
// 0.8.1 - pridan zvuk
// 0.7.1 - opravena delka jednoho nextlvl
//       - pridan destruktor pro odstraneni nacacheovaneho tintu
*/

#include <string.h>
#include <stdio.h>
#include "exceptions.h"
#include "types.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "tint.h"
#include "wsound.h"
#include "game.h"
#include "sprites.h"

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Teleport::Teleport(int onx, int ony, Property *prop)
{
   type = TPORT;
   id = prop->id;
   fTgs = prop->fTgs;
   direction = DIR_L;
   surfw = prop->r16h->dwWidth; 
   surfh = prop->r16h->dwHeight;
   celw = prop->r16h->dwCelWidth;
   celh = prop->r16h->dwCelHeight;
   celr = prop->r16h->byCelRow;
   cachedgfx = NULL;
   xsize = prop->xsize;
   ysize = prop->ysize;
   xoff = prop->xoff;
   yoff = prop->yoff;
   tintcolor = prop->tintcolor;
   tintcolordest = prop->tintcolor2;
   offset_dest = (BYTE)prop->value1;
   frames.frames = &(prop->frames);
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   jumpNext();   
   gfx = prop->surf;
   collision = 0;
   gravity = DFT_GRAVITY;
   newx = oldx = onx;
   newy = oldy = ony;
   mul_x = newx << SHIFT_MOVE + MUL_HALF;
   mul_y = newy << SHIFT_MOVE + MUL_HALF;
   movex = movey = 0;
   stable_l = prop->stablel;
   stable_r = prop->stabler;
   topx = prop->speed;
   topy = JUMP_STRENGTH;
   bottomx = -topx;
   bottomy = -topy;
   sort_priority = 0;
   is_stable = 0;
   cachedgfx = NULL;
   cachedgfxdest = NULL;
   redden = 0;
}

//-------------------------------------------------------------------------
// predpripravi tinty pod teleportem
//-------------------------------------------------------------------------
void Teleport::prepare(Surface dest, char *name)
{
   TintMask mask = surface2mask(gfx);

   if (cachedgfx != NULL)
   {
      surfaceManager->remove(cachedgfx,true);
      cachedgfx = NULL;
   }

   if (type == FTPORT)
      cachedgfx = surfaceManager->create(surfw,surfh,name,
      DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY);
   else
      cachedgfx = surfaceManager->create(surfw,surfh,name);
   
   RECT rdest;
   
   rdest.left = newx; 
   rdest.right = newx+celw;
   rdest.top = newy;
   rdest.bottom = newy+celh;
      
   DWORD j,i;
   BYTE k = 0;
   for (j = 0; j<surfh; j += celh)
   {
      for (i = 0; i < surfw; i += celw)
      {
         putcel(i,j,cachedgfx,rdest,dest,DDBLTFAST_NOCOLORKEY);
         k++;
      }
   }
   
   rdest.left = 0;
   rdest.right = surfw;
   rdest.top = 0;
   rdest.bottom = surfh;

   if (mask == NULL)
      throw new BubException("Teleport: mask is NULL!");
   puttint50(0,0,cachedgfx,rdest,mask,tintcolor);
   delete [] mask;
}

//-------------------------------------------------------------------------
// predpripravi tinty pod teleportem (cil)
//-------------------------------------------------------------------------
void Teleport::prepareDest(Surface dest, char *name)
{
	// tohle byl zejm zdroj problm
	// pokud je to FTport, tak je nesmysl generovat grafiku pro cl
	// nejjistj to bude oetit tady
	// V. 2004
	if (type == FTPORT) return;
	//

   TintMask mask = surface2mask(gfx);

   if (cachedgfxdest != NULL)
   {
      surfaceManager->remove(cachedgfxdest,true);
      cachedgfxdest = NULL;
   }

   cachedgfxdest = surfaceManager->create(surfw,surfh,name);
   
   RECT rdest;
   
   rdest.left = destx; 
   rdest.right = destx+celw;
   rdest.top = desty;
   rdest.bottom = desty+celh;
      
   DWORD j,i;
   BYTE k = 0;
   for (j = 0; j<surfh; j += celh)
   {
      for (i = 0; i < surfw; i += celw)
      {
         putcel(i,j,cachedgfxdest,rdest,dest,DDBLTFAST_NOCOLORKEY);
         k++;
      }
   }
   
   rdest.left = 0;
   rdest.right = surfw;
   rdest.top = 0;
   rdest.bottom = surfh;

   if (mask == NULL)
      throw new BubException("Teleport: mask is NULL!");
   puttint50(0,0,cachedgfxdest,rdest,mask,tintcolordest);
   delete [] mask;
}

//-------------------------------------------------------------------------
// Prevali cely predpocitany teleport tintem50
//-------------------------------------------------------------------------
void Teleport::tintPrepared()
{
   int framebit = 1 << frames.frameid;
   if (!(redden & framebit))
   {
      TintMask mask = surface2mask(gfx);
      if (cachedgfx != NULL)
      {
         tint_redalert(cachedgfx,frameRect);
         puttint50(frameRect.left,frameRect.top,cachedgfx,frameRect,mask,tintcolor);
      }
      if (cachedgfxdest != NULL)
      {
         tint_redalert(cachedgfxdest,frameRect);
         puttint50(frameRect.left,frameRect.top,cachedgfxdest,frameRect,mask,tintcolordest);
      }
      redden |= framebit;
   }
}


//---------------------------------------------------------------------------
// Vykresluje teleport
//---------------------------------------------------------------------------
void Teleport::Paint(Surface dest)
{
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
   if (cachedgfx != NULL)
   {
      putcel(newx,newy,dest,frameRect,cachedgfx,DDBLTFAST_NOCOLORKEY);
      SET_USEDMAP(newx,newy);
      SET_USEDMAP(xright,newy);
      SET_USEDMAP(newx,ybottom);
      SET_USEDMAP(xright,ybottom);
		// pro jistotu pidan jet jedna podmnka - V. 2004
		// po oprav prepareDesc u asi nen nutn - V. 2004. v2
      //if ((type != FTPORT) && (cachedgfxdest != NULL))
		if (cachedgfxdest != NULL)
      {
         putcel(destx,desty,dest,destRect,cachedgfxdest,DDBLTFAST_NOCOLORKEY);
         xright = destx + xsize - 1;
         ybottom = desty + ysize - 1;
         SET_USEDMAP(destx,desty);
         SET_USEDMAP(xright,desty);
         SET_USEDMAP(destx,ybottom);
         SET_USEDMAP(xright,ybottom);
      }
   }
   else
   {
      puttint50(newx,newy,dest,frameRect,properties.member[ID_TPORT]->tintmask,
      tintcolor);
      SET_USEDMAP(newx,newy);
      SET_USEDMAP(xright,newy);
      SET_USEDMAP(newx,ybottom);
      SET_USEDMAP(xright,ybottom);
      if (type != FTPORT)
      {
         puttint50(destx,desty,dest,destRect,properties.member[ID_TPORT]->tintmask,
         tintcolordest);
         xright = destx + xsize - 1;
         ybottom = desty + ysize - 1;
         SET_USEDMAP(destx,desty);
         SET_USEDMAP(xright,desty);
         SET_USEDMAP(destx,ybottom);
         SET_USEDMAP(xright,ybottom);
      }
   }
}

//-------------------------------------------------------------------------
// posunuje se na dalsi frame a nastavuje okraje celu v surface s grafikou
//-------------------------------------------------------------------------
void Teleport::jumpNext()
{
   int l,t;
   frames.jumpNext();                       
   l = (frames.frameid % celr) * celw + xoff;
   t = (frames.frameid / celr) * celh + yoff;
   frameRect.left = l;
   frameRect.top = t;
   frameRect.right = l + xsize;  
   frameRect.bottom = t + ysize;
   if (type != FTPORT)
   {
      l = ((offset_dest - frames.frameid) % celr) * celw + xoff;
      t = ((offset_dest - frames.frameid) / celr) * celh + yoff;
      destRect.left = l;
      destRect.top = t;
      destRect.right = l + xsize;  
      destRect.bottom = t + ysize;
   }
}

//-------------------------------------------------------------------------
// maze teleport
//-------------------------------------------------------------------------
void Teleport::Clear(Surface dest, Surface bckg)
{
}

//-------------------------------------------------------------------------
// transportuje pasazera
//-------------------------------------------------------------------------
void Teleport::Transport(Creature *passenger)
{
   sound_device->Play(11);
   if (type == FTPORT)
   {
      ((Player *) passenger)->sheet.AddPoints(500);
      strcpy(levelname,nextlvls+9*destx);
      player1->state |= INBUBBLE;
      player2->state |= INBUBBLE;
      ARRAY_REMOVE(monsters,monsters_ctrl,player1);
      ARRAY_REMOVE(monsters,monsters_ctrl,player2);
      exitgame = true;
      vanish = 2;
      return;
   }
   passenger->newx = destx;
   passenger->newy = desty;
   passenger->mul_x = passenger->newx << SHIFT_MOVE + MUL_HALF;
   passenger->mul_y = passenger->newy << SHIFT_MOVE + MUL_HALF;
}

//-------------------------------------------------------------------------
// zjistuje kolize s objekty
//-------------------------------------------------------------------------
int Teleport::CollideWith(Sprite *what)
{
   if (abs(newx-what->newx) < (what->xsize/2))
      if (abs(newy-what->newy) < (what->ysize/2))
         return 1;
      return 0;
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Teleport::~Teleport()
{
   if (cachedgfx != NULL)
   {
      surfaceManager->remove(cachedgfx,true);
      cachedgfx = NULL;
   }
   if (cachedgfxdest != NULL)
   {
      surfaceManager->remove(cachedgfxdest,true);
      cachedgfxdest = NULL;
   }
}