//**************************************************************************
//                             Various utils                               
//**************************************************************************
//  Written by : Vojvo
//  Language   : VisuC 6 
//  Version    : 1.0.1
//  Last update: 14.04.2000
//**************************************************************************

#include "utils.h"

#ifdef PROFILER_DEBUG
PerformanceProfiler *profiler = NULL;
#endif

char pom_string[256];

//-------------------------------------------------------------------------
// prevadi cisla na text
//-------------------------------------------------------------------------
void numtos(long numb, int width, char *target)
{
   target[width] = 0;
   for(int i=width; i>0; i--)
   {
      target[i-1] = (numb % 10) + '0';
      numb /= 10;
   }
}

//-------------------------------------------------------------------------
//------------------------------ struct Timer -----------------------------
//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Timer::Timer()
{
   options(1000,0,100,true,true);
}

//-------------------------------------------------------------------------
// konstruktor s parametry
//-------------------------------------------------------------------------
Timer::Timer(int timemax,unsigned long xmin,unsigned long xmax,
             bool cutmax,bool ltor)
{
   options(timemax,xmin,xmax,cutmax,ltor);
}

//-------------------------------------------------------------------------
// nastavuje vlastnosti timeru
//-------------------------------------------------------------------------
void Timer::options(int timemax,unsigned long xmin,unsigned long xmax,
                    bool cutmax,bool ltor)
{
   this->timemax = timemax;
   this->xmin = xmin;
   this->xmax = xmax;
   this->cutmax = cutmax;
   this->ltor = ltor;
   
   xcurrent = ltor ? xmin : xmax;
   timeunit = timemax / (xmax - xmin);
   timecurrent = timemax;
}


//-------------------------------------------------------------------------
// nastavuje casovac na zadanou hodnotu
//-------------------------------------------------------------------------
void Timer::set(int value)
{
   timecurrent = value;
   if (timecurrent < 0)
      timecurrent = 0;
   if (timecurrent > timemax)
   {
      if (!cutmax)
      {
         timemax = timecurrent;
         timeunit = timemax / (xmax-xmin);
      }
      timecurrent = timemax;
      xcurrent = ltor ? xmin : xmax;
   }
   else
   {
      if (ltor)
      {
         xcurrent = xmax - timecurrent/timeunit;
      }
      else
      {
         xcurrent = xmin + timecurrent/timeunit;
      }
   }
}

//-------------------------------------------------------------------------
// snizuje timer o zadany pocet milisekund
//-------------------------------------------------------------------------
int Timer::decrease(int amount)
{
   timecurrent -= amount;
   if (timecurrent < 0)
      timecurrent = 0;

   if (ltor)
   {
      xcurrent = xmax - timecurrent/timeunit;
   }
   else
   {
      xcurrent = xmin + timecurrent/timeunit;
   }
   return timecurrent;
}

//-------------------------------------------------------------------------
// zvysuje timer o zadany pocet milisekund
//-------------------------------------------------------------------------
int Timer::increase(int amount)
{
   timecurrent += amount;
   if (timecurrent > timemax)
   {
      if (!cutmax)
      {
         timemax = timecurrent;
         timeunit = timemax / (xmax-xmin);
      }
      timecurrent = timemax;
      xcurrent = ltor ? xmin : xmax;
   }
   else
   {
      if (ltor)
      {
         xcurrent = xmax - timecurrent/timeunit;
      }
      else
      {
         xcurrent = xmin + timecurrent/timeunit;
      }
   }
   return xcurrent;
}


