/*
*--------------------------------------------------------------------
*
* Application
*
* Aplikace - nejvyssi uroven prubehu hry
*
* verze   0.99.4 - 6.12.1999
* autori  Vojvo & G.
*
*--------------------------------------------------------------------
* zmeny:
* 0.99.4 - uprava pro AVIPlayer
* 0.99.3 - oprava spatneho nacitani z neexistujicich hodnot registru
*        - z registru se cte reverse stereo
* 0.99.2 - oprava nacitani hlasitosti z registru
* 0.99.1 - demoverze
* 0.11.2 - zabity onTimer a timer jako takovy
* 0.10.2 - stav klavesnice se necte v onTimer (timer se asi vyhodi)
* 0.9.8 / pridano osetrovani switche darken
* 0.9.4 - toppers jiz nejsou gamestage => ubyly herni stavy 9-11
*--------------------------------------------------------------------
*/

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <windowsx.h>
#include <vfw.h>
#include "version.h"
#include "types.h"
#include "graphDD.h"
#include "keyboard.h"
#include "surfaceManager.h"
#include "exceptions.h"
#include "r16.h"
#include "font.h"
#include "registry.h"
#include "resource.h"
#include "utils.h"
#include "application.h"


Application *app;

const char * Application::szAppName = "Bubliny for Windows 9x";
const char * Application::szWCName = "Bubwiny";

const char *advert_shotformat = "data\\advert%d.r16";
const char *theend_shotformat = "data\\theend%d.r16";


//----------------------------------------------------------------------------
// obsluha zprav dialogu console
//----------------------------------------------------------------------------
BOOL CALLBACK ConsoleDlgProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, 
                             LPARAM lParam)
{
   switch (uMsg)
   {
   case WM_INITDIALOG:
      app->consoleDlg_onInitDialog(hWndDlg,wParam,lParam);
      return (FALSE);
   case WM_COMMAND:
      app->consoleDlg_onCommand(hWndDlg,wParam,lParam);
      return (TRUE);
   case WM_CLOSE:
      app->m_hWndConsole = 0;
      app->m_hWndConsoleContent = 0;
      DestroyWindow(hWndDlg);
      return (0);
   }
   return (FALSE);
}

//-------------------------------------------------------------------------
// implicitni konstruktor
//------------------------------------------------------------------------- 
Application::Application()
{
   m_bDDActive = false;
   m_bInitialized = false;
   m_hWnd = 0;
   game = NULL;
   init = NULL;
   menu = NULL;
   fader = NULL;
   interscreen = NULL;
   slideShow = NULL;
   aviPlayer = NULL;
   strcpy(m_szDataDir,"data\\");
   strcpy(levelname,"");
   m_bConsole = false;
   m_hWndConsole = 0;
   m_hWndConsoleContent = 0;
   state = 0;
   stage = NULL;
   destroyed = false;
#ifdef PROFILER_DEBUG
   profiler = new PerformanceProfiler(20);
#endif
}

//-------------------------------------------------------------------------
// cteni hodnot z registru
//-------------------------------------------------------------------------
void Application::readRegistry()
{
   DWORD dwVol = 255;
   Registry *reg = new Registry();
   strcpy(s_password,"Kreslo");
   *((DWORD *)s_key[0]) = 0x02031210;
   *((DWORD *)s_key[1]) = 0x9dc8cdcb;
   strcpy(s_name[0],"Prvni");
   strcpy(s_name[1],"Druhy");
   s_drops = true;
   s_darken = false;
   s_test = false;
   s_testsymbol = false;
   s_testholy = false;
   s_solidbubbles = false;
   s_reverse_stereo = false;
   s_vol = 255;
   levelname[0] = '\x0';
#ifdef DEMOVERSION
   reg->open("Bubliny",KEY_READ);
#else
   reg->open("Bubliny FULL",KEY_READ);
#endif
   if (reg->opened)
   {
      reg->readString("Heslo",s_password,12);
      if (s_password[0] == '\x0')
         strcpy(s_password,"kreslo");
      
      reg->readString("Jmeno 1",s_name[0],32);
      if (s_name[0][0] == '\x0')
         strcpy(s_name[0],"Prvni");
      s_name[0][11] = '\x0';   // omezeni na 11 znaku
      
      reg->readString("Jmeno 2",s_name[1],32);
      if (s_name[1][0] == '\x0')
         strcpy(s_name[1],"Druhy");
      s_name[1][11] = '\x0';   // omezeni na 11 znaku
      
      pom_string[0] = '\x0';
      reg->readString("Console",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            m_bConsole = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Drops",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_drops = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Darken",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_darken = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Test",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_test = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("Obrysove",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_solidbubbles = true;
      }
      
      pom_string[0] = '\x0';
      reg->readString("RevStereo",pom_string,10);
      if (pom_string[0] != '\x0')
      {
         if (!stricmp(pom_string,"Ano"))
            s_reverse_stereo = true;
      }
      
      reg->readDWORD("Hlasitost",&dwVol);
      if (dwVol <= 10)
         s_vol = (BYTE)dwVol;
      
      reg->readDWORD("Ovladani 1",(DWORD *)s_key[0]);
      reg->readDWORD("Ovladani 2",(DWORD *)s_key[1]);
      
      reg->close();
   }
   delete reg;
}

//-------------------------------------------------------------------------
// zapis hodnot do registru
//-------------------------------------------------------------------------
void Application::writeRegistry()
{
   Registry *reg = new Registry();
#ifdef DEMOVERSION
   reg->create("Bubliny");
#else
   reg->create("Bubliny FULL");
#endif
   reg->writeString("Heslo",s_password);
   reg->writeString("Jmeno 1",s_name[0]);
   reg->writeString("Jmeno 2",s_name[1]);
   reg->writeString("Console",m_bConsole ? "Ano":"Ne");
   reg->writeString("Drops",s_drops ? "Ano":"Ne");
   reg->writeString("Darken",s_darken ? "Ano":"Ne");
   reg->writeString("Test",s_test ? "Ano":"Ne");
   reg->writeString("Obrysove",s_solidbubbles ? "Ano":"Ne");
   reg->writeString("RevStereo",s_reverse_stereo ? "Ano":"Ne");
   if (s_vol != 255)
      reg->writeDWORD("Hlasitost",(DWORD)s_vol);
   reg->writeDWORD("Ovladani 1",*((DWORD *)s_key[0]));
   reg->writeDWORD("Ovladani 2",*((DWORD *)s_key[1]));
   reg->close();
   delete reg;
}

//-------------------------------------------------------------------------
// inicializace aplikace
//-------------------------------------------------------------------------
bool Application::initInstance(HINSTANCE hInstance, LPSTR lpCmdLine, int nCmdShow, 
                               WNDPROC lpWndProc)
{   
   WNDCLASSEX wcex;
   
   // Set up and register window class
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.style = CS_HREDRAW | CS_VREDRAW;
   wcex.lpfnWndProc = lpWndProc;
   wcex.cbClsExtra = 0;
   wcex.cbWndExtra = 0;
   wcex.hInstance = hInstance;
#ifdef DEMOVERSION
   wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBDEMO1));
   wcex.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBDEMO1));
#else
   wcex.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBFULL1));
   wcex.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON_BUBFULL1));
#endif
   wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
   wcex.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);;
   wcex.lpszMenuName = szAppName;
   wcex.lpszClassName = szWCName;
   RegisterClassEx(&wcex);
   
   MCIWndRegisterClass();
   
   SystemParametersInfo(SPI_GETWORKAREA,0,&m_rcDispArea,0);
   m_rcDispArea.right = GetSystemMetrics(SM_CXSCREEN);
   m_rcDispArea.bottom = GetSystemMetrics(SM_CYSCREEN);
   m_hInstance = hInstance;
   
   // Create a window
   m_hWnd = CreateWindowEx(WS_EX_TOPMOST,
      szWCName,
      szAppName,
      WS_POPUP,
      0,
      0,
      m_rcDispArea.right,
      m_rcDispArea.bottom,
      NULL,
      NULL,
      hInstance,
      NULL);
   
   if (!m_hWnd)
      return false;
   
   if (m_bConsole)
   {
      m_hWndConsole = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_DLG_CONSOLE),
         NULL, (DLGPROC) ConsoleDlgProc);
      ShowWindow(m_hWndConsole,SW_SHOWNORMAL);
   }
   
   ShowWindow(m_hWnd, nCmdShow);
   UpdateWindow(m_hWnd);
   SetFocus(m_hWnd);
   
   try
   {
      initDD(app->getMainWindow(),640,480,16,0);
      sprintf(pom_string,"Grafika je 5%d5",gBits);
      app->sayConsole(pom_string);
      kbd.init(hInstance,app->getMainWindow());
      kbd.acquire(); 
      surfaceManager->setDirectory(m_szDataDir);
      
      init = new Initializer();
      init->load();
      game = new Game();
      menu = new Menu();
      fader = new Fader();
      interscreen = new InterScreen();
      slideShow = new SlideShow();
      slideShow->shotformat = advert_shotformat;
      aviPlayer = new AVIPlayer();
      
      stateChange();
   }
   catch (Exception *exc)
   {
      DEBUG_MESSAGE("E1:");
      destroyInstance();
      MessageBox(m_hWnd,exc->msg,"Error",MB_OK);
      PostMessage(m_hWnd,WM_CLOSE,0,0);
   }
   catch (...)
   {
      DEBUG_MESSAGE("E2:");
      destroyInstance();
      MessageBox(m_hWnd,"unknown error","Error",MB_OK);
      PostMessage(m_hWnd,WM_CLOSE,0,0);
   }
   return true;
}

//-------------------------------------------------------------------------
// cinnost aplikace v dobe, kdy nechodi zadne zpravy
//-------------------------------------------------------------------------
void Application::update()
{
   bool isLost;
   
   if (m_bDDActive)
   {
      try 
      {
         isLost = true;
         try 
         {
            if (stage != NULL)
               stage->update();
            isLost = false;
         }
         catch(IsLostException *)
         {
            DEBUG_MESSAGE("update: surface lost!");
            surfaceManager->restore();
            stage->restore();
            ddRestored = false;
         }
      }
      catch (Exception *exc)
      {         
         DEBUG_MESSAGE("E3:");
         destroyInstance();
         MessageBox(m_hWnd,exc->msg,"Error",MB_OK);
         PostMessage(m_hWnd,WM_CLOSE,0,0);
      }
      catch (...)
      {
         DEBUG_MESSAGE("E4:");
         destroyInstance();
         MessageBox(m_hWnd,"unknown error","Error",MB_OK);
         PostMessage(m_hWnd,WM_CLOSE,0,0);
      }
   }
   else
   {
      WaitMessage();
   }
}

//-------------------------------------------------------------------------
// uklidi aplikaci pred ukoncenim
//-------------------------------------------------------------------------
void Application::destroyInstance()
{
   if (destroyed)
      return;
   DEBUG_MESSAGE("Destroying...\n");
   destroyed = true;   

   kbd.done(); 
   unload();
   if (lpDD != NULL)
      doneDD();
#ifdef PROFILER_DEBUG
   if (profiler != NULL)
      profiler->makeReport();
   SAFE_DELETE(profiler);
#endif
}

//-------------------------------------------------------------------------
// provadi cinnost pri aktivaci aplikace
//-------------------------------------------------------------------------
void Application::onActivateApp(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   // Pause if minimized or not the top window
   m_bDDActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
}

//-------------------------------------------------------------------------
// reakce na stisk klavesy v aplikaci
//-------------------------------------------------------------------------
void Application::onKeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
}

//-------------------------------------------------------------------------
// reakce na zpravu, vyzyvajici aplikaci k nastaveni kurzoru
//-------------------------------------------------------------------------
void Application::onSetCursor(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   SetCursor(NULL);
}

//-------------------------------------------------------------------------
// reakce na zpravu, informujici o zmene v prehravani AVI
//-------------------------------------------------------------------------
void Application::onAVIMode(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
   if (lParam == MCI_MODE_STOP)
      aviPlayer->done = true;
}

//-------------------------------------------------------------------------
// odstranuje herni objekty
//-------------------------------------------------------------------------
void Application::unload()
{   
   DEBUG_MESSAGE("Unload: S");
   if (stage != NULL)
   {
      stage->stageError = true;
      stage->stop();
      stage = NULL;
   }
   kbd.setLastScanSet(NULL,0);
   kbd.lastASCIIReset();
   kbd.lastScanReset();
   
   
   DEBUG_MESSAGE("G");
   SAFE_DELETE(game);   
   
   DEBUG_MESSAGE("I");
   SAFE_DELETE(interscreen);   
   
   DEBUG_MESSAGE("S");
   SAFE_DELETE(slideShow);   
   
   DEBUG_MESSAGE("F");
   SAFE_DELETE(fader);   
   
   DEBUG_MESSAGE("M");
   SAFE_DELETE(menu);
   
   DEBUG_MESSAGE("A");
   SAFE_DELETE(aviPlayer);
   
   DEBUG_MESSAGE("I\n");
   SAFE_DELETE(init);
}

//-------------------------------------------------------------------------
// zmeni usek hry podle preddefinovaneho 'automatu'
//-------------------------------------------------------------------------
void Application::stateChange(int reason)
{
   if (stage != NULL)
   {
      stage->stageError = false;
      stage->stop();
   }
   switch (state)
   {
   case 0:      
      DEBUG_MESSAGE("Finished state 0\n");
      init->start();
      stage = init;
      state = 1;
      break;
   case 1:      
      DEBUG_MESSAGE("Finished state 1\n");
      stage = fader;
      fader->setFadeDir(FADE_OUT);
      fader->start();
      state = 2;
      break;
   case 2:       
      DEBUG_MESSAGE("Finished state 2\n");
      stage = aviPlayer;
      clear(backbuffer,0,NULL);
      aviPlayer->setAVI("data\\intro.avi",480,320);
      aviPlayer->start();
      state = 12;
      break;
   case 3:       
      DEBUG_MESSAGE("Finished state 3\n");
      stage = menu;
      state = 4;
      break;
   case 4:       
      DEBUG_MESSAGE("Finished state 4\n");
      if (reason == REASON_EXIT)
      {
#ifdef DEMOVERSION
         slideShow->shotnum = 2;
#else
         slideShow->shotnum = 3;
#endif
         slideShow->shotformat = advert_shotformat;
         slideShow->shotlast = 999;
         slideShow->start();
         stage = fader;
         fader->setFadeDir(FADE_IN);
         fader->start();
         state = 13;
         break;
      }
      stage = fader;
      fader->setFadeDir(FADE_OUT);
      fader->start();
      game->StartGame();
      state = 5;
      break;
   case 5:       
      DEBUG_MESSAGE("Finished state 5\n");
      game->startLevel();
      if (levelname[0] == '\x0')
      {
         menu->start();
         state = 3;
      }
      else
      {
         interscreen->start();
         state = 6;
      }
      stage = fader;
      fader->setFadeDir(FADE_IN);
      fader->start();       
      break;
   case 6:       
      DEBUG_MESSAGE("Finished state 6\n");
      stage = interscreen;    
      state = 7;
      break;
   case 7:       
      DEBUG_MESSAGE("Finished state 7\n");
      game->start();
      stage = game;    
      state = 8;
      break;
   case 8:      
      DEBUG_MESSAGE("Finished state 8\n");
      if (reason == REASON_NEXTLEVEL)
      { 
         stage = fader;
         fader->setFadeDir(FADE_OUT);
         fader->start();
         state = 5;
         break;
      }
      else if (reason == REASON_THEEND)
      {
         menu->menu = &(menu->toppermenu);
         game->DoneGame();
         stage = fader;
         fader->setFadeDir(FADE_OUT);
         fader->start();
         state = 9;
         break;
      }
      else
      {
         stage = fader;
         fader->setFadeDir(FADE_OUT);
         fader->start();
         if (reason == REASON_NOTOPPERS)
         {
            game->DoneGame();
            state = 12;
         }
         else
         {
            menu->menu = &(menu->toppermenu);
            game->DoneGame();
            state = 12;
         }
      }
      break;
   case 9:      
      DEBUG_MESSAGE("Finished state 9\n");
#ifdef DEMOVERSION
      slideShow->shotformat = advert_shotformat;
      slideShow->shotnum = 1;
      slideShow->shotlast = 1;
#else
      slideShow->shotformat = theend_shotformat;
      slideShow->shotnum = 1;
      slideShow->shotlast = 999;
#endif
      slideShow->start();

      stage = fader;
      fader->setFadeDir(FADE_IN);
      fader->start();
      state = 10;
      break;
   case 10:      
      DEBUG_MESSAGE("Finished state 10\n");
      stage = slideShow;
      state = 11;
      break;
   case 11:      
      DEBUG_MESSAGE("Finished state 11\n");
      state = 12;
      kbd.clearKey(DIK_ESCAPE);
      kbd.clearKey(DIK_RETURN);
      kbd.clearKey(DIK_SPACE);
      stage = fader;
      fader->setFadeDir(FADE_OUT);
      fader->start();
      break;
   case 12:      
      DEBUG_MESSAGE("Finished state 12\n");
      menu->start();
      if (menu->menu == &(menu->toppermenu))
         menu->startToppers();
      stage = fader;
      fader->setFadeDir(FADE_IN);
      fader->start();
      state = 3;
      break;
   case 13:      
      DEBUG_MESSAGE("Finished state 13\n");
      stage = slideShow;
      state = 14;
      break;
   case 14:     
      DEBUG_MESSAGE("Finished state 14\n");
      stage = NULL; 
      PostMessage(m_hWnd,WM_QUIT,0,0);
      break;
   }
}

//-------------------------------------------------------------------------
// inicializuje dialog console
//-------------------------------------------------------------------------
void Application::consoleDlg_onInitDialog(HWND hWndDlg,WPARAM wParam,LPARAM lParam)
{
   m_hWndConsoleContent = GetDlgItem(hWndDlg,IDC_LI_CONSOLE);
   //   SetFocus(GetDlgItem(hWndDlg,IDC_ED_LINES));
   //   SetDlgItemInt(hWndDlg,IDC_ED_LINES,CONSOLE_SIZE,FALSE);
   ListBox_ResetContent(m_hWndConsoleContent);
   console_lines = 0;
}

//-------------------------------------------------------------------------
// reakce na akce v dialogu parametru
//-------------------------------------------------------------------------
void Application::consoleDlg_onCommand(HWND hWndDlg,WPARAM wParam,LPARAM lParam)
{
}

//-------------------------------------------------------------------------
// reakce na akce v dialogu parametru
//-------------------------------------------------------------------------
void Application::sayConsole(char *sayWhat)
{
   if (m_hWndConsoleContent == 0)
      return;
   if (console_lines == CONSOLE_SIZE)
   {
      ListBox_DeleteString(m_hWndConsoleContent,0);
      console_lines--;
   }
   ListBox_AddString(m_hWndConsoleContent,sayWhat);
   console_lines++;
}

