#ifndef APPLICATION_H_BUBWINY
#define APPLICATION_H_BUBWINY

#include "dxconsts.h"
#include <ddraw.h>
#include "consts.h"
#include "types.h"
#include "gamestage.h"
#include "game.h"
#include "menu.h"
#include "initializer.h"
#include "fader.h"
#include "interscreen.h"
#include "slideshow.h"
#include "aviplayer.h"

class Application {
   static const char *szAppName;
   static const char *szWCName;

   HWND m_hWnd;
   HINSTANCE m_hInstance;
   RECT m_rcDispArea;
   int console_lines;
   bool m_bDDActive;
   bool m_bInitialized;
   char m_szDataDir[128];
   bool m_bConsole;
   int state;
   GameStage *stage;
   Game *game;
   Menu *menu;
   Fader *fader;
   InterScreen *interscreen;
   Initializer *init;
   SlideShow *slideShow;
   AVIPlayer *aviPlayer;
   bool destroyed;
 
public:
   HWND m_hWndConsole, m_hWndConsoleContent;

   Application();
	
   bool initInstance(HINSTANCE hInstance, LPSTR lpCmdLine, int nCmdShow, 
      WNDPROC lpWndProc);
   void destroyInstance();
   void update();
   void unload();

   HWND getMainWindow() { return m_hWnd; };
   HWND getConsoleWindow() { return m_hWndConsole; };
   void stateChange(int reason = REASON_DEFAULT);
   void readRegistry();
   void writeRegistry();
   void sayConsole(char *sayWhat);

   void onActivateApp(HWND hWnd, WPARAM wParam, LPARAM lParam);
   void onSetCursor(HWND hWnd, WPARAM wParam, LPARAM lParam);
   void onKeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam);
   void onAVIMode(HWND hWnd, WPARAM wParam, LPARAM lParam);

   void consoleDlg_onInitDialog(HWND hWndDlg,WPARAM wParam,LPARAM lParam);
   void consoleDlg_onCommand(HWND hWndDlg,WPARAM wParam,LPARAM lParam);

   ~Application() { destroyInstance(); };

   friend class Menu;
   friend class Initializer;
   friend class Game;
   friend class Fader;
   friend class InterScreen;
   friend class AVIPlayer;
   friend void StartToppers();
   friend void StopToppers();
   friend void CheckValidKeys();
};


extern Application *app;

#endif
