//**************************************************************************
//                 Bubble related classes implementation                  
//**************************************************************************
// Written by : Vojvo & Gunny    
// Language   : Visuc6 + DX6.1  
// Version    : 0.10.2
// Last update: 17.11.1999 
//**************************************************************************
// changes
// 0.10.3 - bubliny pouzivaji kopii backgroundu v pameti
// 0.10.2 - plne a prazdne bubliny praskaji s jinym samplem 
// 0.9.3 - skrabani na bublinu zevnitr
//       - domino pusobi i na symbolove bubliny
// 0.8.2 - pridany zvuky
// 0.8.1 - pridany symbolove bubliny
// 0.7.2 - opraven bug se spatnym volanim jumpNext z Bub konstruktoru u 
//         Release buildu

#include <stdio.h>
#include "gonio.h"
#include "exceptions.h"
#include "utils.h"
#include "graphdd.h"
#include "map.h"
#include "wsound.h"
#include "drops.h"
#include "game.h"
#include "application.h"
#include "sprites.h"

#define INERTIA_FIRE 18
#define INERTIA_NORM 2


//##########################################################################
//---------------------------------- Bub -----------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bub::Bub (int onx, int ony, Property *prop)
{
   type = BUBBLE;
   id = prop->id;
   fTgs = prop->fTgs;
   if (id == ID_BUBBLE || id == ID_SBUBBLE)
   {
      xsize = glasses[0]->GetWidth();
      ysize = glasses[0]->GetHeight();
   }
   else
   {
      xsize = prop->xsize;
      ysize = prop->ysize;
      gfx = prop->surf;
      celw = prop->r16h->dwCelWidth;
      celh = prop->r16h->dwCelHeight;
      celr = prop->r16h->byCelRow;
   }
   b_maxy = PLAYFIELD_MAX_Y - ysize;
   b_maxx = PLAYFIELD_MAX_X - xsize;
   frames.frames = &(prop->frames);
   direction = DIR_L;
   frames.setDirection(DIR_L);
   frames.setChain(CHAIN_STAND);
   collision = 0;
   newx = oldx = onx;
   newy = oldy = ony;
   mul_x = newx << SHIFT_MOVE + MUL_HALF;
   mul_y = newy << SHIFT_MOVE + MUL_HALF;
   movex = movey = 0;
   topx = prop->speed;
   topy = prop->speed;
   bottomx = -topx;
   bottomy = -topy;
   sort_priority = 0;
   is_stable = 0;
   clipped = false;
   counter = BUB_LIFETIME;
   state = 0;
   sender = NULL;
   tint = TNT_GLASS;
   bubbuf = NULL;
   computed = false;
   monsterbehind = false;
}

//-------------------------------------------------------------------------
// vraci TRUE, pokud je bublina pomala
//-------------------------------------------------------------------------
int Bub::IsSlow()
{
   return (movex >= bottomx && movex <= topx);
}

//==========================================================================
int Bub::Move()
{
   int result = 0;
   
   SlowIt(INERTIA_NORM,INERTIA_NORM);

   // vitr a zpomalovani rychle bubliny
   if (movex >= bottomx && movex <= topx)
   {
      if (windmap != NULL)
      {
         movex += wndmap_value(newx,newy-PLAYFIELD_MIN_Y,windmap);
         if (movex > topx) movex = topx;
         if (movex < bottomx) movex = bottomx;
      }    
   }
   else
      SlowIt(INERTIA_FIRE,0);
   
   mul_x += movex;
   newx = mul_x >> SHIFT_MOVE;
   if (newx < PLAYFIELD_MIN_X) {
      newx = PLAYFIELD_MIN_X;
      mul_x = newx << MUL_MOVE;
      movex = -movex;
      result |= SP_COL_L;
   }
   else if (newx > b_maxx) {
      newx = b_maxx;
      mul_x = newx << SHIFT_MOVE;
      movex = -movex;
      result |= SP_COL_R;
   }

   // vitr v y-smeru
   if (movex >= bottomx && movex <= topx && windmap != NULL)
   {
      movey += wndmap_value(newx,newy-PLAYFIELD_MIN_Y,windmap+WNDMAP_SIZE);
      if (movey > topy) movey = topy;
      if (movey < bottomy) movey = bottomy;
   }
   
   mul_y += movey;
   newy = mul_y >> SHIFT_MOVE;
   if (newy < PLAYFIELD_MIN_Y) {
      newy = PLAYFIELD_MIN_Y;
      mul_y = newy << SHIFT_MOVE;
      movey = -movey;
      result |= SP_COL_D;
   }
   else if (newy>b_maxy) { 
      newy = b_maxy;
      mul_y = newy << SHIFT_MOVE;
      movey = -movey;
      result |= SP_COL_U;
   }
   
   counter--;
   if (counter == BUB_BLOWTIME)
   {
      result |= BUB_BLOWSTART;
   }
   
   if (!counter)
   {
      result |= BUB_BLOWEND;
   }
   
   return result;
}

//==========================================================================
int Bub::Domino()
// Dominovy efekt
// otestuje, jestli jsou pobliz nejake pomale nebo symbolove bubliny
// pokud jsou, nastavi je tesne pred prasknuti a jako sender nastavi
// stejny pointer jako u bubliny, ktera prave praskla
// vraci pocet bublin, ktere exploze ovlivnila
{
   int res=0;
   //nejdriv otestuju normalni bubliny
   int bi=0;
   while (bi < sbubbles_ctrl.count)
   {
      Bubble *pom_b = ARRAY_AT(sbubbles,bi);
      int ax = abs(newx-pom_b->newx);
      if (ax < 44 && abs(newy-pom_b->newy) < ycircle40[ax])
      {
         if (pom_b->counter>BUB_DOMINATED)
         {
            res++;
            pom_b->counter=BUB_DOMINATED;
            pom_b->sender=sender;
         }
      }
      bi++;
   }
   //ted zkusim, co to udela se symbolovyma...
   bi=0;
   while (bi < symbubs_ctrl.count)
   {
      SymbolBubble * pom_s = ARRAY_AT(symbubs,bi);
      int ax = abs(newx-pom_s->newx);
      if (ax < 44 && abs(newy-pom_s->newy) < ycircle40[ax])
      {
         if (pom_s->counter>BUB_DOMINATED)
         {
            res++;
            pom_s->counter=BUB_DOMINATED;
            pom_s->sender=sender;
         }
      }
      bi++;
   } 

   return res;
}

//===========================================================================
int Bub::CollideWith(Sprite *what)
// Testovani kolizi a rozstrakavani
{
   int result,sx,sy,ax,ay,sgnx,sgny;
   WORD normx, normy;
   int diff_x,diff_y;
   
   if (state & BLOWING)
      return 0;
      
   result = 0;
   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   switch (what->type) {
   case PLAYER: if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                   diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                  if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                      diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))                  
                      if (movex >= bottomx && movex <= topx)
                         result = 1;
                      break;
   case BUBBLE: sx = newx - what->newx;
      sy = newy - what->newy;
      ax = abs(sx);
      ay = abs(sy);
      sgnx = (sx > 0) ? 1:-1;
      sgny = (sy > 0) ? 1:-1;
      if (ax < 32 && ay < ycircle32[ax])
      {
         normx = normx50(ax,ay);
         normy = normy50(ax,ay);
         movex = sgnx * normx;
         movey = sgny * normy;
         what->movex = (-sgnx) * normx;
         what->movey = (-sgny) * normy;
      }
      result = 1;
   }
   return result;
}

//-------------------------------------------------------------------------
// testuje jestli je neco za bublinou
//-------------------------------------------------------------------------
void Bub::checkBehind(Sprite *what)
{
   int diff_x,diff_y;

   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   if (diff_x < 0 && -diff_x < what->xsize || diff_x >= 0 && diff_x < xsize)
      if (diff_y < 0 && -diff_y < what->ysize || 
         diff_y >= 0 && diff_y < ysize)                  
         if (movex >= bottomx && movex <= topx)
         {
            monsterbehind = true;
         }
}


//##########################################################################
//-------------------------------- Bubble ----------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Bubble::Bubble(int onx, int ony, Property *prop) 
: monster(NULL), whose(NULL), Bub (onx,ony,prop)
{
   nextscrape = 0xFFFF;
   selfblowing = 0;
   jumpNext();   
}

//-------------------------------------------------------------------------
// testovani kolizi
//-------------------------------------------------------------------------
int Bubble::CollideWith(Sprite *what)
{
   int result;
   int diff_x,diff_y;
   
   if (state & BLOWING)
      return 0;
   
   result = 0;
   diff_x = what->newx - newx;
   diff_y = what->newy - newy;
   switch (what->type) {
   case MONSTER: if (!monster && counter < BUB_START)
                    if (movex < bottomx || movex > topx)
                       if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                           diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                          if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                              diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))
                             result = 1;
                 break;
   case PLAYER: if (diff_x < 0 && -diff_x < (what->xsize-BUBBLEMONSTER_HITTOLX) || 
                   diff_x >= 0 && diff_x < (xsize-BUBBLEMONSTER_HITTOLX))
                  if (diff_y < 0 && -diff_y < (what->ysize-BUBBLEMONSTER_HITTOLY) || 
                      diff_y >= 0 && diff_y < (ysize-BUBBLEMONSTER_HITTOLY))
                     if (movex >= bottomx && movex <= topx)
                         result = 1;
                     else
                        if (!monster)
                        {
                           Player *pom_p = (Player *)what;
                           if (!(pom_p->state & (BUBBLE_RESISTANT)))
                           {
                              if (((movex > 0) ^ (pom_p->direction == DIR_R)) 
                                 || (s_catchy && whose != pom_p))
                              {
                                 pom_p->state |= INBUBBLE;
                                 result = 2;
                              }
                           }
                        } 
                break;
   case BUBBLE: result = Bub::CollideWith(what);
   }
   return result;
}

//===========================================================================
int Bubble::Move()
{
   int result;
   result = Bub::Move();
   
   // praskla-li bublina, vratime energii
   if (result & BUB_BLOWEND && whose)
   {
      if (whose->sheet.energy<whose->sheet.maxenergy)
      {
         whose->sheet.energy++;
         whose->sheet.energy_changed = 2;
      }
   }

   // skrabani zevnitr na bublinu
   if (monster)
   {
      if (monster->updateable)
      {
         monster->BubbleAction(newx,newy);
      }
      
      if (counter < nextscrape && monster->type == PLAYER)
      {
         monster->KeyReflex();
         Player * pmon = (Player *)monster;
         if (pmon->state & WANNALEAVE)
         {
            pmon->state ^= WANNALEAVE;
            pmon->frames.setChain(CHAIN_SCRAPE);
            if (counter > BUB_LASTSCRAPE)
            {
               sender = pmon;
               counter -= BUB_SCRAPEEFFECT;
               if (counter > BUB_SCRAPELENGTH)
                  //nextscrape = counter-(BUB_SCRAPELENGTH+3-((pmon->acceleration/3)); //pokud je hbitejsi, vrze rychleji
                  nextscrape = counter-BUB_SCRAPELENGTH;
               else
                  nextscrape = 0;
               sound_device->Play(8);
            }
            else
            {
               counter = BUB_BLOWTIME+1;
               nextscrape = 0;
            }
         }
      }
   }
   if ((counter == BUB_BLOWCOME1 || counter == BUB_BLOWCOME2) 
       && monster && monster->type == MONSTER)
   {
      sound_device->Play(8,newx+xsize/2);
      monster->frames.setChain(CHAIN_SCRAPE);
   }

   return result;
}

//===========================================================================
void Bubble::jumpNext()
{
   // animace potwurky
   if (monster)
      monster->jumpNext();
   // a vlastni bubliny
   frames.jumpNext();
   if (id != ID_BUBBLE)
   {
      frameLeft = (frames.frameid % celr) * celw;
      frameTop = (frames.frameid / celr) * celh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + xsize;  
      frameRect.bottom = frameTop + ysize;  
      if (monster != NULL)
      {
         monster->frameRect.left = monster->frameLeft;
         monster->frameRect.top = monster->frameTop;
         monster->frameRect.right = monster->frameLeft + xsize;  
         monster->frameRect.bottom = monster->frameTop + ysize;  
      }
   }
}

//===========================================================================
void Bubble::Paint(Surface dest)
{
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
   if (id != ID_BUBBLE)
   {
      if (monster != NULL)
      {
         putcel(newx,newy,dest,monster->frameRect,monster->bubgfx);
      }
      putcel(newx,newy,dest,frameRect,gfx);
      oldx = newx;
      oldy = newy;
      SET_USEDMAP(newx,newy);
      SET_USEDMAP(xright,newy);
      SET_USEDMAP(newx,ybottom);
      SET_USEDMAP(xright,ybottom);
      return;
   }

   Surface mask;
   RECT r,valid = {0,0,0,0};
   int x,y,xx,yy,xx2,yy2;
   RGlass *glass;
   DDSURFACEDESC2 ddsd2;
   Surface realdest;

   glass = glasses[frames.frameid];

   if (bubbuf != NULL)
   {
      if (bubbuf->IsLost())
      {
         bubbuf->Release();
         bubbuf = NULL;
      }
      if (computed && newx == oldx && newy == oldy)
      {
         r.right = glass->GetWidth();
         r.bottom = glass->GetHeight();
         r.left = 0;
         r.top = 0;

         putcel(newx,newy,dest,r,bubbuf);
         computed = false;
         SET_USEDMAP(newx,newy);
         SET_USEDMAP(xright,newy);
         SET_USEDMAP(newx,ybottom);
         SET_USEDMAP(xright,ybottom);
         return;
      }
   }

   computed = true;
   if (bubbuf == NULL)
   {
      // vytvorime surface pro vysledek
      ZeroMemory(&ddsd2,sizeof(ddsd2));
      ddsd2.dwSize = sizeof(ddsd2);
      ddsd2.dwFlags = DDSD_WIDTH | DDSD_HEIGHT | DDSD_CAPS;
      ddsd2.dwWidth = glass->GetWidth();
      ddsd2.dwHeight = glass->GetHeight();
      ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
      ddResult = lpDD4->CreateSurface(&ddsd2,&bubbuf,NULL);
      if (ddResult != DD_OK)
      {
         throw new DDException("Bubble::Paint: CreateSurface() failed!",
         ddResult);
      }
      setColorKey(bubbuf,colorKey);
   }
   
   // urcime dulezite rozmery
   xx = newx - glass->GetXPitch();
   xx2 = newx + glass->GetWidth() + glass->GetXPitch();
   yy = newy - glass->GetYPitch();
   yy2 = newy + glass->GetHeight() + glass->GetYPitch();
   
   if (xx < PLAYFIELD_MIN_X)
   {
      valid.left = PLAYFIELD_MIN_X - xx;
      valid.right = xx2 - xx;
   }
   else if (xx2 > PLAYFIELD_MAX_X)
   {
      valid.right = PLAYFIELD_MAX_X - xx;
   }
   
   if (yy < PLAYFIELD_MIN_Y)
   {
      valid.top = PLAYFIELD_MIN_Y - yy;
      valid.bottom = yy2 - yy;
   }
   if (yy2 > PLAYFIELD_MAX_Y)
   {
      valid.bottom = PLAYFIELD_MAX_Y - yy;
   }
   
   if (valid.right ^ valid.bottom)
   {
      if (!valid.right)
         valid.right = xx2 - xx;
      if (!valid.bottom)
         valid.bottom = yy2 - yy;
   }

   x = valid.left;
   y = valid.top;

   r.left = xx + x;
   r.top  = yy + y;
   r.right = min(xx2,PLAYFIELD_MAX_X);
   r.bottom = min(yy2,PLAYFIELD_MAX_Y);
  
   if (monsterbehind)
      realdest = dest;
   else
      realdest = m_background;

   putcel(x,y,RGlass::bckg_buf,r,realdest,DDBLTFAST_NOCOLORKEY);
    
   mask = NULL;
   if (monster)
   {
      x = monster->frameLeft;
      y = monster->frameTop;
      mask = monster->bubgfx;
   }
   doBubble(mask,x,y,RGlass::bckg_buf,valid,tnts[tint],glass,bubbuf);

   r.right = glass->GetWidth();
   r.bottom = glass->GetHeight();
   r.left = 0;
   r.top = 0;

   putcel(newx,newy,dest,r,bubbuf);

   SET_USEDMAP(newx,newy);
   SET_USEDMAP(xright,newy);
   SET_USEDMAP(newx,ybottom);
   SET_USEDMAP(xright,ybottom);
   
   oldx = newx;
   oldy = newy;
}

//-------------------------------------------------------------------------
// akce pri chyceni cehosi do bubliny
//-------------------------------------------------------------------------
void Bubble::Catch(Creature *monster)
{
   mon_chain = monster->frames.chainid;
   mon_dir = monster->direction;
   this->monster = monster;
   if (whose == player1)
   {
      monster->direction = DIR_L;
      monster->frames.setDirection(DIR_L);
   }
   else
   {
      monster->direction = DIR_R;
      monster->frames.setDirection(DIR_R);
   }
   fTgs = 60;        // proc tu byla 100?
   monster->frames.setChain(CHAIN_BUBBLE);
   frames.setChain(CHAIN_STAND);
   jumpNext();
}

//===========================================================================
int Bubble::Blow(Player *thissender)
{
   int result;
   int bc;
   byte bi;
   Bonus *pom_bo;
   
   //kdo sezere body?
   if (thissender!=NULL)
   {
      sender=thissender;
   }
   
   result = 0;
   if (monster != NULL)
   {
      sound_device->Play(39,newx+20);
      if (sender && monster->type == MONSTER)
      {
         bc = random(((Monster *)monster)->bonus_cr)+1;
         if(bc) for(int k=0;k<bc;k++)
         {
            //V: po uprave bude 'bi' kandidat na zruseni!
            bi = 40 + (BYTE)(random(7));
            pom_bo = new Bonus(newx,newy,properties.member[bi]);
            if (ARRAY_FULL(bonuses_ctrl))
            {
               delete pom_bo;
               app->sayConsole("Array (bonuses) is full - deleted!");
            }
            else
            {
               ARRAY_ADD(bonuses,bonuses_ctrl,pom_bo);
            }
         }
         int cp = ((Monster *)monster)->catch_points;
         int kp = ((Monster *)monster)->kill_points;
         whose->sheet.AddPoints(cp);
         sender->sheet.AddPoints(kp);
         delete (monster);
         monstercnt--;
         monster = NULL;
         result = 1;
      }
      else
      {
         monster->newx = monster->oldx = newx;
         monster->newy = monster->oldy = newy;
         monster->mul_x = mul_x;
         monster->mul_y = mul_y;
         monster->direction = mon_dir;
         monster->frames.setDirection(mon_dir);
         if (mon_chain != CHAIN_FIRE)
            monster->frames.setChain(mon_chain);
         else
            monster->frames.setChain(CHAIN_STAND);
         if (monster->type == PLAYER)
            ((Player *)monster)->state ^= INBUBBLE;
         monster->jumpNext();
         monster->Reincarnate();
         if (ARRAY_FULL(monsters_ctrl))
         {
            delete monster;
            app->sayConsole("Array (monsters) is full - deleted!");
         }
         else
         {
            ARRAY_ADD(monsters,monsters_ctrl,monster);
         }
         monster = NULL;
      }
   }
   else
   {
      sound_device->Play(4,newx+20);
   }

   counter = BUB_BLOWTIME;
   frames.setChain(CHAIN_BLOW);
   fTgs = 15;
   state |= BLOWING;
   if (s_drops) drops->Blast(newx+10,newy+10,dropcolor,gravitation);
   //dominovy efekt
   Domino();

   return result;
}

//##########################################################################
//------------------------------ SymbolBubble ------------------------------
//##########################################################################
//-------------------------------------------------------------------------
// vykresleni bubliny
//-------------------------------------------------------------------------
void SymbolBubble::Paint(Surface dest)
{
   Surface mask = NULL;
   int ybottom = newy + ysize - 1, xright = newx + xsize - 1;
   if (id != ID_SBUBBLE)
   {
      RECT r;
      if (this->mask == NULL)
      {
         if (frames.chainid == CHAIN_STAND)
         {
            mask = sym_bubgfx;
            r.left = symbol*40;
         }
      }
      else
      {
         mask = this->mask;
         r.left = 0;
      }
      if (mask != NULL)
      {
         r.right = r.left+40;
         r.top = 0;
         r.bottom = 40;
         putcel(newx,newy,dest,r,mask);
      }
      putcel(newx,newy,dest,frameRect,gfx);
      oldx = newx;
      oldy = newy;
      SET_USEDMAP(newx,newy);
      SET_USEDMAP(xright,newy);
      SET_USEDMAP(newx,ybottom);
      SET_USEDMAP(xright,ybottom);
      return;
   }

   RECT r,valid = {0,0,0,0};
   int x,y,xx,yy,xx2,yy2;
   RGlass *glass;
   DWORD gw,gh,gxp,gyp;
   DDSURFACEDESC2 ddsd2;
   Surface realdest;

   glass = glasses[frames.frameid];
   gw = glass->GetWidth();
   gh = glass->GetHeight();
   gxp = glass->GetXPitch();
   gyp = glass->GetYPitch();

   if (bubbuf != NULL)
   {
      if (bubbuf->IsLost())
      {
         bubbuf->Release();
         bubbuf = NULL;
      }
      if (computed && newx == oldx && newy == oldy)
      {
         r.right = gw;
         r.bottom = gh;
         r.left = 0;
         r.top = 0;
         
         putcel(newx,newy,dest,r,bubbuf);
         computed = false;
         SET_USEDMAP(newx,newy);
         SET_USEDMAP(xright,newy);
         SET_USEDMAP(newx,ybottom);
         SET_USEDMAP(xright,ybottom);
         return;
      }
   }

   computed = true;
   if (bubbuf == NULL)
   {
      // vytvorime surface pro vysledek
      ZeroMemory(&ddsd2,sizeof(ddsd2));
      ddsd2.dwSize = sizeof(ddsd2);
      ddsd2.dwFlags = DDSD_WIDTH | DDSD_HEIGHT | DDSD_CAPS;
      ddsd2.dwWidth = gw;
      ddsd2.dwHeight = gh;
      ddsd2.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
      ddResult = lpDD4->CreateSurface(&ddsd2,&bubbuf,NULL);
      if (ddResult != DD_OK)
      {
         throw new DDException("Bubble::Paint: CreateSurface() failed!",
         ddResult);
      }
      setColorKey(bubbuf,colorKey);
   }
   
   // urcime dulezite rozmery
   xx = newx - gxp;
   xx2 = newx + gw + gxp;
   yy = newy - gyp;
   yy2 = newy + gh + gyp;
   
   if (xx < PLAYFIELD_MIN_X)
   {
      valid.left = PLAYFIELD_MIN_X - xx;
      valid.right = xx2 - xx;
   }
   else if (xx2 > PLAYFIELD_MAX_X)
   {
      valid.right = PLAYFIELD_MAX_X - xx;
   }
   
   if (yy < PLAYFIELD_MIN_Y)
   {
      valid.top = PLAYFIELD_MIN_Y - yy;
      valid.bottom = yy2 - yy;
   }
   if (yy2 > PLAYFIELD_MAX_Y)
   {
      valid.bottom = PLAYFIELD_MAX_Y - yy;
   }
   
   if (valid.right ^ valid.bottom)
   {
      if (!valid.right)
         valid.right = xx2 - xx;
      if (!valid.bottom)
         valid.bottom = yy2 - yy;
   }

   x = valid.left;
   y = valid.top;

   r.left = xx + x;
   r.top  = yy + y;
   r.right = min(xx2,PLAYFIELD_MAX_X);
   r.bottom = min(yy2,PLAYFIELD_MAX_Y);

   if (monsterbehind)
      realdest = dest;
   else
      realdest = m_background;

   putcel(x,y,RGlass::bckg_buf,r,realdest,DDBLTFAST_NOCOLORKEY);
   
   if (this->mask != NULL)
   {
      mask = this->mask;
      x = y = 0;
   }
   else if (frames.chainid == CHAIN_STAND)
   {
      mask = sym_bubgfx;
      x = symbol * gw;
      y = 0;
   }
   else
   {
      mask = NULL;
      x = y= 0;
   }

   doBubble(mask,x,y,RGlass::bckg_buf,valid,tnts[TNT_GLASS],glass,bubbuf);

   r.right = gw;
   r.bottom = gh;
   r.left = 0;
   r.top = 0;

   putcel(newx,newy,dest,r,bubbuf);
   SET_USEDMAP(newx,newy);
   SET_USEDMAP(xright,newy);
   SET_USEDMAP(newx,ybottom);
   SET_USEDMAP(xright,ybottom);
   
   oldx = newx;
   oldy = newy;
}

//-------------------------------------------------------------------------
// posun frejmu ve framechainu
//-------------------------------------------------------------------------
void SymbolBubble::jumpNext()
{
   frames.jumpNext();
   if (id != ID_SBUBBLE)
   {
      frameLeft = (frames.frameid % celr) * celw;
      frameTop = (frames.frameid / celr) * celh;
      frameRect.left = frameLeft;
      frameRect.top = frameTop;
      frameRect.right = frameLeft + xsize;  
      frameRect.bottom = frameTop + ysize;  
   }
}

//===========================================================================
int SymbolBubble::Blow(Player *thissender)
{
   int result;
   //kdo sezere body? (pro domino)
   if (thissender!=NULL)
   {
      sender=thissender;
   }
   
   result = 0;
   sound_device->Play(39,newx+20);
   if (sender)
   {
      sender->symbuf.Add(symbol);
      sender->symbuf.Realize(sender);
   }
   counter = BUB_BLOWTIME;
   frames.setChain(CHAIN_BLOW);
   fTgs = 15;
   state |= BLOWING;
   if (s_drops) drops->Blast(newx+10,newy+10,DROP_WHITE,gravitation);
   //dominovy efekt
   Domino();
   
   return result;
}
