/*
 *--------------------------------------------------------------------
 *
 * Initializer
 *
 * Usek hry - inicializace ostatnich useku a nahravani globalnich dat
 *
 * verze   1.0.1 - 14.04.2000
 * autori  Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny:
 * 1.0.1 - snaha o prvni plnou verzi
 * 0.99.5 - vypis cisla verze posunut o 10pix nahoru
 * 0.99.4 - aviPlayer
 * 0.99.1 - demoverze
 *        - vypisovani cisla verze
 * 0.11.1 - zmena vykreslovani inicializeru (update:putcel,flip)
 *        - zvuk posunut kvuli menu kupredu
 *        - hlasitost se pri startu nacte a na konci nastavi
 * 0.9.5 - toppers se inicializuji, ale jenom logicky objekt (no gfx)
 * 0.9.4 - neinicializuji se toppers - zmena strategie
 * 0.8.1 - pridany zvuky
 * 0.7.2 - postupna inicializace podle initlevelu
 * 0.0.1 - prvni verze
 *--------------------------------------------------------------------
 */

#include <stdio.h>
#include <tchar.h>
#include "version.h"
#include "exceptions.h"
#include "utils.h"
#include "graphDD.h"
#include "surfaceManager.h"
#include "tint.h"
#include "font.h"
#include "topper.h"
#include "wsound.h"
#include "game.h"
#include "initializer.h"
#include "application.h"

const char *ok_msg = "Ok";
const char *fail_msg = "Failed";

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
Initializer::Initializer()
{
   initlevel = 0;
   vol = 0;
}

//-------------------------------------------------------------------------
// inicializuje veci potrebne pro spousteni initializeru
//-------------------------------------------------------------------------
void Initializer::load()
{
}

//-------------------------------------------------------------------------
// zahajuje cinnost initializeru
//-------------------------------------------------------------------------
void Initializer::start()
{
   gfx_load();
   if (gBits == 5)
      loadTint("data\\555.tnt",TNT_SIZE555);
   else
      loadTint("data\\565.tnt",TNT_SIZE565);
   surfaceManager->remove(background,true);
   background = surfaceManager->load("title.r16","data\\pictures.dir",true);
}

//-------------------------------------------------------------------------
// pravidelna cinnost pri kazdem pruchodu hlavni smyckou programu
//-------------------------------------------------------------------------
void Initializer::update()
{
   switch (initlevel)
   {
   case 0:
      sprintf(pom_string,"Bubliny %s",VERSION);
      putString(40,45,background,pom_string,qfont_r);
      putString(40,80,background, "Starting sound........",qfont_w);
      break;
   case 1:
      putString(40,100,background,"Loading fader.........",qfont_w);
      break;
   case 2:
      putString(40,120,background,"Loading menu..........",qfont_w);
      break;
   case 3:
      putString(40,140,background,"Loading interscreen...",qfont_w);
      break;
   case 4:
      putString(40,160,background,"Loading topper........",qfont_w);
      break;
   case 5:
      putString(40,180,background,"Loading game..........",qfont_w);
      break;
   case 6:
      break;
   }
   putcel(0,0,backbuffer,rFullScreen,background,DDBLTFAST_NOCOLORKEY);
   flip(0);
   switch (initlevel)
   {
   case 0:
      OutputDebugString("Sound1\n");
      sound_device = new WSound(app->getMainWindow(),NULL,
         "data\\BLASTER.NFX");
      OutputDebugString("SoundA\n");
      if (sound_device->werror != 0)         
         putString(305,80,background,fail_msg,qfont_r);
      else
      {
         sound_device->GetVolume(&vol);
         if (s_vol == 255)
         {
            if (vol < -2000)
               s_vol = 0;
            else
               s_vol = BYTE((vol+2000)/200);
            if (s_vol > 10)
               s_vol = 10;
         }
         else
         {
            if (s_vol == 0)
               sound_device->SetVolume(-10000);
            else
               sound_device->SetVolume(((int)s_vol)*200-2000);
         }
         OutputDebugString("Sound2\n");
         putString(305,80,background,ok_msg,qfont_y);
      }
      break;
   case 1:
      app->fader->load();
      app->aviPlayer->load();
      putString(305,100,background,ok_msg,qfont_y);
      break;
   case 2:
      app->menu->load();
      putString(305,120,background,ok_msg,qfont_y);
      break;
   case 3:
      app->interscreen->load();
      putString(305,140,background,ok_msg,qfont_y);
      break;
   case 4:
      toppers = new Toppers();
      if (toppers->Load())
         toppers->New();
      putString(305,160,background,ok_msg,qfont_y);
      break;
   case 5:
      app->game->load();
      putString(305,180,background,ok_msg,qfont_y);
      break;
   case 6:
      app->stateChange();
      break;
   }

   initlevel++;
}

//-------------------------------------------------------------------------
// obnoveni po surface lost
//-------------------------------------------------------------------------
void Initializer::restore()
{
}

//-------------------------------------------------------------------------
// cinnost pri zastaveni zobrazovani objektu
//-------------------------------------------------------------------------
void Initializer::stop()
{
}

//-------------------------------------------------------------------------
// cinnost pri prechodu na jiny aplikacni objekt 
//-------------------------------------------------------------------------
void Initializer::unload()
{
   if (toppers != NULL)
   {
      toppers->Save();
      delete toppers;
      toppers = NULL;
   }
   doneTint();
   if (sound_device != NULL && !sound_device->werror)
      sound_device->SetVolume(vol);
   SAFE_DELETE(sound_device);
}

//-------------------------------------------------------------------------
// destruktor
//-------------------------------------------------------------------------
Initializer::~Initializer()
{
   unload();
}