/*--------------------------------------------------------------------
 *
 * InterScreen
 *
 * Usek hry - mezistav - popis nasledujiciho levelu
 *
 * verze   1.1.2 - 08.11.2004
 * autori  Vojvo & G.
 *
 *--------------------------------------------------------------------
 * zmeny
 *
   1.1.2 / oprava rozhozene grafiky po pepnut tasku (print())
 * 1.0.1 - snaha o prvni plnou verzi
 * 0.10.2 - stav klavesnice se cte primo v update
 * 0.10.1 / nikdy nemeni heslo na '/'
 * 0.9.3 - prvni funkcnost interscreenu, zatim testovaci
 * 0.7.1 - prvni verze
 *--------------------------------------------------------------------
 */

#include "exceptions.h"
#include "keyboard.h"
#include "graphdd.h"
#include "surfaceManager.h"
#include "font.h"
#include "pass.h"
#include "application.h"
#include "game.h"
#include "interscreen.h"

//-------------------------------------------------------------------------
// inicializuje veci potrebne pro spousteni
//-------------------------------------------------------------------------
void InterScreen::load()
{
}


//-------------------------------------------------------------------------
// koncentrovany vypis textu
//-------------------------------------------------------------------------
void InterScreen::print()
{
   int i;
   RECT rLifeSymbol = {0,260,20,280};

	// zivoty
   for(i=0; i<player1->sheet.lives; i++)
      putcel(320-player1->sheet.lives*12+i*24+2,160,background,rLifeSymbol,tiles);

	// level name
   putStringCentered(320,90,background,levelname,sfont_w);

	// autor + pokec
   putStringCentered(320,125,background,s_leveldesc[0],qfont_w);
   putStringCentered(320,200,background,s_leveldesc[1],qfont_g);
   for(i=2; i<MAX_TEXTS && s_leveldesc[i][0] != '\x0'; i++)
      putString(20,210+i*25,background,s_leveldesc[i],qfont_w);
}

//-------------------------------------------------------------------------
// zahajuje cinnost
//-------------------------------------------------------------------------
void InterScreen::start()
{
   DDSURFACEDESC2 ddsd2;
   long tempPitch, previewPitch;
   WORD *tempData, *previewData;
   DWORD tempOffset, previewOffset;
   int i,j;

   //--- vytvoreni surfaces ---
   preview = surfaceManager->create(320,200,"*preview",
      DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY);
   temp = surfaceManager->create(640,480,"*temp",
      DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY);
   putcel(0,0,temp,rFullScreen,background,DDBLTFAST_NOCOLORKEY);

   //--- vykresleni objektu ---
   // teleporty
   I while(i<teleports_ctrl.count)
      ARRAY_AT(teleports,i++)->Paint(temp);
   // bonusy
   I while(i<bonuses_ctrl.count)
      ARRAY_AT(bonuses,i++)->Paint(temp);
   // potfurky
   I while(i<monsters_ctrl.count)
      ARRAY_AT(monsters,i++)->Paint(temp);

   //--- vytvoreni preview ---
   lock(temp,NULL,&ddsd2);
   tempPitch = ddsd2.lPitch/2;
   tempData = (WORD *)ddsd2.lpSurface;

   lock(preview,NULL,&ddsd2);
   previewPitch = ddsd2.lPitch/2;
   previewData = (WORD *)ddsd2.lpSurface;

   for (j=40; j<440; j++)
   {
      tempOffset = j*tempPitch;
      if (!(j&1))
         previewOffset = ((j-40)>>1)*previewPitch;
      for  (i=0; i<640; i++, tempOffset++)
         if (i&j&1)
            previewData[previewOffset++] = tempData[tempOffset];
   }
   
   preview->Unlock(NULL);
   temp->Unlock(NULL);

	// vytahnu si zalohu pozadi bez sprajtu - jinak by tam zstvali viset
	putcel(0,0,temp,rFullScreen,background,DDBLTFAST_NOCOLORKEY);

   // nacteni pozadi
   surfaceManager->remove(background,true);
   background = surfaceManager->load("interscreen.r16","data\\pictures.dir",true);

   // priprava vypisu hesla
	if (enteredpass)
   {
      strcpy(levelname,"Heslo: ");
      strcat(levelname,s_password);
      enteredpass=false;
   }
   else
   {
      char passwd[12];
      encode(passwd,levelname,&player1->sheet.lives);
      strcpy(levelname,"Heslo: ");
      strcat(levelname,passwd);
      //tady muzu zmenit i heslo menu
      if ((s_changepass || player1->sheet.lives>=DEF_LIVES) && *s_password!='/')
      {
         strcpy(s_password,passwd);
      }
   }

	// vsechno vypisu
	print();

	// flipy a spol.
   putcel(0,0,backbuffer,rFullScreen,background,DDBLTFAST_NOCOLORKEY);
   flip(0);
   putcel(0,0,backbuffer,rFullScreen,background,DDBLTFAST_NOCOLORKEY);
   kbd.lastScanReset();
   kbd.lastASCIIReset();
   kbd.filter_lastscan = false;
}

//-------------------------------------------------------------------------
// pravidelna cinnost pri kazdem pruchodu hlavni smyckou programu
//-------------------------------------------------------------------------
void InterScreen::update()
{
   RECT r = {0,0,320,200};
   BYTE k;

   flip(0);
   putcel(0,0,backbuffer,rFullScreen,background,DDBLTFAST_NOCOLORKEY);
   putcel(290,250,backbuffer,r,preview,DDBLTFAST_NOCOLORKEY);

   // klavesnice
   kbd.getState();

   //------------------------------------------------------------ screenshot
   if (KEYDOWN(kbd.diKeys,DIK_F9))
   {
      kbd.clearKey(DIK_F9);
	   kbd.lastScanReset();
      filnam[5]=(picnum / 100)+'0';
      filnam[6]=(picnum % 100)/10+'0';
      filnam[7]=(picnum % 10)+'0';
      screenshot(backbuffer,filnam);
      picnum++;
   }
	//--- odchod z interscreenu 
   if ((k = kbd.LastScan())!= 0)
   {
      if (k == DIK_LMENU || k == DIK_RMENU || k == DIK_TAB)
         return;
      kbd.clearKey(k);
		// vytahnu si tu odlozenou zalohu pozadi
		// BTW background je v tuto chvili rozhodne nastaveny na _normal
      putcel(0,0,background,rFullScreen,temp,DDBLTFAST_NOCOLORKEY);

      surfaceManager->setName(background,backname);
      app->stateChange();
   }
}

//-------------------------------------------------------------------------
// obnoveni po surface lost
//-------------------------------------------------------------------------
void InterScreen::restore()
{
	// (samotn interscreen je obrzek a preview je staticky)
	// grafika u by mla bt OK, take u jen vykreslm texty
	print();
}

//-------------------------------------------------------------------------
// cinnost pri zastaveni zobrazovani objektu
//-------------------------------------------------------------------------
void InterScreen::stop()
{
   DEBUG_MESSAGE("InterScreen::stop()\n");
   kbd.filter_lastscan = true;
   kbd.lastScanReset();
   kbd.lastASCIIReset();
   surfaceManager->remove(temp);
   surfaceManager->remove(preview);
   DEBUG_MESSAGE("InterScreen::stop()\n");
}

//-------------------------------------------------------------------------
// cinnost pri definitivnim ukonceni pouzivani
//-------------------------------------------------------------------------
void InterScreen::unload()
{
}
