/*--------------------------------------------------------------------
 *
 * SlideShow
 *
 * Usek hry - zobrazovac serie obrazku
 *
 * verze   1.0.1 - 14.4.2000
 * autori  Vojvo
 *
 *--------------------------------------------------------------------
 * zmeny
 * 1.0.1  - snaha o prvni plnou verzi
 * 0.99.6 - prejmenovano TheEnd -> SlideShow a umozneno zadat parametry
 * 0.99.1 - demoverze
 *        - pridan zvuk pri prepinani
 *        - advert vyuziva fakeopen
 * 0.11.2 - prvni verze
 */

#include "dxconsts.h"
#include <stdio.h>
#include "exceptions.h"
#include "keyboard.h"
#include "graphdd.h"
#include "surfaceManager.h"
#include "fakedir.h"
#include "wsound.h"
#include "application.h"
#include "slideshow.h"

//-------------------------------------------------------------------------
// konstruktor
//-------------------------------------------------------------------------
SlideShow::SlideShow()
{
   shotnum = 1;
   shotlast = 999;
   shotformat = "%d.r16";
}

//-------------------------------------------------------------------------
// inicializuje veci potrebne pro spousteni
//-------------------------------------------------------------------------
void SlideShow::load()
{
}

//-------------------------------------------------------------------------
// zahajuje cinnost
//-------------------------------------------------------------------------
void SlideShow::start()
{
   DEBUG_MESSAGE("SlideShow::start()\n");
   FILE *ftest;
   char fname[20];
   
   nextnotfound = false;
   sprintf(fname,shotformat,shotnum);
   ftest = fakeopen("data\\pictures.dir",fname,"rb");
   if (ftest == NULL)
   {
      nextnotfound = true;
   }
   else
   {
      fclose(ftest);
      surfaceManager->remove(background,true);
      background = surfaceManager->load(fname+5,"data\\pictures.dir",true);
   }
   kbd.lastScanReset();
   kbd.lastASCIIReset();
   kbd.filter_lastscan = false;
   sound_device->Play(24);
   DEBUG_MESSAGE("SlideShow::start()\n");
}

//-------------------------------------------------------------------------
// pravidelna cinnost pri kazdem pruchodu hlavni smyckou programu
//-------------------------------------------------------------------------
void SlideShow::update()
{
   BYTE k;

   if (nextnotfound || shotnum > shotlast)
   {
      app->stateChange();
      return;
   }

   // klavesnice
   kbd.getState();

   if ((k=kbd.LastScan()) != 0)
   {
      kbd.clearKey(k);
  	   kbd.lastScanReset();
      FILE *ftest;
      char fname[20];
      shotnum++;
      if (shotnum > shotlast)
      {
         nextnotfound = true;
      }
      else
      {
         sprintf(fname,shotformat,shotnum);
         ftest = fakeopen("data\\pictures.dir",fname,"rb");
         if (ftest == NULL)
         {
            nextnotfound = true;
         }
         else
         {
            fclose(ftest);
            surfaceManager->remove(background);
            background = surfaceManager->load(fname+5,"data\\pictures.dir");
            sound_device->Play(24);
         }
      }
   }
   putcel(0,0,backbuffer,rFullScreen,background,DDBLTFAST_NOCOLORKEY);
   flip(0);
}

//-------------------------------------------------------------------------
// obnoveni po surface lost
//-------------------------------------------------------------------------
void SlideShow::restore()
{
   FILE *ftest;
   char fname[20];
   sprintf(fname,shotformat,shotnum);
   ftest = fakeopen("data\\pictures.dir",fname,"rb");
   if (ftest == NULL)
   {
      app->stateChange();
   }
   else
   {
      fclose(ftest);
      surfaceManager->remove(background);
      background = surfaceManager->load(fname+5,"data\\pictures.dir");
   }
}

//-------------------------------------------------------------------------
// cinnost pri zastaveni zobrazovani objektu
//-------------------------------------------------------------------------
void SlideShow::stop()
{
   DEBUG_MESSAGE("SlideShow::stop()\n");
   kbd.filter_lastscan = true;
   kbd.lastScanReset();
   kbd.lastASCIIReset();
   DEBUG_MESSAGE("SlideShow::stop()\n");
}

//-------------------------------------------------------------------------
// cinnost pri definitivnim ukonceni pouzivani
//-------------------------------------------------------------------------
void SlideShow::unload()
{
}
